#!/bin/sh

# TIME is set to seconds
# SITE is the location in which the keep alive will ping
# DIRECTORY checks to see if the WAN connection has been established via linux OS
# INTERFACE stores the name of the WAN interface used
TIME=$(/psft/bin/cfgModify read wan/keepalive)
SITE=$(/psft/bin/cfgModify read wan/keepaddr)
DIRECTORY="/sys/class/net"
INTERFACE=
LOOP=0

/usr/bin/logger -t "ICX35-KeepAlive" -p USER.NOTICE -s "Keep Alive Time (in sec): ""$TIME"
/usr/bin/logger -t "ICX35-KeepAlive" -p USER.NOTICE -s "Keep Alive Address: ""$SITE"

# If time is set to 0, keep-alive is off, exit keep-alive
if [ "$TIME" = "0" ]
    then
     LOOP=1
	 /usr/bin/logger -t "ICX35-KeepAlive" -p USER.NOTICE -s "Time is 0 - exiting"
	 exit 2
fi

# Argument input check for time
if [ -z "$TIME" ]
  then
    /usr/bin/logger -t "ICX35-KeepAlive" -p USER.NOTICE -s "No Time argument supplied"
    /usr/bin/logger -t "ICX35-KeepAlive" -p USER.NOTICE -s "Please input: Time of duration (in seconds) and web address"
    exit 1
fi

# Argument input check for site
if [ -z "$SITE" ]
  then
    /usr/bin/logger -t "ICX35-KeepAlive" -p USER.NOTICE -s "No Site argument supplied"
    /usr/bin/logger -t "ICX35-KeepAlive" -p USER.NOTICE -s "Please input: Time of duration (in seconds) and web address"
    exit 1
fi

# Check if a WAN directory exists and the interface has a link(ppp0/wwan0/wwan1 has been established)
# If not established, sleep for 10 seconds and try again
while [ -z "$INTERFACE" ]; do
	 /usr/bin/logger -t "ICX35-KeepAlive" -p USER.NOTICE -s "WAN not up - waiting for connection"
	 sleep 10
     if [ -d "$DIRECTORY/ppp0" ] && [ "$(cat $DIRECTORY/ppp0/operstate)" != "down" ]
        then
            INTERFACE="ppp0"
    elif [ -d "$DIRECTORY/wwan0" ] && [ "$(cat $DIRECTORY/wwan0/operstate)" != "down" ]
        then
            INTERFACE="wwan0"
    elif [ -d "$DIRECTORY/wwan1" ] && [ "$(cat $DIRECTORY/wwan1/operstate)" != "down" ]
        then
            INTERFACE="wwan1"
    fi
done

# Loop infinitely for a value of seconds greater than 0
# ping user input for website on the INTERFACE (WAN)
while [ $LOOP -eq 0 ]; do
    /psft/bin/icx20-ping.sh "$INTERFACE" "$SITE" | /usr/bin/logger -t "ICX35-KeepAlive" -p USER.NOTICE -s
    if [ $? -eq 1 ] && [ $INTERFACE == "ppp0" ]
      then
        /usr/bin/logger -t "ICX35-KeepAlive" -p USER.NOTICE -s "keepalive ping failed, killing pppd"
        pkill pppd
    fi
    sleep "$TIME"
done
