#!/bin/sh

#Get configuration
apn=$(/psft/bin/cfgModify read ppp/apn)
username=$(/psft/bin/cfgModify read ppp/apnuser)
password=$(/psft/bin/cfgModify read ppp/apnpass)
auth=$(/psft/bin/cfgModify read ppp/authentication)
device="/dev/cdc-wdm0"
modes=all
CELLCMD=/psft/bin/cellCmdRequest
ntpEnabled=$(/psft/bin/cfgModify read system/ntpenable)

# Map webUI auth type to pap|chap|both|none
case "$auth" in
0)auth=chap
;;
1)auth=pap
;;
2)auth=both
;;
*)auth=none
;;
esac

devpath="$(readlink -f /sys/class/usbmisc/cdc-wdm0/device)"
ifname="$( ls "$devpath"/net )"
[ -n "$ifname" ] || {
	/usr/bin/logger -t "ICX35-QMI" -s   "The network interface could not be found."
	return 1
	}

[ -n "$apn" ] || {
	/usr/bin/logger -t "ICX35-QMI" -s  "No APN specified"
	pkill pppwd;
	sleep 5;
	return 1;

}

#disable autoconnect
uqmi -s -d "/dev/cdc-wdm0" --stop-network 0xffffffff --autoconnect > /dev/null 2>&1
sleep 1
uqmi -s -d "/dev/cdc-wdm1" --stop-network 0xffffffff --autoconnect > /dev/null 2>&1
sleep 1

[ -n "$modes" ] && uqmi -s -d "$device" --set-network-modes "$modes"
sleep 1

uqmi -s -d "$device" --set-data-format 802.3
sleep 1

uqmi -s -d "$device" --wda-set-data-format 802.3
sleep 1

if uqmi -s -d "$device" --get-serving-system | grep '"not_registered"' > /dev/null;
then
	/usr/bin/logger -t "ICX35-QMI" -s   "Starting GSM network registration"
	uqmi -s -d "$device" --network-register
	sleep 2;
fi

/usr/bin/logger -t "ICX35-QMI" -s  "Waiting for network registration"
while uqmi -s -d "$device" --get-serving-system | grep '"searching"' > /dev/null; do
	sleep 5;
done

/usr/bin/logger -t "ICX35-QMI" -s  "Starting network $apn"
	cid_4=$(uqmi -s -d "$device" --get-client-id wds)
	[ $? -ne 0 ] && {
		/usr/bin/logger -t "ICX35-QMI" -s  "Unable to obtain client ID"
		echo "" > /tmp/qmi_cid
		return 1
	}
	echo "$cid_4" > /tmp/qmi_cid
	pdh_4=$(uqmi -s -d "$device" --set-client-id wds,"$cid_4" \
		--start-network "$apn" \
		${auth:+--auth-type $auth} \
		${username:+--username $username} \
		${password:+--password $password} )
	[ $? -ne 0 ] && {
		/usr/bin/logger -t "ICX35-QMI" -s  "Unable to connect IPv4. uqmi returns: $pdh_4"
		uqmi -s -d "$device" --set-client-id wds,"$cid_4" --release-client-id wds
		echo "" > /tmp/qmi_pdh
		return 1
	}
	echo "$pdh_4" > /tmp/qmi_pdh

#Aquire IP address
pkill udhcpc
#echo "" > /var/db/dhclient.leases
dhcpcret=$(udhcpc -i wwan0 -S -n)

[ $? -ne 0 ] && {
	/usr/bin/logger -t "ICX35-QMI" -s  "udhcpc returned $dhcpcret. Retry connection ..."
	return 1
}

# If NTP feature is disabled, 
# execute one shot NTP time set and update radio time as well
if  [ "$ntpEnabled" -eq 0 ];
then
	ntpdate -s time.nist.gov
	ICX_DATE=$(date +%y/%m/%d,%H:%M:%S)
	$CELLCMD at+cclk="\"$ICX_DATE\"" > /dev/null 2>&1
fi

/psft/bin/usageModify linktime

exit 0
