#!/bin/sh
#
# dhcpd config generator
#

export LD_LIBRARY_PATH=/psft/lib

# ICX35 system command wrapper functions
source /psft/scripts/icx35-syscom.sh

# dhcpd server config values
icx35ConfigRead dhcpServer lan/dhcpserver
icx35ConfigRead dhcpPoolLo lan/dhcppoollo
icx35ConfigRead dhcpPoolHi lan/dhcppoolhi

# Lease time params
icx35ConfigRead dhcpLeaseUnits lan/dhcpleaseunit
icx35ConfigRead dhcpLeaseTime lan/dhcpleasetime

# LAN port information
icx35ConfigRead lanIpAddress lan/ipadr
icx35GetLanBroadcast lanBroadcast
icx35GetLanNetmask lanNetmask
icx35GetLanNetwork lanNetwork


# need to adjust the lease time based on the time units
if [ "s" == "$dhcpLeaseUnits" ] ; then
    dhcpLeaseTime=$((dhcpLeaseTime*1))
elif [ "m" == "$dhcpLeaseUnits" ] ; then
    dhcpLeaseTime=$((dhcpLeaseTime*60))
elif [ "h" == "$dhcpLeaseUnits" ] ; then
    dhcpLeaseTime=$((dhcpLeaseTime*3600))
fi

printf "Generating /psft/etc/dhcpd.conf config file... "
cat > /psft/etc/dhcpd.conf << _EOF
# dhcpd.conf
#
# Configuration file for DHCP server
#

# Prevent clients from sending DNS information.
ddns-update-style none;

# Indicates the DHCP server to send DHCPNAK messages to
# misconfigured clients.
authoritative;

# Sets the length in seconds that will be assigned to a lease
# if the client requesting the lease does not ask for a
# specific  expiration time.
default-lease-time $dhcpLeaseTime;

# Sets the maximum length in seconds that will be assigned
# to a lease.
max-lease-time $dhcpLeaseTime;

subnet $lanNetwork netmask $lanNetmask {
	range $dhcpPoolLo $dhcpPoolHi;
	option subnet-mask $lanNetmask;
	option broadcast-address $lanBroadcast;
	option routers $lanIpAddress;
	option domain-name-servers $lanIpAddress;
}

_EOF

# Static leases params
icx35ConfigRead dhcpLeaseNum lan/dhcpleasenum

for it in $(seq 0 "$((dhcpLeaseNum -1))"); do
	icx35ConfigRead dhcpLeaseEntry lan/dhcplease"$it"

	cat >> /psft/etc/dhcpd.conf <<- _EOF
		host $(echo "$dhcpLeaseEntry" | cut -d ";" -f 1) {
		    hardware ethernet $(echo "$dhcpLeaseEntry" | cut -d ";" -f 2);
		    fixed-address $(echo "$dhcpLeaseEntry" | cut -d ";" -f 3);
		}

	_EOF
done

# Flush the dhcpd.leases file
echo "" > /var/db/dhcpd.leases
