#!/bin/sh
#
# ntpd configurator
# /tmp/ntp.conf, configuration for ntpd; see ntp.conf(5) for help
#

# ICX35 system command wrapper functions
source /psft/scripts/icx35-syscom.sh

# Paths to files used by ntpd daemon
pathToDrift="/etc/ntp/ntp.drift"
pathToConfig="/tmp/ntp.conf"
pathToLog="/tmp/messages"
pathToStats="/tmp/ntpstats/"

#
# Generate ntpd.conf file
# $1 - debug/release mode
#
ntpdGenerateConfigFile () {
	icx35ConfigRead ntpMode system/ntpmode
	icx35ConfigRead ntpServer system/ntpserver

	# Generate config file
	cat > "$pathToConfig" << _EOF
# ntp.conf
#
# Configuration file for ntpd daemon
#

# Disable threshhold panic
tinker panic 0
 
# Enable logging
driftfile $pathToDrift
logfile $pathToLog

# NTP server - user defined
server $ntpServer iburst

# Block access to NTP daemon
restrict default ignore
restrict 127.0.0.1
restrict $ntpServer nomodify notrap nopeer noquery

_EOF

	if [ "1" == "$ntpMode" ] ; then
		icx35GetLanNetwork lanNetwork
		icx35GetLanNetmask lanNetmask

		cat  >> "$pathToConfig" << _EOF
# NTP server mode - allow LAN queries
restrict $lanNetwork mask $lanNetmask nomodify notrap nopeer

_EOF
	fi


	if [ "debug" == "$1" ] ; then
		cat  >> "$pathToConfig" << _EOF
# Debug statistics
statsdir $pathToStats
statistics loopstats peerstats clockstats
filegen loopstats file loopstats type day enable
filegen peerstats file peerstats type day enable
filegen clockstats file clockstats type day enable

_EOF
	fi
}

#
# Launch ntpd daemon
# $1 
#
ntpdStartDaemon () {
	# Query NTP enablement status
	icx35ConfigRead ntpEnable system/ntpenable

	if [ "1" == "$ntpEnable" ] ; then
		# run ntpd daemon
		/bin/ntpd -c /tmp/ntp.conf &
    fi
}

if [ "0" == "$#" ] ; then
    /usr/bin/logger -t "ntpd" -p USER.NOTICE -s "Starting process: $(date)"
	# Generate configuration file
	ntpdGenerateConfigFile
	# Launch ntpd daemon
	ntpdStartDaemon
elif [ "1" == "$#" ] && [ "debug" == "$1" ] ; then
	/usr/bin/logger -t "ntpd" -p USER.NOTICE -s "Starting debug process: $(date)"
	# Generate configuration file
	ntpdGenerateConfigFile "$1"
	# Launch ntpd daemon
	ntpdStartDaemon
fi
