#!/bin/sh
#
# system-command.sh
# Wrapper for system command that are dependent to hsagred libraries
#

export LD_LIBRARY_PATH=/lib:/usr/lib:/psft/lib:/opt/lib
export XTABLES_LIBDIR=/lib/xtables

#
# Read param from config file
# $1 - return variable
# $2 - parameter to be read from config file
#
icx35ConfigRead () {
	local __resultvar=$1
	local __readCommand="$(/psft/bin/cfgModify read "$2")"
	eval "$__resultvar"="'$__readCommand'"
}

#
# Query connection state
# $1 - return variable
# $2 - connection tag to be read
#
icx35ConnectionState () {
	local __resultvar=$1
	local __readCommnad="$(/psft/bin/sharedStatModify --"$2" get)"
	eval "$__resultvar"="'$__readCommand'"
}

#
# Verify if interface is active
# $1 - return variable
# $2 - interface name
#
icx35InterfaceInfo () {
	local __resultvar=$1
	local __readCommand="$(ifconfig | grep "$2" | cut -d' ' -f1)"
	if test "$__readCommand" == "$2" ; then
		eval "$__resultvar"="1"
	else
		eval "$__resultvar"="0"
	fi
}

#
# Load iptables rules for the current connection
#
icx35LoadFirewall () {
	/psft/bin/netfilterd start
}

#
# Return lan network subnet
# $1 - return variable
#
icx35GetLanNetwork () {
	local __resultvar=$1
	icx35ConfigRead lanIpAddr lan/ipadr
	icx35ConfigRead lanipCIDRmask lan/ipCIDRmask
	local __readCommand="$(/bin/busybox ipcalc -n "$lanIpAddr"/"$lanipCIDRmask" \
	| cut -d'=' -f2)"
	eval "$__resultvar"="'$__readCommand'"
}

#
# Return lan network broadcast
# $1 - return variable
#
icx35GetLanBroadcast () {
	local __resultvar=$1
	icx35ConfigRead lanIpAddr lan/ipadr
	icx35ConfigRead lanipCIDRmask lan/ipCIDRmask
	local __readCommand="$(/bin/busybox ipcalc -b "$lanIpAddr"/"$lanipCIDRmask"\
	| cut -d'=' -f2)"
	eval "$__resultvar"="'$__readCommand'"
}

#
# Return lan network broadcast
# $1 - return variable
#
icx35GetLanNetmask () {
	local __resultvar=$1
	icx35ConfigRead lanIpAddr lan/ipadr
	icx35ConfigRead lanipCIDRmask lan/ipCIDRmask
	local __readCommand="$(/bin/busybox ipcalc -m "$lanIpAddr"/"$lanipCIDRmask"\
	| cut -d'=' -f2)"
	eval "$__resultvar"="'$__readCommand'"
}

#
# ARPING address $3 through interface $2
# $1 - return variable
# $2 - interface to use for ping
# $3 - address to ping
#
icx35ARPPing () {
	local __resultvar=$1
	/usr/bin/arping -D -I "$2" -c 3 "$3"
	eval "$__resultvar"="$?"
}

#
# Generate crond task file
# $1 - return variable
#
icx35GenerateCronFile () {
	local __resultvar=$1
	local __workingDir="/tmp/"
	local __manifestFile="/psft/etc/cron.manifest"
	local __writeCommand
	local __deleteCommand

	# Delete old cron file
	__deleteCommand="$(crontab -c "$__workingDir" -u $(whoami) -r)"

	# Read line by line from manifest file and execute crontab
	while IFS='' read -r line || [[ -n "$line" ]]; do
		__readCommand="$({ crontab -c "$__workingDir" -l -u "$(whoami)" 2> \
		/dev/null; echo "$line"; } | crontab -c "$__workingDir" -u "$(whoami)" -)"

		eval "$__resultvar"="'$__readCommand'"
	done < "$__manifestFile"
}
