<?php
    session_start();
    require_once('../inc/userfunction.php');
    require_once('../inc/config.inc.php');
    require_once('../inc/systemfunction.php');

    // Check the login session
    if (CheckUserSession() == false) {
        header("Location: /components/session_expired.php");
        exit;
    }

    $method = $_SERVER['REQUEST_METHOD'];
    $csrf = $_POST['csrf'];

    // Check the CSRF token
    if (CheckCSRFToken($method, $csrf) == false) {
    	error_log("Warning! Possible CSRF detected.", 0);
    	exit;
    }
	
    $validate_err = 0;
	$action = $_REQUEST['action'];

    if ($action == 1) {
        $ValidIpv4AddressRegex = "/^(([01]?[0-9]?[0-9]|2([0-4][0-9]|5[0-5]))\.){3}([01]?[0-9]?[0-9]|2([0-4][0-9]|5[0-5]))$/";
        $ValidChars = "/^[a-zA-Z0-9.\~\`\!\@\#\$\%\^\*\(\)\-\+\=\{\}\:\?\_]+$/i";

        /* User input field sterilization */
        $txt_webport=  htmlspecialchars_decode(trim($_POST['txt_webport']));
        $txt_newusername= htmlspecialchars(trim($_POST['txt_newusername']));
        $txt_newuserpass= htmlspecialchars(trim($_POST['txt_newuserpass']));
        $txt_newuserpass2 = htmlspecialchars(trim($_POST['txt_newuserpass2']));
        $hid_whitewanlen = htmlspecialchars_decode(trim($_POST['hid_whitewanlen']));

        /* Menu select options */
        $sel_webprotocol=$_POST['sel_webprotocol'];
        $sel_webaccessWAN=$_POST['sel_webaccessWAN'];

        /* get ports that must be unique and not configured on this page */
        $PortParams = "ovpn/port ssh/port discserv/port dns/port ftp/controlport " .
                      "ipsec/IKEport ipsec/L2TPport";
        exec("/psft/bin/cfgModify readlist " . $PortParams, $PortData);
        $PortData[7] = 502;
        $PortData[8] = 2000;
        $PortData[9] = 2001;

        /* Validate parameters */
        $tempErrFlag=false;
        for ($i=0; $i < count($PortData); $i++) {
            if ($txt_webport == $PortData[$i]) {
                $tempErrFlag=true;
                break;
            }
        }

        if ($tempErrFlag) {
            $errMsgs[$validate_err] = "Web Port is identical to a reserved or inuse port, please choose another port.";
            $errArray[$validate_err++] = "txt_webport";
        } else if ( !ctype_digit($txt_webport) || (strlen($txt_webport) == 0) ||
                    $txt_webport < 0 || $txt_webport > 65535 || $tempErrFlag) {
            $errMsgs[$validate_err] = "Web Port must be a number between 0 and 65535.";
            $errArray[$validate_err++] = "txt_webport";
        } else if ($txt_webport == 443) {
            $errMsgs[$validate_err] = "ProSoft Connect currently uses port 443 to tunnel.  Selecting port 443 will prevent Connect from functioning properly.  HTTPS can function properly using port 8080 or other ports.";
            $errArray[$validate_err++] = "txt_webport";
        }

        if (strlen($txt_newusername) > 40 || strlen($txt_newusername) == 0) {
            $errMsgs[$validate_err] = "User may not be blank, or exceed 40 character limit.";
            $errArray[$validate_err++] = "txt_newusername";
        } else if (!preg_match($ValidChars, $txt_newusername)) {
            $errMsgs[$validate_err] = "User uses invalid characters.";
            $errArray[$validate_err++] = "txt_newusername";
        }
        
        $len_pass = strlen($txt_newuserpass);
        $len_pass2 = strlen($txt_newuserpass2);

        if ($len_pass != 0 || $len_pass2 != 0) {
            if (!preg_match($ValidChars, $txt_newuserpass)) {
                $errMsgs[$validate_err] = "New Password contains invalid characters.";
                $errArray[$validate_err++] = "txt_newuserpass";
            } else if (($len_pass > 40) || ($len_pass2 > 40) || ($len_pass < 6) || ($len_pass2 < 6)) {
                $errMsgs[$validate_err] = "Password fields must have between 6-40 characters.";
                $errArray[$validate_err++] = "txt_newuserpass";
                $errArray[$validate_err++] = "txt_newuserpass2";
            } else if (strncmp($txt_newuserpass, $txt_newuserpass2, 40) != 0) {
            	$errMsgs[$validate_err] = "Password fields do not match.";
            	$errArray[$validate_err++] = "txt_newuserpass";
            	$errArray[$validate_err++] = "txt_newuserpass2";
            } else if (CheckPasswordReq($txt_newuserpass) == false) {
            	$errMsgs[$validate_err] = "Password must contain characters from three of the following four categories:<br/>
                                           - Uppercase characters of European languages<br/>
                                           - Lowercase characters of European languages<br/>
                                           - Base 10 digits (0 through 9)<br/>
                                           - Nonalphanumeric characters: .~`!@#$%^*()-+={}:?_";
            	$errArray[$validate_err++] = "txt_newuserpass";
            	$errArray[$validate_err++] = "txt_newuserpass2";
            }
        }

        if (0 < trim($_POST['hid_whitewanlen'])) {
            for ($i=0; $i<trim($_POST['hid_whitewanlen']); $i++) {
                $temp = trim($_POST['txt_wlwan_ipaddr'][$i]);
                if (FALSE !== ($slashloc = strpos($temp,'/')) ) {
                    //echo "txt_wlwan_ipaddr[" . $i ."] is CIDR type with / at " . $slashloc . ": " . substr($temp,0,$slashloc) . " / " . substr($temp,$slashloc+1) . "<br>";
                    if (0 == preg_match($ValidIpv4AddressRegex,substr($temp,0,$slashloc))) {
                        $errMsgs[$validate_err] = "Whitelist entry " . (string)($i+1) . " does not have a valid IP address before the \"/\".";
                        $errArray[$validate_err++] = "txt_wlwan_ipaddr[" . $i . "]";
                    } else if ( ( 1 > substr($temp,$slashloc+1)) ||
                                (32 < substr($temp,$slashloc+1)) ||
                                (!ctype_digit(substr($temp,$slashloc+1))) ||
                                (1 > strlen(substr($temp,$slashloc+1))) ||
                                (2 < strlen(substr($temp,$slashloc+1)))) {
                        $errMsgs[$validate_err] = "Whitelist entry " . (string)($i+1) . " must have a numeric value between 1 and 32 after the slash.";
                        $errArray[$validate_err++] = "txt_wlwan_ipaddr[" . $i . "]";
                    }
                } else if (FALSE !== ($dashloc = strpos($temp,'-')) ) {
                    //echo "txt_wlwan_ipaddr[" . $i ."] is range type with - at " . $dashloc . ": " . substr($temp,0,$dashloc) . " - " . substr($temp,$dashloc+1) . "<br>";
                    if (0 == preg_match($ValidIpv4AddressRegex,substr($temp,0,$dashloc))) {
                        $errMsgs[$validate_err] = "Whitelist entry " . (string)($i+1) . " does not have a valid IP address before the \"-\".";
                        $errArray[$validate_err++] = "txt_wlwan_ipaddr[" . $i . "]";
                    } else if (0 == preg_match($ValidIpv4AddressRegex,substr($temp,$dashloc+1))) {
                        $errMsgs[$validate_err] = "Whitelist entry " . (string)($i+1) . " does not have a valid IP address after the \"-\".";
                        $errArray[$validate_err++] = "txt_wlwan_ipaddr[" . $i . "]";
                    }
                } else {
                    //echo "txt_wlwan_ipaddr[" . $i ."] is IP type<br>";
                    if ((0 == preg_match($ValidIpv4AddressRegex,$temp)) || (strlen($temp) > 32)) {
                        $errMsgs[$validate_err] = "Whitelist entry " . (string)($i+1) . " is not a valid IP address.";
                        $errArray[$validate_err++] = "txt_wlwan_ipaddr[" . $i . "]";
                    }
                }
            }
        }

        /* Save parameters */
        if (0 == $validate_err) {
            /* if validation OK, write settings to flash */
            $error = false;

            $retval = ConfigWrite("web/port",$txt_webport);
            if (0 > $retval[0])	{
                echo "<SCRIPT>
                  alert('Error setting web port!');
                window.location = 'administrator_access.php';</SCRIPT>";
                $error = true;
            }
            $retval = ConfigWrite("web/protocol",$sel_webprotocol);
            if (0 > $retval[0]) {
                echo "<SCRIPT>
                  alert('Error setting protocol!');
                window.location = 'administrator_access.php';</SCRIPT>";
                $error = true;
            }
            $retval = ConfigWrite("web/accessonwan",$sel_webaccessWAN);
            if (0 > $retval[0]) {
                echo "<SCRIPT>
                  alert('Error setting End Device IP address!');
                window.location = 'administrator_access.php';</SCRIPT>";
                $error = true;
            }
            if (strlen($txt_newuserpass) > 0) {
                $md5txt_newuserpass = md5(trim($txt_newuserpass));
                $retval = ConfigWrite("web/pass",$md5txt_newuserpass);
                if (0 > $retval[0]) {
                    echo "<SCRIPT>
                      alert('Error setting password!');
                    window.location = 'administrator_access.php';</SCRIPT>";
                    $error = true;
                }
            }
            $retval = ConfigWrite("web/uname",$txt_newusername);
            if (0 > $retval[0]) {
                echo "<SCRIPT>
                  alert('Error setting new user name.');
                window.location = 'administrator_access.php';</SCRIPT>";
                $error = true;
            }

            if (htmlspecialchars_decode(trim($_POST['sel_WLActive'])) == 0) {
                $temp = 0;
            } else {
                $temp = htmlspecialchars_decode(trim($_POST['hid_whitewanlen']));
            }
            $retval = ConfigWrite("whitelist/wanlen", $temp);
            if (0 > $retval[0]) {
                echo "<SCRIPT>
                  alert('Error setting WAN whitelist table length!');
                window.location = 'configuration_advance.php';</SCRIPT>";
                $error = true;
            }

            for ($i=0; $i < $temp; $i++) {
                $retval = ConfigWrite("whitelist/wanaddr" . $i,htmlspecialchars_decode(trim($_POST["txt_wlwan_ipaddr"][$i])));
                if (0 > $retval[0]) {
                    echo "<SCRIPT>
                      alert('Error setting IP Address for WAN whitelist entry " . $i . "!');
                    window.location = 'configuration_advance.php';</SCRIPT>";
                    $error = true;
                }
            }

            if (!$error) {
                $retval = ConfigToFlash();
                if (1 != $retval[0]) {
                    echo "<SCRIPT>
                      alert('Error saving new configuration!');
                    window.location = 'configuration_basic.php';</SCRIPT>";
                }
            }

            RebootModule();
            echo "<script>window.location = 'rebooting.php';</script>";
        }
    }

    /* Get $ConfigInfo */
    $CfgRequest="web/port web/protocol web/accessonwan web/uname whitelist/wanlen";
    /* Load / Reload values */

    if ($validate_err > 0) {
        $CfgStarter[0]=trim($_POST['txt_webport']);
        $CfgStarter[1]=trim($_POST['sel_webprotocol']);
        $CfgStarter[2]=trim($_POST['sel_webaccessWAN']);
        $CfgStarter[3]= trim($_POST['txt_newusername']);
        $CfgStarter[4]=trim($_POST['hid_whitewanlen']);
    } else {
        exec("/psft/bin/cfgModify readlist " . $CfgRequest, $CfgStarter);
    }

    $ConfigInfo=array();
    $ConfigInfo["web"] = array(
      "port"        => $CfgStarter[0],
      "protocol"    => $CfgStarter[1],
      "accessonwan" => $CfgStarter[2],
      "uname"       => $CfgStarter[3] );
    $ConfigInfo["whitelist"] = array(
      "wanlen"      => $CfgStarter[4] );
    if ($validate_err > 0) {
        /* validation error - load from $_POST, not CFG API */
        for ($i=0; $i<$ConfigInfo["whitelist"]["wanlen"]; $i++) {
            $TblData[$i]=trim($_POST['txt_wlwan_ipaddr'][$i]);
        }
    } else {
        /* load parameters normally */
        $CfgRequest = null;
        for ($i=0; $i<$ConfigInfo["whitelist"]["wanlen"]; $i++) {
            if ($i != 0) {
                $CfgRequest .= " ";
            }
            $CfgRequest .= "whitelist/wanaddr" . $i . " ";
        }
        exec("/psft/bin/cfgModify readlist " . $CfgRequest, $TblData);
    }
    for ($i=0; $i<$ConfigInfo["whitelist"]["wanlen"]; $i++) {
        $ConfigInfo["whitelist"]["wanaddr"][$i] = $TblData[$i];
    }

?>
<html>
<head>
<META HTTP-EQUIV="Pragma" CONTENT="no-cache"/>
<META HTTP-EQUIV="Expires" CONTENT="-1"/>
<link rel="stylesheet" href="/css/styles.css" type="text/css" />
<script src="/scripts/angular.min.js" type="text/JavaScript"></script>
<script src="/scripts/jquery.min.js" type="text/JavaScript"></script>
<script src="/js/application.js" type="text/JavaScript"></script>
<script src="/js/main.js" type="text/JavaScript"></script>
<style type="text/css">
<!--
.STYLE1 {font-size: 9px}
-->
</style>
</head>
<body leftmargin="0" topmargin="0" rightmargin="0" bottommargin="0" marginwidth="0" onload='parent.resizeIframe(document.body.scrollHeight)'>
<!-- Start Body -->
<table width=100% height="550" cellpadding="0" cellspacing="0" >
  <tr height=13 >
    <td width=14 ><img src="/images/leftopangle.jpg" /></td>
    <td background="/images/bgtop.jpg" ></td>
    <td width=14 ><img src="/images/rightopangle.jpg" /></td>
  </tr>
  <tr>
    <td background="/images/bgleft.jpg" >&nbsp;</td>
    <td align="center" valign="top">
<!--Start content here -->
    <table width="98%" border="0" cellspacing="0" cellpadding="0" bgcolor="#002569">
      <tr>
        <td width="100%" bgcolor="#FFFFFF">
<?php if ($validate_err > 0) {
  echo "<h3 style='color:#f04040'>The parameters highlighted in red are incorrect.  Please update with a valid value.</h3>";
  for ($i=0;$i < $validate_err; $i++) {
    echo "<h4 style='color:#f04040'>" .$errMsgs[$i] . "</h4>";
  }
} ?>
<!-- Body starts here -->
          <form action="administrator_access.php?action=1" name="formwebinfo" id="formwebinfo" method="post" >
          <input type="hidden" name="csrf" value="<?php echo $_SESSION['csrf']; ?>" />
          <table width="100%" border="0" cellspacing="0" cellpadding="0"><tr><td>
            <table width="100%" border="0" align="left" cellpadding="4" cellspacing="0">
              <tr class="tablehead"><td colspan="2">Web Access</td></tr>
              <tr><td height="4" colspan="4" align="left" bgcolor="#4d92a8"></td></tr>
              <tr class="sorttable">
                <td width="21%" align="left">Web Port </td>
                <td width="79%" class="inputcol"><input name="txt_webport" type="text" class="copy" id="txt_webport" value="<?php echo htmlspecialchars($ConfigInfo['web']['port']);?>" size="10" maxlength="20" /></td>
              </tr>
              <tr class="sorttablenobg">
                <td >Web Protocol </td>
                <td class="inputcol"><select name="sel_webprotocol" class="copy" id="sel_webprotocol">
                  <option value="0" <?php if ($ConfigInfo['web']['protocol'] == 0) {
                      echo ("selected");} ?> >HTTP</option>
                  <option value="1" <?php if ($ConfigInfo['web']['protocol'] == 1) {
                      echo ("selected");} ?> >HTTPS</option>
                </select></td>
              </tr>
              <tr class="sorttable">
                <td >Web Access On WAN </td>
                <td class="inputcol"><select name="sel_webaccessWAN" class="copy" id="sel_webaccessWAN">
                  <option value="1" <?php
                    if ($ConfigInfo['web']['accessonwan'] == 1) {
                      echo ("selected");
                    } ?> >Enable</option>
                  <option value="0" <?php
                    if ($ConfigInfo['web']['accessonwan'] == 0) {
                      echo ("selected");
                    }?> >Disable</option>
                </select></td>
              </tr>
            </table>
          </td></tr>
          <tr><td>
            <table width="100%" border="0" align="left" cellpadding="4" cellspacing="0">
              <tr class="tablehead"><td height="10" colspan="2">&nbsp;</td></tr>
              <tr class="tablehead"><td colspan="2">Web Login</td></tr>
              <tr><td height="4" colspan="4" align="left" bgcolor="#4d92a8"></td></tr>
              <tr class="sorttable">
                <td width="21%">User</td>
                <td width="79%" class="inputcol"><input name="txt_newusername"  id="txt_newusername" value="<?php echo htmlspecialchars($ConfigInfo['web']['uname']);?>" type="text" class="copy" size="30" maxlength="64"/></td>
              </tr>
              <tr class="sorttablenobg">
                <td>New Password </td>
                <td class="inputcol"><input name="txt_newuserpass" id="txt_newuserpass" type="password" class="copy" size="30" maxlength="64"/></td>
              </tr>
              <tr class="sorttable">
                <td>Confirm Password</td>
                <td class="inputcol"><input name="txt_newuserpass2" id="txt_newuserpass2" type="password" class="copy" size="30" maxlength="64" /></td>
              </tr>
            </table>
          </td></tr>
          <tr><td>
            <table width="100%" border="0" align="left" cellpadding="4" cellspacing="0">
              <tr class="tablehead"><td colspan="2" height="10"></td></tr>
              <tr class="tablehead"><td colspan="2">White List</td></tr>
              <tr><td height="4" colspan="4" align="left" bgcolor="#4d92a8"></td></tr>
              <tr class="sorttable">
                <td align="left">Active</td>
                <td class="inputcol"><select name="sel_WLActive" class="copy" id="sel_WLActive" onChange="ShowHideAdmAccess('sel_WLActive')">
                  <option value="0" <?php if ($ConfigInfo['whitelist']['wanlen'] == 0) {
                          echo ("selected");}?>>Disable</option>
                  <option value="1" <?php if ($ConfigInfo['whitelist']['wanlen'] > 0) {
                          echo ("selected");}?>>Enable</option>
                </select></td>
              </tr>
            </table>
            <div id="divWhiteList_WANlist" style="display:none">
              <table id="tbl_whitewanlist" border="0" cellspacing="0" cellpadding="0" width="100%">
                <tr> <td height="25" colspan="7" bgcolor="#3399CC" class="STYLE1" id="tdWhiteWanList"><b>WAN Whitelist</b></td></tr>
<?php
  for ($i=0; $i<$ConfigInfo["whitelist"]["wanlen"]; $i++) {
    echo '<tr class="';
    if ($i%2 == 1) echo 'sorttablenobg'; else echo 'sorttable';
    echo '"><td width="25" class="inputcol"><input name = "ck_wlwan_del[' . $i . ']" id="ck_wlwan_del[' . $i . ']" type="checkbox" class="copy"/></td> ';
    echo '<td width="150">IP Address</td>';
    echo '<td class="inputcol"><input name="txt_wlwan_ipaddr[' . $i . ']" id="txt_wlwan_ipaddr[' . $i . ']" type="text" class="copy" value="' . htmlspecialchars($ConfigInfo["whitelist"]["wanaddr"][$i]) . '" size="31" maxlength="31"/></td>';
  }
?>
              </table>
              <div>
                <input name="wlwan_addtolist" type="button" id="wlwan_addtolist" value="Add Entry to WAN Whitelist" onClick="AddWLWANTableEntry();" class="tablehead"/>
                <input name="wlwan_removefromlist" type="button" id="wlwan_removefromlist" value="Remove Selected Entres" onClick="RemoveWLWANTableEntry();" class="tablehead"/>
              </div>
            </div>
            <div id="hidTableLen" style="display:none">
              <input name="hid_whitewanlen" type="text" id="hid_whitewanlen" value="<?php echo htmlspecialchars($ConfigInfo["whitelist"]["wanlen"]); ?>">
            </div>
            <tr><td height="4" colspan="4" align="left" bgcolor="#4d92a8"></td></tr>
                  <tr class="tablehead">
                    <td colspan="2" height="100" valign="bottom" align="center" ><input name="btn_WLApply" type="submit" id="btn_WLApply" value="Apply" class="copy"></td>
                  </tr>
                </td>
              </tr>
              </table></td>
            </tr>
          </table>
          </form>
    <!--End content here -->
    </td>
    <td background="/images/bgright.jpg">&nbsp;</td>
  </tr>
  <tr height=16>
    <td><img src="/images/leftbtmangle.jpg" /></td>
    <td background="/images/bgbtm.jpg"></td>
    <td><img src="/images/rightbtmangle.jpg" /></td>
  </tr>
</table>
<!--End body -->
<SCRIPT>
<?php //Set the top menu style
//alert(window.parent.document.getElementById("leftlink_administrator_access")); ?>
window.parent.document.getElementById("leftlink_administrator_system").className = "leftNavLink";
window.parent.document.getElementById("leftlink_administrator_access").className = "leftNavLinkActive";
window.parent.document.getElementById("leftlink_administrator_logs").className = "leftNavLink";
window.parent.document.getElementById("leftlink_administrator_sms").className = "leftNavLink";
window.parent.document.getElementById("leftlink_administrator_ping").className = "leftNavLink";
ShowHideAdmAccess("sel_WLActive");
var errcnt = <?php echo $validate_err;?>;
var errarray<?php
if ($validate_err > 0) {
  echo '=[';
}
for ($i=0; $i<$validate_err; $i++) {
  if ($i != 0 ) {
    echo ',';
  }
  echo "\"$errArray[$i]\"";
}
if ($validate_err > 0) {
  echo ']';
}

?>;

HighlightValidationErrors(errcnt, errarray);</SCRIPT>
</body>
</html>
