<?php
    session_start();
    require_once('../inc/userfunction.php');
    require_once('../inc/systemfunction.php');
    require_once('../inc/filefunction.php');
    require_once('../inc/uploadclass.php');

    // Check the login session
    if (CheckUserSession() == false) {
        header("Location: /components/session_expired.php");
        exit;
    }
    
    $method = $_SERVER['REQUEST_METHOD'];
    $csrf = $_POST['csrf'];
    
    // Check the CSRF token
    if (CheckCSRFToken($method, $csrf) == false) {
    	error_log("Warning! Possible CSRF detected.", 0);
    	exit;
    }

    $validate_err = 0;
    $section_err = array(
        "tdWANTitle"    => false,
        "tdDDNSTitle"   => false,
        "tdVPNTitle"    => false,
        "tdSerialTitle" => false,
        "tdCellUsage"   => false,
        "tdNTPConfig"   => false );

    /* Verify what service settings were modified */
    $serviceStatus = array(
        "serviceWAN"        => serviceAction::None,
        "serviceKeepAlive"  => serviceAction::None,
        "serviceCellUsage"  => serviceAction::None,
        "serviceDDNS"       => serviceAction::None,
        "serviceVPN"        => serviceAction::None,
        "serviceIPSec"      => serviceAction::None,
        "serviceSerencap"   => serviceAction::None,
        "serviceNTP"        => serviceAction::None,
        "serviceEIP"        => serviceAction::None );
    
     /* Get $ConfigInfo */
    $CfgRequest = "wan/autopin wan/pin wan/keepalive wan/keepaddr wan/datalimit " .
                  "wan/planstartday wan/plansize1 wan/plansize2 wan/datablock ppp/dialup " .
                  "ppp/apnuser ppp/apnpass ppp/redialtime ppp/authentication ppp/roaming " .
                  "ddns/mode ddns/system ddns/hostname ddns/username ddns/password ovpn/mode " .
                  "ovpn/dir ovpn/tls ovpn/remote ovpn/port ovpn/customconfig ovpn/certauth " .
                  "ovpn/clientcert ovpn/clientkey ovpn/daemon ovpn/verb ipsec/mode ipsec/dir " .
                  "ipsec/IKEport ipsec/L2TPport ipsec/localid ipsec/remoteip ipsec/remoteid " .
                  "ipsec/remotesubnet ipsec/presharedkey ipsec/version ipsec/usrconn " .
                  "ipsec/remoteCIDRmask ipsec/dhcp serial/bdrate serial/data serial/parity " .
                  "serial/stop serial/flow serial/mode encap/mode encap/role encap/maxpkt " .
                  "encap/timeoutbits encap/listenport encap/porttype encap/modfilter " .
                  "encap/df1errmode encap/tablelen module/name system/ntpenable system/ntpmode " .
                  "system/ntpserver system/eipenable lan/ipadr lan/ipCIDRmask";

    /* Load parameters to populate array first */
    exec("/psft/bin/cfgModify readlist " . $CfgRequest, $CfgStarter);

    $ConfigInfo = array();
    $ConfigInfo["wan"] = array(
        "autopin" => $CfgStarter[0],
        "pin" => $CfgStarter[1],
        "keepalive" => $CfgStarter[2],
        "keepaddr" => $CfgStarter[3],
        "datalimit" => $CfgStarter[4],
        "planstartday" => $CfgStarter[5],
        "plansize1" => $CfgStarter[6],
        "plansize2" => $CfgStarter[7],
        "datablock" => $CfgStarter[8] );
    $ConfigInfo["ppp"] = array(
        "dialup" => $CfgStarter[9],
        "apnuser" => $CfgStarter[10],
        "apnpass" => $CfgStarter[11],
        "redialtime" => $CfgStarter[12],
        "authentication" => $CfgStarter[13],
        "roaming" => $CfgStarter[14] );
    $ConfigInfo["ddns"] = array(
        "mode" => $CfgStarter[15],
        "system" => $CfgStarter[16],
        "hostname" => $CfgStarter[17],
        "username" => $CfgStarter[18],
        "password" => $CfgStarter[19] );
    $ConfigInfo["ovpn"] = array(
        "mode" => $CfgStarter[20],
        "dir" => $CfgStarter[21],
        "tls" => $CfgStarter[22],
        "server" => $CfgStarter[23],
        "port" => $CfgStarter[24],
        "customconfig" => $CfgStarter[25],
        "certauth" => $CfgStarter[26],
        "clientcert" => $CfgStarter[27],
        "clientkey" => $CfgStarter[28],
        "daemon" => $CfgStarter[29],
        "verb" => $CfgStarter[30] );
    $ConfigInfo["ipsec"] = array(
        "mode" => $CfgStarter[31],
        "dir" => $CfgStarter[32],
        "IKEport" => $CfgStarter[33],
        "L2TPport" => $CfgStarter[34],
        "localid" => $CfgStarter[35],
        "remoteip" => $CfgStarter[36],
        "remoteid" => $CfgStarter[37],
        "remotesubnet" => $CfgStarter[38],
        "presharedkey" => base64_decode($CfgStarter[39]),
        "version" => $CfgStarter[40],
        "usrconn" => $CfgStarter[41],
        "remoteCIDRmask" => $CfgStarter[42],
        "dhcp" => $CfgStarter[43] );
    $ConfigInfo["serial"] = array(
        "bdrate" => $CfgStarter[44],
        "data" => $CfgStarter[45],
        "parity" => $CfgStarter[46],
        "stop" => $CfgStarter[47],
        "flow" => $CfgStarter[48],
        "mode" => $CfgStarter[49] );
    $ConfigInfo["encap"] = array(
        "mode" => $CfgStarter[50],
        "role" => $CfgStarter[51],
        "maxpkt" => $CfgStarter[52],
        "timeoutbits" => $CfgStarter[53],
        "listenport" => $CfgStarter[54],
        "porttype" => $CfgStarter[55],
        "modfilter" => $CfgStarter[56],
        "df1errmode" => $CfgStarter[57],
        "tablelen" => $CfgStarter[58] );
    $ConfigInfo["module"] = array(
        "name" => $CfgStarter[59] );
    $ConfigInfo["ntp"] = array(
        "enable" => $CfgStarter[60],
        "mode" => $CfgStarter[61],
        "server" => $CfgStarter[62] );
    $ConfigInfo["eip"] = array(
        "enable" => $CfgStarter[63] );
    $ConfigInfo["lan"] = array(
    	"ipadr" => $CfgStarter[64],
    	"ipCIDRmask" => $CfgStarter[65] );

    $cfgEncapData = null;
    for ($i = 0; $i < $ConfigInfo["encap"]["tablelen"]; $i++) {
        if ($i != 0) {
            $cfgEncapData .= " ";
        }
        $cfgEncapData .= "encap/tableseraddr" . $i . " ";
        $cfgEncapData .= "encap/tableip" . $i . " ";
        $cfgEncapData .= "encap/tableport" . $i . " ";
        $cfgEncapData .= "encap/tableporttype" . $i;
    }
    exec("/psft/bin/cfgModify readlist " . $cfgEncapData, $TblData);
    $ConfigInfo["encap"]["table"] = array();
    for ($i = 0; $i < $ConfigInfo["encap"]["tablelen"]; $i++) {
        $ConfigInfo["encap"]["table"][$i] = array(
            "seraddr" => $TblData[$i*4],
            "ip" => $TblData[$i*4+1],
            "port" => $TblData[$i*4+2],
            "porttype" => $TblData[$i*4+3]
        );
    }

    $action = $_REQUEST['action'];
    if ($action == 1 ) {
        /* @deprecated Insufficient validation */
        $ValidIpv4AddressRegex = "/^(([01]?[0-9]?[0-9]|2([0-4][0-9]|5[0-5]))\.){3}([01]?[0-9]?[0-9]|2([0-4][0-9]|5[0-5]))$/";

        /* @deprecated Insufficient validation */
        $ValidHostnameRegex = "/^([a-zA-Z0-9]|[a-zA-Z0-9][a-zA-Z0-9\-]{0,61}[a-zA-Z0-9])(\.([a-zA-Z0-9]|[a-zA-Z0-9][a-zA-Z0-9\-]{0,61}[a-zA-Z0-9]))*$/";
        $ValidChars = "/^$|^[a-zA-Z0-9.\~\!\@\#\$\%\^\*\(\)\-\+\=\{\}\:\?\_]+$/i";
        $validServerIpRegex = "/^(?:(?!(?:127)(?:\.\d{1,3}){3})(?:[1-9]\d?|1\d\d|2[0-4]\d|25[0-5])(?:\.(?:1?\d{1,2}|2[0-4]\d|25[0-5])){2}(?:\.(?:[1-9]\d?|1\d\d|2[0-4]\d|25[0-4])))$/";
        $validServerNameRegex = "/^(?:(?:(?:[a-z0-9]-*)*[a-z0-9]+)(?:\.(?:[a-z0-9]-*)*[a-z0-9]+)*(?:\.(?:[a-z]{2,}))\.?)$/";

        /* Get Credential Files for ovpn */
        $cfgOvpnCert = "ovpn/certauth ovpn/clientcert ovpn/clientkey";
        exec("/psft/bin/cfgModify readlist " . $cfgOvpnCert, $CredFiles);
        $cacert = $CredFiles[0];
        $cccert = $CredFiles[1];
        $ckcert = $CredFiles[2];
        /* Get Configuration Files for ovpn */
        exec("/psft/bin/cfgModify read ovpn/customconfig", $OVPNCfgFile);

        /* Get ports that must be unique and not configured on this page */
        $PortParams = "web/port ssh/port discserv/port dns/port ftp/controlport " .
            "ipsec/IKEport ipsec/L2TPport";
        exec("/psft/bin/cfgModify readlist " . $PortParams, $PortData);
        $PortData[7] = 502;
        $PortData[8] = 30722;

        $protocol = trim($_POST['slt_SL_Protocol']);
        $portMode = trim($_POST['slt_SL_PT']);

        $temp = trim($_POST['txt_pppredialtime']);
        if (!ctype_digit($temp) || ($temp < 0) || ($temp > 86400)) {
            $errMsgs[$validate_err] = "Wait between Reconnect must be an integer between 0 and 86400.";
            $errArray[$validate_err++] = "txt_pppredialtime";
            $section_err["tdWANTitle"] = true;
        }

        $txt_pppapnuser = htmlspecialchars(trim($_POST['txt_pppapnuser']));
        if (strlen($txt_pppapnuser) > 40) {
            $errMsgs[$validate_err] = "User Name must be less than 40 characters long.";
            $errArray[$validate_err++] = "txt_pppapnuser";
            $section_err["tdWANTitle"] = true;
        }

        if (!preg_match($ValidChars, $txt_pppapnuser)) {
            $errMsgs[$validate_err] = "User Name uses invalid characters.";
            $errArray[$validate_err++] = "txt_pppapnuser";
            $section_err["tdWANTitle"] = true;
        }

        $txt_ppppass = htmlspecialchars(trim($_POST['txt_ppppass']));
        if (strlen($txt_ppppass) > 40) {
            $errMsgs[$validate_err] = "Password must be less than 40 characters long.";
            $errArray[$validate_err++] = "txt_ppppass";
            $section_err["tdWANTitle"] = true;
        }
        if (!preg_match($ValidChars, $txt_ppppass)) {
            $errMsgs[$validate_err] = "Password uses invalid characters.";
            $errArray[$validate_err++] = "txt_ppppass";
            $section_err["tdWANTitle"] = true;
        }

        if (1 == trim($_POST['slt_wankeepalive'])) {
            $temp = trim($_POST['txt_wankeepaddr']);
            if (((0 == preg_match($validServerIpRegex, $temp)) &&
                (0 == preg_match($validServerNameRegex, $temp))) ||
                (strlen($temp) > 36)) {
                $errMsgs[$validate_err] = "Keep Alive Ping Address is not a valid IP address or hostname.";
                $errArray[$validate_err++] = "txt_wankeepaddr";
                $section_err["tdWANTitle"] = true;
            }
            $temp = trim($_POST['txt_wankeepalive']);
            if (!ctype_digit($temp) || ($temp < 0) || ($temp > 86400)) {
                $errMsgs[$validate_err] = "Keep Alive Ping Period must be an integer between 0 and 86400.";
                $errArray[$validate_err++] = "txt_wankeepalive";
                $section_err["tdWANTitle"] = true;
            }

        }

        $temp = trim($_POST['txt_planstartday']);
        if (!ctype_digit($temp) || ($temp < 1) || ($temp > 31) ) {
            $errMsgs[$validate_err] = "Plan Start Day must be an integer between 1 and 31.";
            $errArray[$validate_err++] = "txt_planstartday";
            $section_err["tdCellUsage"] = true;
        }

        $temp = trim($_POST['txt_wanplansize']);
        if (!ctype_digit($temp) || ($temp < 1)) {
            $errMsgs[$validate_err] = "Plan Size may only be a positive integer.";
            $errArray[$validate_err++] = "txt_wanplansize";
            $section_err["tdCellUsage"] = true;
        }

        if (1 == trim($_POST['slt_DDNSActive']) || 2 == trim($_POST['slt_DDNSActive'])) {
            $temp = trim($_POST['txt_DDNSServer']);
            if ((strlen($temp) == 0) || (strlen($temp) > 36)) {
                $errMsgs[$validate_err] = "DDNS Server may not be blank (36 character limit).";
                $errArray[$validate_err++] = "txt_DDNSServer";
                $section_err["tdDDNSTitle"] = true;
            } elseif (!preg_match($ValidChars, $temp)) {
                $errMsgs[$validate_err] = "DDNS Server uses invalid characters.";
                $errArray[$validate_err++] = "txt_DDNSServer";
                $section_err["tdDDNSTitle"] = true;
            }
            $temp = trim($_POST['txt_DDNSDomainName']);
            if (((0 == preg_match($ValidIpv4AddressRegex, $temp)) &&
                (0 == preg_match($ValidHostnameRegex, $temp)) ) ||
                (strlen($temp) > 35)) {
                $errMsgs[$validate_err] = "ICX35 Domain Name is not a valid IP address or hostname (35 character limit).";
                $errArray[$validate_err++] = "txt_DDNSDomainName";
                $section_err["tdDDNSTitle"] = true;
            }
            $temp = trim($_POST['txt_DDNSUserName']);
            if ((strlen($temp) == 0) || (strlen($temp) > 40)) {
                $errMsgs[$validate_err] = "DDNS User may not be blank, or exceed 40 character limit.";
                $errArray[$validate_err++] = "txt_DDNSUserName";
                $section_err["tdDDNSTitle"] = true;
            } elseif (!preg_match($ValidChars, $temp)) {
                $errMsgs[$validate_err] = "DDNS User uses invalid characters.";
                $errArray[$validate_err++] = "txt_DDNSUserName";
                $section_err["tdDDNSTitle"] = true;
            }
            $temp = trim($_POST['txt_DDNSPass']);
            if ((strlen($temp) == 0) || (strlen($temp) > 40)) {
                $errMsgs[$validate_err] = "DDNS Password may not be blank, or exceed 40 character limit.";
                $errArray[$validate_err++] = "txt_DDNSPass";
                $section_err["tdDDNSTitle"] = true;
            } elseif (!preg_match($ValidChars, $temp)) {
                $errMsgs[$validate_err] = "DDNS Password uses invalid characters.";
                $errArray[$validate_err++] = "txt_DDNSPass";
                $section_err["tdDDNSTitle"] = true;
            }
        }

        if (1 == trim($_POST['sltVPNType'])) {
            $temp = trim($_POST['txt_OpenVPN_TLS']);
            if (!ctype_digit($temp) || ($temp < 0) || ($temp > 86400)) {
                $errMsgs[$validate_err] = "TLS Renegotiation Time must be a positive integer between 86400 or less.";
                $errArray[$validate_err++] = "txt_OpenVPN_TLS";
                $section_err["tdVPNTitle"] = true;
            }
            $temp = trim($_POST['txt_OpenVPN_ServerIP']);
            if (((0 == preg_match($ValidIpv4AddressRegex, $temp)) &&
                (0 == preg_match($ValidHostnameRegex, $temp)) ) || (strlen($temp) > 40)) {
                $errMsgs[$validate_err] = "OpenVPN Server Address is not a valid IP address or hostname (40 character limit).";
                $errArray[$validate_err++] = "txt_OpenVPN_ServerIP";
                $section_err["tdVPNTitle"] = true;
            }
            $temp = trim($_POST['txt_OpenVPN_ServerPort']);
            $tempErrFlag = false;
            for ($i = 0; $i < count($PortData); $i++) {
                if ($temp == $PortData[$i]) {
                    $tempErrFlag = true;
                    break;
                }
            }
            if ($tempErrFlag) {
                $errMsgs[$validate_err] = "OpenVPN Server Port is identical to a reserved or inuse port, please choose another port.";
                $errArray[$validate_err++] = "txt_OpenVPN_ServerPort";
                $section_err["tdVPNTitle"] = true;
            }
            if (!ctype_digit($temp) || ($temp < 1) || ($temp > 65535)) {
                $errMsgs[$validate_err] = "OpenVPN Server Port is not a valid port number (1 - 65535).";
                $errArray[$validate_err++] = "txt_OpenVPN_ServerPort";
                $section_err["tdVPNTitle"] = true;
            }

            /* Submit OVPN Credential Files, if available */
            global $con;
            $ca_file = basename($_FILES['file_OpenVPN_CA']['name']);
            $cc_file = basename($_FILES['file_OpenVPN_CC']['name']);
            $ck_file = basename($_FILES['file_OpenVPN_CK']['name']);

            /* check if all files were selected */
            if (!$ca_file && !$cacert) {
                $errMsgs[$validate_err] = "Must select a valid OpenVPN Certificate Authority file.";
                $errArray[$validate_err++] = "file_OpenVPN_CA";
                $section_err["tdVPNTitle"] = true;
            }
            if (!$cc_file && !$cccert) {
                $errMsgs[$validate_err] = "Must select a valid OpenVPN Client Certificate file.";
                $errArray[$validate_err++] = "file_OpenVPN_CC";
                $section_err["tdVPNTitle"] = true;
            }
            if (!$ck_file && !$ckcert) {
                $errMsgs[$validate_err] = "Must select a valid OpenVPN Client Key file.";
                $errArray[$validate_err++] = "file_OpenVPN_CK";
                $section_err["tdVPNTitle"] = true;
            }

            /* Submit OVPN Configuration File, if available */
            $conf_file = basename($_FILES['file_OpenVPN_conf']['name']);
        }
        if (2 == trim($_POST['sltVPNType'])) {
            $temp = trim($_POST['txt_ipsec_localid']);
            if ((strlen($temp) == 0) || (strlen($temp) > 40)) {
                $errMsgs[$validate_err] = "IPSec Local Identifier may not be blank, or exceed 40 character limit.";
                $errArray[$validate_err++] = "txt_ipsec_localid";
                $section_err["tdVPNTitle"] = true;
            }
            $temp = trim($_POST['txt_ipsec_remoteip']);
            if (((!preg_match($validServerIpRegex, $temp)) &&
                (!preg_match($validServerNameRegex, $temp))) || (strlen($temp) > 40)) {
                $errMsgs[$validate_err] = "IPSec Remote Address is not a valid IP address or hostname (40 character limit).";
                $errArray[$validate_err++] = "txt_ipsec_remoteip";
                $section_err["tdVPNTitle"] = true;
            }
            $temp = trim($_POST['txt_ipsec_remoteid']);
            if ((strlen($temp) == 0) || (strlen($temp) > 40)) {
                $errMsgs[$validate_err] = "IPSec Remote Identifier may not be blank (40 character limit).";
                $errArray[$validate_err++] = "txt_ipsec_remoteid";
                $section_err["tdVPNTitle"] = true;
            }
            $txt_IPSecIPadr = trim($_POST['txt_ipsec_remotesubnet']);
            if ((0 == preg_match($ValidIpv4AddressRegex,$txt_IPSecIPadr)) || (strlen($txt_IPSecIPadr) > 15)) {
                $errMsgs[$validate_err] = "IPSec Remote Subnet does not contain a valid IP address";
                $errArray[$validate_err++] = "txt_ipsec_remotesubnet";
                $section_err["tdVPNTitle"] = true;
            }
            $txt_IPSecIPmask = trim($_POST['txt_ipsec_remoteCIDRmask']);
            if (!ctype_digit($txt_IPSecIPmask) || ($txt_IPSecIPmask< 1) || ($txt_IPSecIPmask> 32)) {
                $errMsgs[$validate_err] = "IPSec Remote Subnet must have a numeric value between 1 and 32 after the slash.";
                $errArray[$validate_err++] = "txt_ipsec_remoteCIDRmask";
                $section_err["tdVPNTitle"] = true;
            }
            
            $subnetLoLimitIPSec = ip2long(ipCalc("n", $txt_IPSecIPadr, $txt_IPSecIPmask)[0]);
            $subnetHiLimitIPSec = ip2long(ipCalc("b", $txt_IPSecIPadr, $txt_IPSecIPmask)[0]);
            $subnetLoLimitLAN = ip2long(ipCalc("n", $ConfigInfo["lan"]["ipadr"], $ConfigInfo["lan"]["ipCIDRmask"])[0]);
            $subnetHiLimitLAN = ip2long(ipCalc("b", $ConfigInfo["lan"]["ipadr"], $ConfigInfo["lan"]["ipCIDRmask"])[0]);
            
            if ($subnetLoLimitIPSec == $subnetLoLimitLAN || $subnetHiLimitIPSec == $subnetHiLimitLAN) {
            	$errMsgs[$validate_err] = "IPSec Remote Subnet cannot be the same as LAN.";
            	$errArray[$validate_err++] = "txt_ipsec_remoteCIDRmask";
            	$section_err["tdVPNTitle"] = true;
            }

            $temp = trim($_POST['txt_ipsec_presharedkey']);
            if ((strlen($temp) == 0) || (strlen($temp) > 30)) {
                $errMsgs[$validate_err] = "IPSec Pre-shared Key may not be blank (30 character limit).";
                $errArray[$validate_err++] = "txt_ipsec_presharedkey";
                $section_err["tdVPNTitle"] = true;
            }
        }

        if (0 != $protocol) {
            $temp = trim($_POST['text_gen_timeout']);
            if (!ctype_digit($temp) || ($temp < 1) || ($temp > 65535)) {
                $errMsgs[$validate_err] = "Timeout Bits must be an integer (1-65535).";
                $errArray[$validate_err++] = "text_gen_timeout";
                $section_err["tdSerialTitle"] = true;
            }

            $temp = 1024;
            if (!ctype_digit($temp) || ($temp < 1) || ($temp > 1024)) {
                $errMsgs[$validate_err] = "Max Packet Size must be an integer (1 - 513).";
                $errArray[$validate_err++] = "txt_gen_maxpacket";
                $section_err["tdSerialTitle"] = true;
            }

            if ($portMode == 2) {
                if ($protocol == 7) {
                    $errMsgs[$validate_err] = "DF1 Full Duplex cannot be used in RS485 mode.";
                    $errArray[$validate_err++] = "slt_SL_PT";
                    $errArray[$validate_err++] = "slt_SL_Protocol";
                    $section_err["tdSerialTitle"] = true;
                } elseif($protocol == 8) {
                    $errMsgs[$validate_err] = "DF1 Radio Modem cannot be used in RS485 mode.";
                    $errArray[$validate_err++] = "slt_SL_PT";
                    $errArray[$validate_err++] = "slt_SL_Protocol";
                    $section_err["tdSerialTitle"] = true;
                }
            }

            if ((1 == trim($_POST['slt_SL_Protocol'])) && (0 == trim($_POST['slt_gen_role']) || 2 == trim($_POST['slt_gen_role']))) {
                $temp = trim($_POST['txt_genclient_addr']);
                if (((0 == preg_match($validServerIpRegex, $temp)) &&
                    (0 == preg_match($validServerNameRegex, $temp)) ) || (strlen($temp) > 40)) {
                    $errMsgs[$validate_err] = "Serial Encapsulation Remote IP/Domain is not a valid IP Address or hostname (40 character limit).";
                    $errArray[$validate_err++] = "txt_genclient_addr";
                    $section_err["tdSerialTitle"] = true;
                }
            }

            if (1 != trim($_POST['slt_SL_Protocol']) && 0 < trim($_POST['encapTableLen']) && (0 == trim($_POST['slt_gen_role']) || 2 == trim($_POST['slt_gen_role']))) {
                for ($i = 0; $i < trim($_POST['encapTableLen']); $i++) {
                    if ( !ctype_digit(trim($_POST['txt_encap_seraddr'][$i])) ) {
                        $errMsgs[$validate_err] = "Encapsulation Table entry " . (string)($i+1) . " must have a numeric value for Serial Address.";
                        $errArray[$validate_err++] = "txt_encap_seraddr[" . $i . "]";
                        $section_err["tdSerialTitle"] = true;
                    }
                    $temp = trim($_POST['txt_encap_ipaddr'][$i]);
                    if (((0 == preg_match($validServerIpRegex, $temp)) &&
                        (0 == preg_match($validServerNameRegex, $temp)) ) || (strlen($temp) > 40)) {
                        $errMsgs[$validate_err] = "Encapsulation Table entry " . (string)($i+1) . " does not have a valid IP Address or hostname (40 character limit).";
                        $errArray[$validate_err++] = "txt_encap_ipaddr[" . $i . "]";
                        $section_err["tdSerialTitle"] = true;
                    }
                }
            }
        }

        /* NTP Settings */
        if (1 == trim($_POST['dropdown_NTPEnable'])) {
            if (strlen(htmlspecialchars_decode(trim($_POST['txt_NTPServer']))) > 40 ||
                (!preg_match($validServerIpRegex, htmlspecialchars(trim($_POST['txt_NTPServer']))) &&
                    !preg_match($validServerNameRegex, htmlspecialchars(trim($_POST['txt_NTPServer']))))) {
                $errMsgs[$validate_err] = "NTP Server Address uses an invalid format (40 character limit).";
                $errArray[$validate_err++] = "txt_NTPServer";
                $section_err["tdNTPConfig"] = true;
            }
        }

        if (0 == $validate_err) {
            /* If validation OK, write settings to flash */
            $error = false;

            /* WAN configuration parameters */
            if (0 != strcmp($txt_pppapnuser, $ConfigInfo["ppp"]["apnuser"])) {
                $serviceStatus["serviceWAN"] = serviceAction::Reboot;
                $retval = ConfigWrite("ppp/apnuser", $txt_pppapnuser);
                if (0 > $retval[0]) {
                    echo "<SCRIPT>
                        alert('Error setting APN username!');
                        window.location = 'configuration_advance.php';</SCRIPT>";
                    $error = true;
                }
            }

            if (0 != strcmp($txt_ppppass, $ConfigInfo["ppp"]["apnpass"])) {
                $serviceStatus["serviceWAN"] = serviceAction::Reboot;
                $retval = ConfigWrite("ppp/apnpass", $txt_ppppass);
                if (0 > $retval[0]) {
                    echo "<SCRIPT>
                        alert('Error setting APN password!');
                        window.location = 'configuration_advance.php';</SCRIPT>";
                    $error = true;
                }
            }

            if (0 != strcmp(htmlspecialchars_decode(trim($_POST['txt_pppredialtime'])), $ConfigInfo["ppp"]["redialtime"])) {
                $serviceStatus["serviceWAN"] = serviceAction::Reboot;
                $retval = ConfigWrite("ppp/redialtime", htmlspecialchars_decode(trim($_POST['txt_pppredialtime'])));
                if (0 > $retval[0]) {
                    echo "<SCRIPT>
                        alert('Error setting reconnect delay!');
                        window.location = 'configuration_advance.php';</SCRIPT>";
                    $error = true;
                }
            }

            if (0 != strcmp(trim($_POST['sel_wanauth']), $ConfigInfo["ppp"]["authentication"])) {
                $serviceStatus["serviceWAN"] = serviceAction::Reboot;
                $retval = ConfigWrite("ppp/authentication", trim($_POST['sel_wanauth']));
                if (0 > $retval[0]) {
                    echo "<SCRIPT>
                        alert('Error setting PPP authentication type!');
                        window.location = 'configuration_advance.php';</SCRIPT>";
                    $error = true;
                }
            }

            if (0 != strcmp(trim($_POST['slt_wanroaming']), $ConfigInfo["ppp"]["roaming"])) {
                $serviceStatus["serviceWAN"] = serviceAction::Reboot;
                $retval = ConfigWrite("ppp/roaming", trim($_POST['slt_wanroaming']));
                if (0 > $retval[0]) {
                    echo "<SCRIPT>
                        alert('Error setting roaming permission!');
                        window.location = 'configuration_advance.php';</SCRIPT>";
                    $error = true;
                }
            }

            /* KeepAlive configuration parameters */
            if (0 == trim($_POST['slt_wankeepalive'])) {
                if(0 != strcmp("0", $ConfigInfo["wan"]["keepalive"])) {
                    $serviceStatus["serviceKeepAlive"] = serviceAction::Stop;
                    $retval = ConfigWrite("wan/keepalive", "0");
                    if (0 > $retval[0]) {
                        echo "<SCRIPT>
                            alert('Error setting connection keepalive!');
                            window.location = 'configuration_advance.php';</SCRIPT>";
                        $error = true;
                    }
                }
            } else if (0 != strcmp(htmlspecialchars_decode(trim($_POST['txt_wankeepalive'])), $ConfigInfo["wan"]["keepalive"])) {
                $serviceStatus["serviceKeepAlive"] = serviceAction::Restart;
                $retval = ConfigWrite("wan/keepalive", htmlspecialchars_decode(trim($_POST['txt_wankeepalive'])));
                if (0 > $retval[0]) {
                    echo "<SCRIPT>
                        alert('Error setting connection keepalive!');
                        window.location = 'configuration_advance.php';</SCRIPT>";
                    $error = true;
                }

                if (0 != strcmp(htmlspecialchars_decode(trim($_POST['txt_wankeepaddr'])), $ConfigInfo["wan"]["keepaddr"])) {
                    $serviceStatus["serviceKeepAlive"] = serviceAction::Restart;
                    $retval = ConfigWrite("wan/keepaddr", htmlspecialchars_decode(trim($_POST['txt_wankeepaddr'])));
                    if (0 > $retval[0]) {
                        echo "<SCRIPT>
                            alert('Error setting connection keepalive!');
                            window.location = 'configuration_advance.php';</SCRIPT>";
                        $error = true;
                    }
                }
            }

            /* Cellular Usage configuration parameters */
            if (0 != strcmp(htmlspecialchars_decode(trim($_POST['txt_planstartday'])), $ConfigInfo["wan"]["planstartday"])) {
                $serviceStatus["serviceCellUsage"] = serviceAction::Restart;
                $retval = ConfigWrite("wan/planstartday", htmlspecialchars_decode(trim($_POST['txt_planstartday'])));
                if (0 > $retval[0]) {
                    echo "<SCRIPT>
                        alert('Error setting cellular usage plan start day!');
                        window.location = 'configuration_advance.php';</SCRIPT>";
                    $error = true;
                }
            }
            if (0 != strcmp(htmlspecialchars_decode(trim($_POST['txt_wanplansize'])), $ConfigInfo["wan"]["plansize2"])) {
                $serviceStatus["serviceCellUsage"] = serviceAction::Restart;
                $retval = ConfigWrite("wan/plansize2", htmlspecialchars_decode(trim($_POST['txt_wanplansize'])));
                if (0 > $retval[0]) {
                    echo "<SCRIPT>
                        alert('Error setting cellular usage plan size!');
                        window.location = 'configuration_advance.php';</SCRIPT>";
                    $error = true;
                }
            }
            if (0 != strcmp(trim($_POST['txt_wanplansize_u']), $ConfigInfo["wan"]["plansize1"])) {
                $serviceStatus["serviceCellUsage"] = serviceAction::Restart;
                $retval = ConfigWrite("wan/plansize1", trim($_POST['txt_wanplansize_u']));
                if (0 > $retval[0]) {
                    echo "<SCRIPT>
                        alert('Error setting cellular usage unit!');
                        window.location = 'configuration_advance.php';</SCRIPT>";
                    $error = true;
                }
            }
            if (0 != strcmp(trim($_POST['slt_wandatablock']), $ConfigInfo["wan"]["datablock"])) {
                $serviceStatus["serviceCellUsage"] = serviceAction::Restart;
                $retval = ConfigWrite("wan/datablock", trim($_POST['slt_wandatablock']));
                if (0 > $retval[0]) {
                    echo "<SCRIPT>
                        alert('Error setting data block mode!');
                        window.location = 'configuration_advance.php';</SCRIPT>";
                    $error = true;
                }
            }
            if (0 != strcmp(trim($_POST['slt_wandataplanlimit']), $ConfigInfo["wan"]["datalimit"])) {
                $serviceStatus["serviceCellUsage"] = (trim($_POST['slt_wandataplanlimit']) == 0) ? serviceAction::Stop : serviceAction::Restart;
                $retval = ConfigWrite("wan/datalimit", trim($_POST['slt_wandataplanlimit']));
                if (0 > $retval[0]) {
                    echo "<SCRIPT>
                        alert('Error setting cellular usage tracking mode!');
                        window.location = 'configuration_advance.php';</SCRIPT>";
                    $error = true;
                }
            }

            /* DDNS configuration parameters */
            if (1 == trim($_POST['slt_DDNSActive']) || 2 == trim($_POST['slt_DDNSActive'])) {
                if (0 != strcmp(trim($_POST['slt_DDNSActive']), $ConfigInfo["ddns"]["mode"])) {
                    $serviceStatus["serviceDDNS"] = serviceAction::Restart;
                    $retval = ConfigWrite("ddns/mode", trim($_POST['slt_DDNSActive']));
                    if (0 > $retval[0]) {
                        echo "<SCRIPT>
                            alert('Error setting DDNS mode!');
                            window.location = 'configuration_advance.php';</SCRIPT>";
                        $error = true;
                    }
                }
                if (0 != strcmp(htmlspecialchars_decode(trim($_POST['txt_DDNSServer'])), $ConfigInfo["ddns"]["system"])) {
                    $serviceStatus["serviceDDNS"] = serviceAction::Restart;
                    $retval = ConfigWrite("ddns/system", htmlspecialchars_decode(trim($_POST['txt_DDNSServer'])));
                    if (0 > $retval[0]) {
                        echo "<SCRIPT>
                            alert('Error setting DDNS system!');
                            window.location = 'configuration_advance.php';</SCRIPT>";
                        $error = true;
                    }
                }
                if (0 != strcmp(htmlspecialchars_decode(trim($_POST['txt_DDNSDomainName'])), $ConfigInfo["ddns"]["hostname"])) {
                    $serviceStatus["serviceDDNS"] = serviceAction::Restart;
                    $retval = ConfigWrite("ddns/hostname", htmlspecialchars_decode(trim($_POST['txt_DDNSDomainName'])));
                    if (0 > $retval[0]) {
                        echo "<SCRIPT>
                            alert('Error setting ICX35 DDNS domainname!');
                            window.location = 'configuration_advance.php';</SCRIPT>";
                        $error = true;
                    }
                }
                if (0 != strcmp(htmlspecialchars_decode(trim($_POST['txt_DDNSUserName'])), $ConfigInfo["ddns"]["username"])) {
                    $serviceStatus["serviceDDNS"] = serviceAction::Restart;
                    $retval == ConfigWrite("ddns/username", htmlspecialchars_decode(trim($_POST['txt_DDNSUserName'])));
                    if (0 > $retval[0]) {
                        echo "<SCRIPT>
                            alert('Error setting DDNS username!');
                            window.location = 'configuration_advance.php';</SCRIPT>";
                        $error = true;
                    }
                }
                if (0 != strcmp(htmlspecialchars_decode(trim($_POST['txt_DDNSPass'])), $ConfigInfo["ddns"]["password"])) {
                    $serviceStatus["serviceDDNS"] = serviceAction::Restart;
                    $retval = ConfigWrite("ddns/password", htmlspecialchars_decode(trim($_POST['txt_DDNSPass'])));
                    if (0 > $retval[0]) {
                        echo "<SCRIPT>
                            alert('Error setting DDNS password!');
                            window.location = 'configuration_advance.php';</SCRIPT>";
                        $error = true;
                    }
                }
            }

            /* On disable clear all saved params */
            if (0 == trim($_POST['slt_DDNSActive'])) {
               if (0 != strcmp(trim($_POST['slt_DDNSActive']), $ConfigInfo["ddns"]["mode"])) {
                    $serviceStatus["serviceDDNS"] = serviceAction::Stop;
                    $retval = ConfigWrite("ddns/mode", trim($_POST['slt_DDNSActive']));
                    if (0 > $retval[0]) {
                        echo "<SCRIPT>
                            alert('Error setting DDNS mode!');
                            window.location = 'configuration_advance.php';</SCRIPT>";
                        $error = true;
                    }
                }
               if (0 != strcmp(htmlspecialchars_decode(trim($_POST[""])), $ConfigInfo["ddns"]["system"])) {
                    $serviceStatus["serviceDDNS"] = serviceAction::Stop;
                    $retval = ConfigWrite("ddns/system", htmlspecialchars_decode(trim($_POST[""])));
                    if (0 > $retval[0]) {
                        echo "<SCRIPT>
                            alert('Error setting DDNS system!');
                            window.location = 'configuration_advance.php';</SCRIPT>";
                        $error = true;
                    }
                }
               if (0 != strcmp(htmlspecialchars_decode(trim($_POST[""])), $ConfigInfo["ddns"]["hostname"])) {
                    $serviceStatus["serviceDDNS"] = serviceAction::Stop;
                    $retval = ConfigWrite("ddns/hostname", htmlspecialchars_decode(trim($_POST[""])));
                    if (0 > $retval[0]) {
                        echo "<SCRIPT>
                            alert('Error setting ICX35 DDNS domainname!');
                            window.location = 'configuration_advance.php';</SCRIPT>";
                        $error = true;
                    }
                }
               if (0 != strcmp(htmlspecialchars_decode(trim($_POST[""])), $ConfigInfo["ddns"]["username"])) {
                    $serviceStatus["serviceDDNS"] = serviceAction::Stop;
                    $retval == ConfigWrite("ddns/username", htmlspecialchars_decode(trim($_POST[""])));
                    if (0 > $retval[0]) {
                        echo "<SCRIPT>
                            alert('Error setting DDNS username!');
                            window.location = 'configuration_advance.php';</SCRIPT>";
                        $error = true;
                    }
                }
               if (0 != strcmp(htmlspecialchars_decode(trim($_POST[""])), $ConfigInfo["ddns"]["password"])) {
                    $serviceStatus["serviceDDNS"] = serviceAction::Stop;
                    $retval = ConfigWrite("ddns/password", htmlspecialchars_decode(trim($_POST[""])));
                    if (0 > $retval[0]) {
                        echo "<SCRIPT>
                            alert('Error setting DDNS password!');
                            window.location = 'configuration_advance.php';</SCRIPT>";
                        $error = true;
                    }
                }
            }

            /* VPN/IPSec configuration parameters */
            switch (trim($_POST['sltVPNType'])) {
                case 0:
                    $ovpnmodetemp = "0";
                    $ipsecmodetemp = "0";

                    // Clear IPSec PreSharedKey field
                    if (0 != strcmp(htmlspecialchars_decode(trim($_POST[""])), $ConfigInfo["ipsec"]["presharedkey"])) {
                        $serviceStatus["serviceIPSec"] = serviceAction::Stop;
                        $serviceStatus["serviceVPN"] = serviceAction::Stop;
                        $retval = ConfigWrite("ipsec/presharedkey", "");
                        if (0 > $retval[0]) {
                            echo "<SCRIPT>
                                alert('Error clearing IPSec PreSharedKey!');
                                window.location = 'configuration_advance.php';</SCRIPT>";
                            $error = true;
                        }
                    }
                    break;
                case 1:
                    $ovpnmodetemp = "5";
                    $ipsecmodetemp = "0";
                    $serviceStatus["serviceIPSec"] = serviceAction::Stop;

                    // Write OVPN settings to config file
                    if (0 != strcmp(htmlspecialchars_decode(trim($_POST['txt_OpenVPN_TLS'])), $ConfigInfo["ovpn"]["tls"])) {
                        $serviceStatus["serviceVPN"] = serviceAction::Restart;
                        $retval = ConfigWrite("ovpn/tls", htmlspecialchars_decode(trim($_POST['txt_OpenVPN_TLS'])));
                        if (0 > $retval[0]) {
                            echo "<SCRIPT>
                                alert('Error setting openvpntls!');
                                window.location = 'configuration_advance.php';</SCRIPT>";
                            $error = true;
                        }
                    }
                    if (0 != strcmp(htmlspecialchars_decode(trim($_POST['txt_OpenVPN_ServerIP'])), $ConfigInfo["ovpn"]["server"])) {
                        $serviceStatus["serviceVPN"] = serviceAction::Restart;
                        $retval = ConfigWrite("ovpn/remote", htmlspecialchars_decode(trim($_POST['txt_OpenVPN_ServerIP'])));
                        if (0 > $retval[0]) {
                            echo "<SCRIPT>
                                alert('Error setting OpenVPN server!');
                                window.location = 'configuration_advance.php';</SCRIPT>";
                            $error = true;
                        }
                    }
                    if (0 != strcmp(htmlspecialchars_decode(trim($_POST['txt_OpenVPN_ServerPort'])), $ConfigInfo["ovpn"]["port"])) {
                        $serviceStatus["serviceVPN"] = serviceAction::Restart;
                        $retval = ConfigWrite("ovpn/port", htmlspecialchars_decode(trim($_POST['txt_OpenVPN_ServerPort'])));
                        if (0 > $retval[0]) {
                            echo "<SCRIPT>
                                alert('Error setting OpenVPN port!');
                                window.location = 'configuration_advance.php';</SCRIPT>";
                            $error = true;
                        }
                    }
                    if ($ca_file) {
                        DelVPNFile($CredFiles[0]);
                        if (0 != strcmp($ca_file, $ConfigInfo["ovpn"]["certauth"])) {
                            $serviceStatus["serviceVPN"] = serviceAction::Restart;
                            $retval = ConfigWrite("ovpn/certauth", $ca_file);
                            if (0 > $retval[0]) {
                                echo "<SCRIPT>
                                    alert('Error setting certificate authority filename!');
                                    window.location = 'configuration_advance.php';</SCRIPT>";
                                $error = true;
                            }
                        }
                    }
                    if ($cc_file) {
                        DelVPNFile($CredFiles[1]);
                        if (0 != strcmp($cc_file, $ConfigInfo["ovpn"]["clientcert"])) {
                            $serviceStatus["serviceVPN"] = serviceAction::Restart;
                            $retval = ConfigWrite("ovpn/clientcert", $cc_file);
                            if (0 > $retval[0]) {
                                echo "<SCRIPT>
                                    alert('Error setting client certificate filename!');
                                    window.location = 'configuration_advance.php';</SCRIPT>";
                                $error = true;
                            }
                        }
                    }
                    if ($ck_file) {
                        DelVPNFile($CredFiles[2]);
                        if (0 != strcmp($ck_file, $ConfigInfo["ovpn"]["clientkey"])) {
                            $serviceStatus["serviceVPN"] = serviceAction::Restart;
                            $retval = ConfigWrite("ovpn/clientkey", $ck_file);
                            if (0 > $retval[0]) {
                                echo "<SCRIPT>
                                    alert('Error setting client key filename!');
                                    window.location = 'configuration_advance.php';</SCRIPT>";
                                $error = true;
                            }
                        }
                    }
                    if ($conf_file) {
                        DelVPNFile($OVPNCfgFile[0]);
                        if (0 != strcmp($conf_file, $ConfigInfo["ovpn"]["customconfig"])) {
                            $serviceStatus["serviceVPN"] = serviceAction::Restart;
                            $retval = ConfigWrite("ovpn/customconfig", $conf_file);
                            if (0 > $retval[0]) {
                                echo "<SCRIPT>
                                    alert('Error setting custom configuration filename!');
                                    window.location = 'configuration_advance.php';</SCRIPT>";
                                $error = true;
                            }
                        }
                    }

                    // Clear IPSec PreSharedKey field
                    if (0 != strcmp("", $ConfigInfo["ovpn"]["presharedkey"])) {
                        // $serviceStatus["serviceVPN"] = serviceAction::Reboot;
                        $retval = ConfigWrite("ipsec/presharedkey", "");
                        if (0 > $retval[0]) {
                            echo "<SCRIPT>
                                alert('Error clearing IPSec PreSharedKey!');
                                window.location = 'configuration_advance.php';</SCRIPT>";
                            $error = true;
                        }
                    }
                    break;
                case 2:
                    $ovpnmodetemp = "0";
                    $ipsecmodetemp = "4";
                    $serviceStatus["serviceVPN"] = serviceAction::Stop;

                    // Write IPSec settings to config file
                    if (0 != strcmp(htmlspecialchars_decode(trim($_POST['txt_ipsec_localid'])), $ConfigInfo["ipsec"]["localid"])) {
                        $serviceStatus["serviceIPSec"] = serviceAction::Restart;
                        $retval = ConfigWrite("ipsec/localid", htmlspecialchars_decode(trim($_POST['txt_ipsec_localid'])));
                        if (0 > $retval[0]) {
                            echo "<SCRIPT>
                                alert('Error setting IPSec local group IP!');
                                window.location = 'configuration_advance.php';</SCRIPT>";
                            $error = true;
                        }
                    }

                    if (0 != strcmp(htmlspecialchars_decode(trim($_POST['txt_ipsec_remoteip'])), $ConfigInfo["ipsec"]["remoteip"])) {
                        $serviceStatus["serviceIPSec"] = serviceAction::Restart;
                        $retval = ConfigWrite("ipsec/remoteip", htmlspecialchars_decode(trim($_POST['txt_ipsec_remoteip'])));
                        if (0 > $retval[0]) {
                            echo "<SCRIPT>
                                alert('Error setting IPSec remote IP!');
                                window.location = 'configuration_advance.php';</SCRIPT>";
                            $error = true;
                        }
                    }

                    if (0 != strcmp(htmlspecialchars_decode(trim($_POST['txt_ipsec_remoteid'])), $ConfigInfo["ipsec"]["remoteid"])) {
                        $serviceStatus["serviceIPSec"] = serviceAction::Restart;
                        $retval = ConfigWrite("ipsec/remoteid", htmlspecialchars_decode(trim($_POST['txt_ipsec_remoteid'])));
                        if (0 > $retval[0]) {
                            echo "<SCRIPT>
                                alert('Error setting IPSec remote group IP!');
                                window.location = 'configuration_advance.php';</SCRIPT>";
                            $error = true;
                        }
                    }

                    if (0 != strcmp(htmlspecialchars_decode(trim($_POST['txt_ipsec_remotesubnet'])), $ConfigInfo["ipsec"]["remotesubnet"])) {
                        $serviceStatus["serviceIPSec"] = serviceAction::Restart;
                        $retval = ConfigWrite("ipsec/remotesubnet", htmlspecialchars_decode(trim($_POST['txt_ipsec_remotesubnet'])));
                        if (0 > $retval[0]) {
                            echo "<SCRIPT>
                                alert('Error setting IPSec remote group subnet mask!');
                                window.location = 'configuration_advance.php';</SCRIPT>";
                            $error = true;
                        }
                    }

                    if (0 != strcmp(htmlspecialchars_decode(trim($_POST['txt_ipsec_remoteCIDRmask'])), $ConfigInfo["ipsec"]["remoteCIDRmask"])) {
                        $serviceStatus["serviceIPSec"] = serviceAction::Restart;
                        $retval = ConfigWrite("ipsec/remoteCIDRmask", htmlspecialchars_decode(trim($_POST['txt_ipsec_remoteCIDRmask'])));
                        if (0 > $retval[0]) {
                            echo "<SCRIPT>
                                alert('Error setting IPSec remote group subnet mask!');
                                window.location = 'configuration_advance.php';</SCRIPT>";
                            $error = true;
                        }
                    }

                    if (0 != strcmp(htmlspecialchars_decode(trim($_POST['txt_ipsec_presharedkey'])), $ConfigInfo["ipsec"]["presharedkey"])) {
                        $serviceStatus["serviceIPSec"] = serviceAction::Restart;
                        $retval = ConfigWrite("ipsec/presharedkey", base64_encode(htmlspecialchars_decode(trim($_POST['txt_ipsec_presharedkey']))));
                        if (0 > $retval[0]) {
                            echo "<SCRIPT>
                                alert('Error setting IPSec PreSharedKey!');
                                window.location = 'configuration_advance.php';</SCRIPT>";
                            $error = true;
                        }
                    }

                    break;
            }

            if (0 != strcmp($ovpnmodetemp, $ConfigInfo["ovpn"]["mode"])) {
                $serviceStatus["serviceVPN"] = ($ovpnmodetemp == 0) ? serviceAction::Stop : serviceAction::Restart;
                $retval = ConfigWrite("ovpn/mode", $ovpnmodetemp);
                if (0 > $retval[0]) {
                    echo "<SCRIPT>
                        alert('Error setting VPN mode!');
                        window.location = 'configuration_advance.php';</SCRIPT>";
                    $error = true;
                }
            }

            if (0 != strcmp($ipsecmodetemp, $ConfigInfo["ipsec"]["mode"])) {
                $serviceStatus["serviceIPSec"] = ($ipsecmodetemp == 0) ? serviceAction::Stop : serviceAction::Restart;
                $retval = ConfigWrite("ipsec/mode", $ipsecmodetemp);
                if (0 > $retval[0]) {
                    echo "<SCRIPT>
                        alert('Error setting VPN mode!');
                        window.location = 'configuration_advance.php';</SCRIPT>";
                    $error = true;
                }
            }

            /* Save Serial settings */
            if (0 != strcmp($portMode, $ConfigInfo["serial"]["mode"])) {
                $serviceStatus["serviceSerencap"] = serviceAction::Restart;
                $retval = ConfigWrite("serial/mode", $portMode);
                if (0 > $retval[0]) {
                    echo "<SCRIPT>
                        alert('Error setting Port Mode!');
                        window.location = 'configuration_advance.php';</SCRIPT>";
                    $error = true;
                }
            }

            if (0 != strcmp(trim($_POST['slt_SL_BR']), $ConfigInfo["serial"]["bdrate"])) {
                $serviceStatus["serviceSerencap"] = serviceAction::Restart;
                $retval = ConfigWrite("serial/bdrate", trim($_POST['slt_SL_BR']));
                if (0 > $retval[0]) {
                    echo "<SCRIPT>
                        alert('Error setting baudrate!');
                        window.location = 'configuration_advance.php';</SCRIPT>";
                    $error = true;
                }
            }

            if (0 != strcmp(trim($_POST['slt_SL_DB']), $ConfigInfo["serial"]["data"])) {
                $serviceStatus["serviceSerencap"] = serviceAction::Restart;
                $retval = ConfigWrite("serial/data", trim($_POST['slt_SL_DB']));
                if (0 > $retval[0]) {
                    echo "<SCRIPT>
                        alert('Error setting databits!');
                        window.location = 'configuration_advance.php';</SCRIPT>";
                    $error = true;
                }
            }

            if (0 != strcmp(trim($_POST['slt_SL_Parity']), $ConfigInfo["serial"]["parity"])) {
                $serviceStatus["serviceSerencap"] = serviceAction::Restart;
                $retval = ConfigWrite("serial/parity", trim($_POST['slt_SL_Parity']));
                if (0 > $retval[0]) {
                    echo "<SCRIPT>
                        alert('Error setting parity!');
                        window.location = 'configuration_advance.php';</SCRIPT>";
                    $error = true;
                }
            }

            if (0 != strcmp(trim($_POST['slt_SL_STB']), $ConfigInfo["serial"]["stop"])) {
                $serviceStatus["serviceSerencap"] = serviceAction::Restart;
                $retval = ConfigWrite("serial/stop", trim($_POST['slt_SL_STB']));
                if (0 > $retval[0]) {
                    echo "<SCRIPT>
                        alert('Error setting stopbits!');
                        window.location = 'configuration_advance.php';</SCRIPT>";
                    $error = true;
                }
            }

            if (1 == trim($_POST['sltVPNType'])) {
                $protocol = "0";
                $serviceStatus["serviceSerencap"] = serviceAction::Stop;
            }
            if (0 != strcmp(trim($_POST['slt_gen_role']), $ConfigInfo["encap"]["role"])) {
                $serviceStatus["serviceSerencap"] = ($serviceStatus["serviceSerencap"] == serviceAction::Restart);
                $retval = ConfigWrite("encap/role", trim($_POST['slt_gen_role']));
                if (0 > $retval[0]) {
                    echo "<SCRIPT>
                        alert('Error setting encapsulation role!');
                        window.location = 'configuration_advance.php';</SCRIPT>";
                    $error = true;
                }
            }
            if (0 != strcmp("1024", $ConfigInfo["encap"]["maxpkt"])) {
                $serviceStatus["serviceSerencap"] = serviceAction::Restart;
                $retval = ConfigWrite("encap/maxpkt", 1024);
                if (0 > $retval[0]) {
                    echo "<SCRIPT>
                        alert('Error setting encapsulation maximum packet size!');
                        window.location = 'configuration_advance.php';</SCRIPT>";
                    $error = true;
                }
            }
            if (0 != strcmp(htmlspecialchars_decode(trim($_POST['text_gen_timeout'])), $ConfigInfo["encap"]["timeoutbits"])) {
                $serviceStatus["serviceSerencap"] = serviceAction::Restart;
                $retval = ConfigWrite("encap/timeoutbits", htmlspecialchars_decode(trim($_POST['text_gen_timeout'])));
                if (0 > $retval[0]) {
                    echo "<SCRIPT>
                        alert('Error setting encapsulation timeout bits!');
                        window.location = 'configuration_advance.php';</SCRIPT>";
                    $error = true;
                }
            }

            /* Port number */
            if ((4 == $_POST['slt_SL_Protocol']) || (5 == $_POST['slt_SL_Protocol'])) {
                $tempport = 502;
            } else {
                $tempport = 30722;
            }
            if (0 != strcmp($tempport, $ConfigInfo["encap"]["listenport"])) {
                $serviceStatus["serviceSerencap"] = serviceAction::Restart;
                $retval = ConfigWrite("encap/listenport", $tempport);
                if (0 > $retval[0]) {
                    echo "<SCRIPT>
                        alert('Error setting encapsulation listen port!');
                        window.location = 'configuration_advance.php';</SCRIPT>";
                    $error = true;
                }
            }
            if (0 != strcmp(trim($_POST['txt_sl_server_pt']), $ConfigInfo["encap"]["porttype"])) {
                $serviceStatus["serviceSerencap"] = serviceAction::Restart;
                $retval = ConfigWrite("encap/porttype", trim($_POST['txt_sl_server_pt']));
                if (0 > $retval[0]) {
                    echo "<SCRIPT>
                        alert('Error setting encapsulation port type!');
                        window.location = 'configuration_advance.php';</SCRIPT>";
                    $error = true;
                }
            }

            /* Permanently disable modfilter, until further notice */
            if (0 != strcmp("0", $ConfigInfo["encap"]["modfilter"])) {
                $serviceStatus["serviceSerencap"] = serviceAction::Restart;
                $retval = ConfigWrite("encap/modfilter", 0);
                if (0 > $retval[0]) {
                    echo "<SCRIPT>
                        alert('Error setting dial number!');
                        window.location = 'configuration_advance.php';</SCRIPT>";
                    $error = true;
                }
            }
            if (0 != strcmp(trim($_POST['slt_df1_ex']), $ConfigInfo["encap"]["df1errmode"])) {
                $serviceStatus["serviceSerencap"] = serviceAction::Restart;
                $retval = ConfigWrite("encap/df1errmode", trim($_POST['slt_df1_ex']));
                if (0 > $retval[0]) {
                    echo "<SCRIPT>
                        alert('Error setting DF1 error mode!');
                        window.location = 'configuration_advance.php';</SCRIPT>";
                    $error = true;
                }
            }
            if (1 == trim($_POST['slt_SL_Protocol']) && 0 == trim($_POST['slt_gen_tro'])) {
                if (0 != strcmp("1", $ConfigInfo["encap"]["tablelen"])) {
                    $serviceStatus["serviceSerencap"] = serviceAction::Restart;
                    $retval = ConfigWrite("encap/tablelen", "1");
                    if (0 > $retval[0]) {
                        echo "<SCRIPT>
                            alert('Error setting encapsulation table length!');
                            window.location = 'configuration_advance.php';</SCRIPT>";
                        $error = true;
                    }
                }
            } elseif (0 == trim($_POST['slt_gen_role'])) {
                if (0 != strcmp(htmlspecialchars_decode(trim($_POST['encapTableLen'])), $ConfigInfo["encap"]["tablelen"])) {
                    $serviceStatus["serviceSerencap"] = serviceAction::Restart;
                    $retval = ConfigWrite("encap/tablelen", htmlspecialchars_decode(trim($_POST['encapTableLen'])));
                    if (0 > $retval[0]) {
                        echo "<SCRIPT>
                            alert('Error setting encapsulation table length!');
                            window.location = 'configuration_advance.php';</SCRIPT>";
                        $error = true;
                    }
                }
            }

            /* Do not save configuration for master's slaves if the unit is configured to run as slave */
            if (0 == trim($_POST['slt_gen_role'])) {
                for ($i = 0; $i < $_POST['encapTableLen']; $i++) {
                    if (0 != strcmp(htmlspecialchars_decode(trim($_POST["txt_encap_seraddr"][$i])), $ConfigInfo["encap"]["table"][$i]["seraddr"])) {
                        $serviceStatus["serviceSerencap"] = serviceAction::Restart;
                        $retval = ConfigWrite("encap/tableseraddr" . $i, htmlspecialchars_decode(trim($_POST["txt_encap_seraddr"][$i])));
                        if (0 > $retval[0]) {
                            echo "<SCRIPT>
                                alert('Error setting serial address for table entry " . $i . "!');
                                window.location = 'configuration_advance.php';</SCRIPT>";
                            $error = true;
                        }
                    }
                    if (1 == trim($_POST['slt_SL_Protocol'])) {
                        if (0 != strcmp(htmlspecialchars_decode(trim($_POST["txt_genclient_addr"])), $ConfigInfo["encap"]["table"][$i]["ip"])) {
                            $serviceStatus["serviceSerencap"] = serviceAction::Restart;
                            $retval = ConfigWrite("encap/tableip" . $i, htmlspecialchars_decode(trim($_POST["txt_genclient_addr"])));
                            if (0 > $retval[0]) {
                                echo "<SCRIPT>
                                    alert('Error setting IP/Domain for table entry " . $i . "!');
                                    window.location = 'configuration_advance.php';</SCRIPT>";
                                $error = true;
                            }
                        }
                    } else {
                        if (0 != strcmp(htmlspecialchars_decode(trim($_POST["txt_encap_ipaddr"][$i])), $ConfigInfo["encap"]["table"][$i]["ip"])) {
                            $serviceStatus["serviceSerencap"] = serviceAction::Restart;
                            $retval = ConfigWrite("encap/tableip" . $i, htmlspecialchars_decode(trim($_POST["txt_encap_ipaddr"][$i])));
                            if (0 > $retval[0]) {
                                echo "<SCRIPT>
                                    alert('Error setting IP/Domain for table entry " . $i . "!');
                                    window.location = 'configuration_advance.php';</SCRIPT>";
                                $error = true;
                            }
                        }
                    }
                    if (0 != strcmp(trim($_POST["txt_sl_server_pt"]), $ConfigInfo["encap"]["table"][$i]["porttype"])) {
                        $serviceStatus["serviceSerencap"] = serviceAction::Restart;
                        $retval = ConfigWrite("encap/tableporttype" . $i, trim($_POST["txt_sl_server_pt"]));
                        if (0 > $retval[0]) {
                            echo "<SCRIPT>
                                alert('Error setting porttype for table entry " . $i . "!');
                                window.location = 'configuration_advance.php';</SCRIPT>";
                            $error = true;
                        }
                    }
                    if (0 != strcmp($tempport, $ConfigInfo["encap"]["table"][$i]["port"])) {
                        $serviceStatus["serviceSerencap"] = serviceAction::Restart;
                        $retval = ConfigWrite("encap/tableport" . $i, $tempport);
                        if (0 > $retval[0]) {
                            echo "<SCRIPT>
                                alert('Error setting port for table entry " . $i . "!');
                                window.location = 'configuration_advance.php';</SCRIPT>";
                            $error = true;
                        }
                    }
                }
            }

            if (0 != strcmp($protocol, $ConfigInfo["encap"]["mode"])) {
                $serviceStatus["serviceSerencap"] = ($protocol == 0) ? serviceAction::Stop : serviceAction::Restart;
                $retval = ConfigWrite("encap/mode", $protocol);
                if (0 > $retval[0]) {
                    echo "<SCRIPT>
                        alert('Error setting encapsulation mode!');
                        window.location = 'configuration_advance.php';</SCRIPT>";
                    $error = true;
                }
            }

            /* Save NTP settings */
            if (0 != strcmp(trim($_POST['dropdown_NTPEnable']), $ConfigInfo["ntp"]["enable"])) {
                $serviceStatus["serviceNTP"] = (trim($_POST['dropdown_NTPEnable']) == 0) ? serviceAction::Stop : serviceAction::Restart;
                $retval = ConfigWrite("system/ntpenable", trim($_POST['dropdown_NTPEnable']));
                if (0 > $retval[0]) {
                    echo "<SCRIPT>
                        alert('Error setting NTP Enable!');
                        window.location = 'configuration_advance.php';</SCRIPT>";
                    $error = true;
                }
            }
            /* Do not save NTP configuration if the feature is disabled */
            if (1 == trim($_POST['dropdown_NTPEnable'])) {
                if (0 != strcmp(trim($_POST['dropdown_NTPMode']), $ConfigInfo["ntp"]["mode"])) {
                    $serviceStatus["serviceNTP"] = serviceAction::Restart;
                    $retval = ConfigWrite("system/ntpmode", trim($_POST['dropdown_NTPMode']));
                    if (0 > $retval[0]) {
                        echo "<SCRIPT>
                            alert('Error setting NTP Mode!');
                            window.location = 'configuration_advance.php';</SCRIPT>";
                        $error = true;
                    }
                }
                if (0 != strcmp(htmlspecialchars_decode(trim($_POST['txt_NTPServer'])), $ConfigInfo["ntp"]["server"])) {
                    $serviceStatus["serviceNTP"] = serviceAction::Restart;
                    $retval = ConfigWrite("system/ntpserver", htmlspecialchars_decode(trim($_POST['txt_NTPServer'])));
                    if (0 > $retval[0]) {
                        echo "<SCRIPT>
                            alert('Error setting NTP Server!');
                            window.location = 'configuration_advance.php';</SCRIPT>";
                        $error = true;
                    }
                }
            }

            /* Save EIP settings */
            if (0 != strcmp(trim($_POST['dropdown_EIPEnable']), $ConfigInfo["eip"]["enable"])) {
                $serviceStatus["serviceEIP"] = serviceAction::Restart;
                $retval = ConfigWrite("system/eipenable", trim($_POST['dropdown_EIPEnable']));
                if (0 > $retval[0]) {
                    echo "<SCRIPT>
                        alert('Error setting EIP Enable!');
                        window.location = 'configuration_advance.php';</SCRIPT>";
                    $error = true;
                }
            }

            if (!$error) {
                $retval = ConfigToFlash();
                if (1 != $retval[0]) {
                    echo "<SCRIPT>
                        alert('Error saving new configuration!\\n' . $retval[0]);
                        window.location = 'configuration_advance.php';</SCRIPT>";
                    $error = true;
                }

                if(!$error) {
                    /* save files */
                    $vpnDir = $con[VPNfilepath];
                    $f_upload = new upload_file;
                    if ($ca_file) {
                        $f_upload->set_file_type($_FILES['file_OpenVPN_CA']['type']);   # get file type
                        $f_upload->set_file_name($_FILES['file_OpenVPN_CA']['name']);   # get file name
                        $f_upload->set_file_size($_FILES['file_OpenVPN_CA']['size']);   # get file size
                        $f_upload->set_upfile($_FILES['file_OpenVPN_CA']['tmp_name']);  # temp file name in server
                        $f_upload->set_base_directory($vpnDir);                         # set upload folder
                        $f_upload->set_size(100);                                       # set max upload KB
                        $f_upload->set_extention("crt");
                        $f_upload->set_url("configuration_advance.php");                # redirect url when sucess
                        //DelVPNFile("cfg");
                        $f_upload->save();                                              # save file
                    }
                    if ($cc_file) {
                        $f_upload->set_file_type($_FILES['file_OpenVPN_CC']['type']);   # get file type
                        $f_upload->set_file_name($_FILES['file_OpenVPN_CC']['name']);   # get file name
                        $f_upload->set_file_size($_FILES['file_OpenVPN_CC']['size']);   # get file size
                        $f_upload->set_upfile($_FILES['file_OpenVPN_CC']['tmp_name']);  # temp file name in server
                        $f_upload->set_base_directory($vpnDir);                         # set upload folder
                        $f_upload->set_size(100);                                       # set max upload KB
                        $f_upload->set_extention("crt");
                        $f_upload->set_url("configuration_advance.php");                # redirect url when sucess
                        $f_upload->save();                                              # save file
                    }
                    if ($ck_file) {
                        $f_upload->set_file_type($_FILES['file_OpenVPN_CK']['type']);   # get file type
                        $f_upload->set_file_name($_FILES['file_OpenVPN_CK']['name']);   # get file name
                        $f_upload->set_file_size($_FILES['file_OpenVPN_CK']['size']);   # get file size
                        $f_upload->set_upfile($_FILES['file_OpenVPN_CK']['tmp_name']);  # temp file name in server
                        $f_upload->set_base_directory($vpnDir);                         # set upload folder
                        $f_upload->set_size(100);                                       # set max upload KB
                        $f_upload->set_extention("key");
                        $f_upload->set_url("configuration_advance.php");                # redirect url when sucess
                        $f_upload->save();                                              # save file
                    }
                    if ($conf_file) {
                        $f_upload->set_file_type($_FILES['file_OpenVPN_conf']['type']);     # get file type
                        $f_upload->set_file_name($_FILES['file_OpenVPN_conf']['name']);     # get file name
                        $f_upload->set_file_size($_FILES['file_OpenVPN_conf']['size']);     # get file size
                        $f_upload->set_upfile($_FILES['file_OpenVPN_conf']['tmp_name']);    # temp file name in server
                        $f_upload->set_base_directory($vpnDir);                             # set upload folder
                        $f_upload->set_size(100);                                           # set max upload KB
                        $f_upload->set_extention("conf");
                        $f_upload->save();                                                  # save file
                    }
                }

                foreach ($serviceStatus as $it => $item) {
                    switch ($serviceStatus[$it]) {
                        case serviceAction::Reboot:
                            exec("/psft/bin/reboot.sh &");
                            echo "<script>window.location = 'rebooting.php';</script>";
                            break;
                        case serviceAction::Start:
                            $retval = serviceModify($it, "start");
                            if (0 != $retval[0]) {
                                echo "<SCRIPT>
                                    alert('Error starting " . $it ."!');
                                    window.location = 'configuration_advance.php';</SCRIPT>";
                            }
                            break;
                        case serviceAction::Stop:
                            $retval = serviceModify($it, "stop");
                            if (0 != $retval[0]) {
                                echo "<SCRIPT>
                                    alert('Error stopping " . $it ."!');
                                    window.location = 'configuration_advance.php';</SCRIPT>";
                            }
                            break;
                        case serviceAction::Restart:
                            $retval = serviceModify($it, "restart");
                            if (0 != $retval[0]) {
                                echo "<SCRIPT>
                                    alert('Error restarting " . $it ."!');
                                    window.location = 'configuration_advance.php';</SCRIPT>";
                            }
                            break;
                    }
                }
            }
        }
    }

    if ($action == 4) {
        /* Delete OVPN Configuration File */
        $error = false;
        exec("/psft/bin/cfgModify read ovpn/customconfig", $OVPNCfgFile);
        DelVPNFile($OVPNCfgFile[0]);

        $retval = ConfigWrite("ovpn/customconfig", "");
        if (0 > $retval[0]) {
            echo "<SCRIPT>
                alert('Error clearing custom config file!');
                window.location = 'configuration_advance.php';</SCRIPT>";
            $error = true;
        }

        if (!$error) {
            $retval = ConfigToFlash();
            if (1 != $retval[0]) {
                echo "<SCRIPT>
                    alert('Error saving new configuration!\\n' . $retval[0]);
                    window.location = 'configuration_advance.php';</SCRIPT>";
                $error = true;
            }
        }
    }

    if (($validate_err > 0) || ( $action > 1)) {
        /* Load parameters to populate array first, then overwrite the values that are different */

        /* Validation error or file load - initialize $CfgStart from $_POST, not CFG API */
        $ConfigInfo = array();
        $ConfigInfo["wan"] = array(
            "autopin" => trim($_POST['slt_AutoPin']),
            "pin" => trim($_POST['txt_wanpin']),
            "keepalive" => (trim($_POST['slt_wankeepalive']) == 0) ? 0 : trim($_POST['txt_wankeepalive']),
            "keepaddr" => trim($_POST['txt_wankeepaddr']),
            "datalimit" => trim($_POST['slt_wandataplanlimit']),
            "planstartday" => trim($_POST['txt_planstartday']),
            "plansize1" => trim($_POST['txt_wanplansize_u']),
            "plansize2" => trim($_POST['txt_wanplansize']),
            "datablock" => trim($_POST['slt_wandatablock']) );
        $ConfigInfo["ppp"] = array(
            "dialup" => trim($_POST['txt_pppdialup']),
            "apnuser" => trim($_POST['txt_pppapnuser']),
            "apnpass" => trim($_POST['txt_ppppass']),
            "redialtime" => trim($_POST['txt_pppredialtime']),
            "authentication" => trim($_POST['sel_wanauth']),
            "roaming" => trim($_POST['slt_wanroaming']) );
        $ConfigInfo["ddns"] = array(
            "mode" => trim($_POST['slt_DDNSActive']),
            "system" => trim($_POST['txt_DDNSServer']),
            "hostname" => trim($_POST['txt_DDNSDomainName']),
            "username" => trim($_POST['txt_DDNSUserName']),
            "password" => trim($_POST['txt_DDNSPass']) );
        $ConfigInfo["ovpn"] = array(
            "mode" => 0,
            "tls" => trim($_POST['txt_OpenVPN_TLS']),
            "server" => trim($_POST['txt_OpenVPN_ServerIP']),
            "port" => trim($_POST['txt_OpenVPN_ServerPort']) );
        $ConfigInfo["ipsec"] = array(
            "mode" => 0,
            "localid" => trim($_POST['txt_ipsec_localid']),
            "remoteip" => trim($_POST['txt_ipsec_remoteip']),
            "remoteid" => trim($_POST['txt_ipsec_remoteid']),
            "remotesubnet" => trim($_POST['txt_ipsec_remotesubnet']),
            "presharedkey" => trim($_POST['txt_ipsec_presharedkey']),
            "remoteCIDRmask" => trim($_POST['txt_ipsec_remoteCIDRmask']) );

        if (1 == trim($_POST['sltVPNType'])) {
            $ConfigInfo["ovpn"]["mode"] = 5;
            $ConfigInfo["ipsec"]["mode"] = 0;
        } else if (2 == trim($_POST['sltVPNType']))  {
            $ConfigInfo["ovpn"]["mode"] = 0;
            $ConfigInfo["ipsec"]["mode"] = 1;
        } else {
            $ConfigInfo["ovpn"]["mode"] = 0;
            $ConfigInfo["ipsec"]["mode"] = 0;
        }
        $ConfigInfo["serial"] = array(
            "bdrate" => trim($_POST['slt_SL_BR']),
            "data" => trim($_POST['slt_SL_DB']),
            "parity" => trim($_POST['slt_SL_Parity']),
            "stop" => trim($_POST['slt_SL_STB']),
            "flow" => trim($_POST['slt_SL_FC']),
            "mode" => trim($_POST['slt_SL_PT']) );
        $ConfigInfo["encap"] = array(
            "mode" => trim($_POST['slt_SL_Protocol']),
            "role" => trim($_POST['slt_gen_role']),
            "maxpkt" => trim($_POST['txt_gen_maxpacket']),
            "timeoutbits" => trim($_POST['text_gen_timeout']),
            "listenport" => trim($_POST['txt_sl_server_lp']),
            "porttype" => trim($_POST['txt_sl_server_pt']),
            "modfilter" => trim($_POST['slt_mbs_filter']),
            "df1errmode" => trim($_POST['slt_df1_ex']),
            "tablelen" => trim($_POST['encapTableLen']) );
        $ConfigInfo["ntp"] = array(
            "enable" => trim($_POST['dropdown_NTPEnable']),
            "mode" => trim($_POST['dropdown_NTPMode']),
            "server" => trim($_POST['txt_NTPServer']) );
        $ConfigInfo["eip"] = array(
            "enable" => trim($_POST['dropdown_EIPEnable']) );

        for ($i = 0; $i < $ConfigInfo["encap"]["tablelen"]; $i++) {
            $ConfigInfo["encap"]["table"][$i] = array(
                "seraddr" => trim($_POST['txt_encap_seraddr'][$i]),
                "ip" => trim($_POST['txt_encap_ipaddr'][$i]),
                "port" => trim($_POST['txt_encap_ipaddr'][$i]),
                "porttype" => trim($_POST['slt_encap_porttype'][$i])
            );
        }
    } else {
        /* Load parameters normally */
        $CfgStarter = null;
        exec("/psft/bin/cfgModify readlist " . $CfgRequest, $CfgStarter);

        $ConfigInfo = array();
        $ConfigInfo["wan"] = array(
            "autopin" => $CfgStarter[0],
            "pin" => $CfgStarter[1],
            "keepalive" => $CfgStarter[2],
            "keepaddr" => $CfgStarter[3],
            "datalimit" => $CfgStarter[4],
            "planstartday" => $CfgStarter[5],
            "plansize1" => $CfgStarter[6],
            "plansize2" => $CfgStarter[7],
            "datablock" => $CfgStarter[8] );
        $ConfigInfo["ppp"] = array(
            "dialup" => $CfgStarter[9],
            "apnuser" => $CfgStarter[10],
            "apnpass" => $CfgStarter[11],
            "redialtime" => $CfgStarter[12],
            "authentication" => $CfgStarter[13],
            "roaming" => $CfgStarter[14] );
        $ConfigInfo["ddns"] = array(
            "mode" => $CfgStarter[15],
            "system" => $CfgStarter[16],
            "hostname" => $CfgStarter[17],
            "username" => $CfgStarter[18],
            "password" => $CfgStarter[19] );
        $ConfigInfo["ovpn"] = array(
            "mode" => $CfgStarter[20],
            "dir" => $CfgStarter[21],
            "tls" => $CfgStarter[22],
            "server" => $CfgStarter[23],
            "port" => $CfgStarter[24],
            "customconfig" => $CfgStarter[25],
            "certauth" => $CfgStarter[26],
            "clientcert" => $CfgStarter[27],
            "clientkey" => $CfgStarter[28],
            "daemon" => $CfgStarter[29],
            "verb" => $CfgStarter[30] );
        $ConfigInfo["ipsec"] = array(
            "mode" => $CfgStarter[31],
            "dir" => $CfgStarter[32],
            "IKEport" => $CfgStarter[33],
            "L2TPport" => $CfgStarter[34],
            "localid" => $CfgStarter[35],
            "remoteip" => $CfgStarter[36],
            "remoteid" => $CfgStarter[37],
            "remotesubnet" => $CfgStarter[38],
            "presharedkey" => base64_decode($CfgStarter[39]),
            "version" => $CfgStarter[40],
            "usrconn" => $CfgStarter[41],
            "remoteCIDRmask" => $CfgStarter[42],
            "dhcp" => $CfgStarter[43] );
        $ConfigInfo["serial"] = array(
            "bdrate" => $CfgStarter[44],
            "data" => $CfgStarter[45],
            "parity" => $CfgStarter[46],
            "stop" => $CfgStarter[47],
            "flow" => $CfgStarter[48],
            "mode" => $CfgStarter[49] );
        $ConfigInfo["encap"] = array(
            "mode" => $CfgStarter[50],
            "role" => $CfgStarter[51],
            "maxpkt" => $CfgStarter[52],
            "timeoutbits" => $CfgStarter[53],
            "listenport" => $CfgStarter[54],
            "porttype" => $CfgStarter[55],
            "modfilter" => $CfgStarter[56],
            "df1errmode" => $CfgStarter[57],
            "tablelen" => $CfgStarter[58] );
        $ConfigInfo["module"] = array(
            "name" => $CfgStarter[59] );
        $ConfigInfo["ntp"] = array(
            "enable" => $CfgStarter[60],
            "mode" => $CfgStarter[61],
            "server" => $CfgStarter[62] );
        $ConfigInfo["eip"] = array(
            "enable" => $CfgStarter[63] );

        $cfgEncapData = null;
        for ($i = 0; $i < $ConfigInfo["encap"]["tablelen"]; $i++) {
            if ($i != 0) {
                $cfgEncapData .= " ";
            }
            $cfgEncapData .= "encap/tableseraddr" . $i . " ";
            $cfgEncapData .= "encap/tableip" . $i . " ";
            $cfgEncapData .= "encap/tableport" . $i . " ";
            $cfgEncapData .= "encap/tableporttype" . $i;
        }

        unset($TblData);
        exec("/psft/bin/cfgModify readlist " . $cfgEncapData, $TblData);
        $ConfigInfo["encap"]["table"] = array();
        for ($i = 0; $i < $ConfigInfo["encap"]["tablelen"]; $i++) {
            $ConfigInfo["encap"]["table"][$i] = array(
                "seraddr" => $TblData[$i*4],
                "ip" => $TblData[$i*4+1],
                "port" => $TblData[$i*4+2],
                "porttype" => $TblData[$i*4+3]
            );
        }
    }
?>

<html>
    <head>
        <META HTTP-EQUIV="Pragma" CONTENT="no-cache"/>
        <META HTTP-EQUIV="Expires" CONTENT="-1"/>
        <link rel="stylesheet" href="/css/styles.css" type="text/css" />
        <script src="/scripts/jquery.min.js" type="text/JavaScript"></script>
        <script src="/scripts/angular.min.js" type="text/JavaScript"></script>
        <script src="/scripts/bootstrap.min.js" type="text/JavaScript"></script>
        <script src="/scripts/ng-file-upload.min.js" type="text/JavaScript"></script>
        <script src="/scripts/ng-file-upload-shim.min.js" type="text/JavaScript"></script>
        <script src="/js/application.js" type="text/JavaScript"></script>
        <script src="/js/main.js" type="text/JavaScript"></script>
        <title>Advanced Configuration</title>
    </head>
    <body leftmargin="0" topmargin="0" rightmargin="0" bottommargin="0" marginwidth="0" onload='parent.resizeIframe(document.body.scrollHeight)'>
        <?php /* Start Body */ ?>
        <table width=100% height="550" cellpadding="0" cellspacing="0" >
            <tr height=13 >
                <td width=14 >
                    <img src="/images/leftopangle.jpg" />
                </td>
                <td background="/images/bgtop.jpg" ></td>
                <td width=14 >
                    <img src="/images/rightopangle.jpg" />
                </td>
            </tr>
            <tr>
                <td background="/images/bgleft.jpg" >&nbsp;</td>
                <?php /* Start content here */ ?>
                <td align="center" valign="top">
                    <table width="98%" border="0" cellspacing="0" cellpadding="0" bgcolor="#002569">
                        <tr>
                            <td width="100%" bgcolor="#FFFFFF">
                                <form action="configuration_advance.php?action=1" name="advancedConfiguration" id="advancedConfiguration" method="post" enctype="multipart/form-data">
                                  <input type="hidden" name="csrf" value="<?php echo $_SESSION['csrf']; ?>" />
                                    <table width="100%" border="0" cellspacing="0" cellpadding="0">
                                        <tr>
                                            <td width="75%">
                                                <?php
                                                    if ($validate_err > 0) {
                                                        echo "<h3 style='color:#f04040'>The parameters highlighted in red are incorrect.  Please update with a valid value.</h3>";
                                                        for ($i = 0; $i < $validate_err; $i++) {
                                                            echo "<h4 style='color:#f04040'>" .$errMsgs[$i] . "</h4>";
                                                        }
                                                    }
                                                ?>
                                            </td>
                                        </tr>
                                    </table>
                                    <?php /* Body starts here */ ?>
                                    <table width="100%" border="0" cellspacing="0" cellpadding="0">
                                        <tr>
                                            <td>
                                                <table width="100%" border="0" cellpadding="4" cellspacing="0">
                                                    <tr>
                                                        <td height="4" colspan="4" align="left" bgcolor="#4d92a8"></td>
                                                    </tr>
                                                    <tr class="tablehead">
                                                        <td colspan="2" id="tdWANTitle"<?php if ($section_err["tdWANTitle"]) {echo ' style="color:red"';}?>>
                                                            <a onClick="ConfigAdvFolderUnFolder(false,0)" style="cursor:pointer">-WAN</a>
                                                        </td>
                                                    </tr>
                                                </table>
                                                <div id="divWANConfig" style="display:block">
                                                    <table width="100%" border="0" cellpadding="4" cellspacing="0">
                                                        <!-- <tr class="sorttable">
                                                            <td width="150">Auto PIN</td>
                                                            <td class="inputcol" width="200">
                                                                <select name="slt_AutoPin" class="copy" id="slt_AutoPin" onChange=" ShowHideConfigAdvanced('slt_AutoPin')">
                                                                    <option value=1 <?php if ("1" == $ConfigInfo["wan"]["autopin"]) echo "selected";?>>Enable</option>
                                                                    <option value=0 <?php if ("0" == $ConfigInfo["wan"]["autopin"]) echo "selected";?>>Disable</option>
                                                                </select>
                                                            </td>
                                                            <td>
                                                                <div></div>
                                                                <div id="divPIN" style="display: <?php if ($ConfigInfo["wan"]["autopin"] > 0) {echo "block";} else {echo "none";} ?>" width="100%">
                                                                    <table width="100%" border="0" cellpadding="0" cellspacing="0">
                                                                        <tr class="sorttable">
                                                                            <td width="50">PIN</td>
                                                                            <td class="inputcol">
                                                                                <input name="txt_wanpin" type="text" class="copy" id="txt_wanpin" value="<?php echo htmlspecialchars($ConfigInfo['wan']['pin']); ?>" size="10" maxlength="8"/>
                                                                        </tr>
                                                                    </table>
                                                                </div>
                                                            </td>
                                                        </tr> -->
                                                        <!-- <tr class="sorttablenobg">
                                                                <td>Dialup</td>
                                                                <td class="inputcol" colspan="2">
                                                                    <input name="txt_pppdialup" id="txt_pppdialup" type="text" class="copy" value="<?php echo htmlspecialchars($ConfigInfo['ppp']['dialup']); ?>" size="30" maxlength="64"/></td>
                                                            </tr> -->
                                                        <tr class="sorttablenobg">
                                                            <td width="150">User Name</td>
                                                            <td class="inputcol" colspan="2">
                                                                <!-- honeypot fields to avoid Mozilla Firefox autocomplete-->
                                                                <input type="text" style="display: none" id="fakeUsername" name="fakeUsername" class="copy" value="" size="30" maxlength="512"/>
                                                                <input type="password" style="display: none" id="fakePassword" name="fakePassword" class="copy" value="" size="30" maxlength="512"/>
                                                                <input name="txt_pppapnuser" id="txt_pppapnuser" type="text" class="copy" autocomplete="username" value="<?php echo htmlspecialchars($ConfigInfo['ppp']['apnuser']); ?>" size="30" maxlength="512"/>
                                                            </td>
                                                        </tr>
                                                        <tr class="sorttable">
                                                            <td>Password</td>
                                                            <td class="inputcol" colspan="2">
                                                                <input name="txt_ppppass" id="txt_ppppass" type="password" class="copy" autocomplete="new-password" value="<?php echo htmlspecialchars($ConfigInfo['ppp']['apnpass']); ?>" size="30" maxlength="512"/>
                                                            </td>
                                                        </tr>
                                                        <tr class="sorttablenobg">
                                                            <td>Wait Between Reconnect</td>
                                                            <td class="inputcol" colspan="2">
                                                                <input name="txt_pppredialtime" type="text" class="copy" id="txt_pppredialtime" value="<?php echo htmlspecialchars($ConfigInfo['ppp']['redialtime']); ?>" size="15" maxlength="512"/> Seconds</td>
                                                        </tr>
                                                        <tr class="sorttable">
                                                            <td>Authentication</td>
                                                            <td class="inputcol" colspan="2">
                                                                <select name="sel_wanauth" class="copy" id="sel_wanauth">
                                                                    <option value="0" <?php if ("0" == $ConfigInfo["ppp"]["authentication"]) echo "selected";?>>CHAP</option>
                                                                    <option value="1" <?php if ("1" == $ConfigInfo["ppp"]["authentication"]) echo "selected";?>>PAP</option>
                                                                    <option value="2" <?php if ("2" == $ConfigInfo["ppp"]["authentication"]) echo "selected";?>>PAP&amp;CHAP</option>
                                                                    <option value="3" <?php if ("3" == $ConfigInfo["ppp"]["authentication"]) echo "selected";?>>None</option>
                                                                </select>
                                                            </td>
                                                        </tr>
                                                        <tr class="sorttablenobg">
                                                            <td>Roaming</td>
                                                            <td class="inputcol" colspan="2">
                                                                <select name="slt_wanroaming" class="copy" id="slt_wanroaming">
                                                                    <option value="0" <?php if ("0" == $ConfigInfo["ppp"]["roaming"]) echo "selected";?>>Disable</option>
                                                                    <option value="1" <?php if ("1" == $ConfigInfo["ppp"]["roaming"]) echo "selected";?>>Enable</option>
                                                                </select>
                                                            </td>
                                                        </tr>
                                                        <tr class="sorttable">
                                                            <td>Keep Alive</td>
                                                            <td class="inputcol" colspan="2">
                                                                <select name="slt_wankeepalive" class="copy" id="slt_wankeepalive" onChange="ShowHideConfigAdvanced('slt_wankeepalive')">
                                                                    <option value="0" <?php if ("0" == $ConfigInfo["wan"]["keepalive"]) echo "selected";?>>Disable</option>
                                                                    <option value="1" <?php if ("0" < $ConfigInfo["wan"]["keepalive"]) echo "selected";?>>Enable</option>
                                                                </select>
                                                            </td>
                                                        </tr>
                                                    </table>
                                                    <div id="divKeepAlive" style="display: <?php if ($ConfigInfo["wan"]["keepalive"] > 0) {echo "block";} else {echo "none";} ?>">
                                                        <table width="100%" border="0" cellpadding="0" cellspacing="0">
                                                            <tr class="sorttablenobg">
                                                                <td width="150">Keep Alive Ping Address</td>
                                                                <td class="inputcol">
                                                                    <input name="txt_wankeepaddr" type="text" class="copy" id="txt_wankeepaddr" value="<?php echo htmlspecialchars($ConfigInfo['wan']['keepaddr']);?>" size="30" maxlength="511"/>
                                                                </td>
                                                            </tr>
                                                            <tr class="sorttable">
                                                                <td>Keep Alive Ping Period</td>
                                                                <td class="inputcol">
                                                                    <input name="txt_wankeepalive" type="text" class="copy" id="txt_wankeepalive" value="<?php if ($ConfigInfo['wan']['keepalive'] == 0) {echo "30";} else {echo htmlspecialchars($ConfigInfo['wan']['keepalive']);} ?>" size="10" maxlength="10"/>Seconds</td>
                                                            </tr>
                                                        </table>
                                                    </div>
                                                </div>
                                            </td>
                                        </tr>
                                        <tr>
                                            <td height="4" colspan="4" align="left" bgcolor="#4d92a8"></td>
                                        </tr>
                                        <tr>
                                            <td>
                                                <table width="100%" border="0" cellpadding="4" cellspacing="0">
                                                    <tr class="tablehead">
                                                        <td colspan="2" id="tdCellUsage"<?php if ($section_err["tdCellUsage"]) {echo ' style="color:red"';}?>>
                                                            <a onClick="ConfigAdvFolderUnFolder(true,4)" style="cursor:pointer">+Cellular Usage Tracking</a>
                                                        </td>
                                                    </tr>
                                                    <tr></tr>
                                                </table>
                                                <div id="divCellUsage" style="display:none">
                                                    <table width="100%" border="0" cellpadding="4" cellspacing="0">
                                                        <tr class="sorttable">
                                                            <td width="150">Data Plan Limit</td>
                                                            <td class="inputcol">
                                                                <select name="slt_wandataplanlimit" class="copy" id="slt_wandataplanlimit">
                                                                    <option value="0" <?php if ("0" == $ConfigInfo["wan"]["datalimit"]) echo "selected";?>>Disable</option>
                                                                    <option value="1" <?php if ("1" == $ConfigInfo["wan"]["datalimit"]) echo "selected";?>>Enable</option>
                                                                </select>
                                                            </td>
                                                        </tr>
                                                        <tr class="sorttablenobg">
                                                            <td>Plan Start Day</td>
                                                            <td class="inputcol">
                                                                <input name="txt_planstartday" id="txt_planstartday" type="text" class="copy" value="<?php echo htmlspecialchars($ConfigInfo['wan']['planstartday']); ?>" size="15" maxlength="30"/>
                                                            </td>
                                                        </tr>
                                                        <tr class="sorttable">
                                                            <td>Plan Size</td>
                                                            <td class="inputcol">
                                                                <input name="txt_wanplansize" id="txt_wanplansize" type="text" class="copy" value="<?php echo htmlspecialchars($ConfigInfo['wan']['plansize2']); ?>" size="15" maxlength="30"/>
                                                                <select name="txt_wanplansize_u" class="copy" id="txt_wanplansize_u">
                                                                    <!--  <option value="0" <?php if ("0" == $ConfigInfo["wan"]["plansize1"]) echo "selected";?>>KB</option> -->
                                                                    <option value="1" <?php if ("1" == $ConfigInfo["wan"]["plansize1"]) echo "selected";?>>MB</option>
                                                                    <option value="2" <?php if ("2" == $ConfigInfo["wan"]["plansize1"]) echo "selected";?>>GB</option>
                                                                    <option value="3" <?php if ("3" == $ConfigInfo["wan"]["plansize1"]) echo "selected";?>>Unlimited</option>
                                                                </select>
                                                            </td>
                                                        </tr>
                                                        <tr class="sorttablenobg">
                                                            <td>Stop Data After Plan Limit Reached</td>
                                                            <td class="inputcol">
                                                                <select name="slt_wandatablock" class="copy" id="slt_wandatablock">
                                                                    <option value="1" <?php if ("1" == $ConfigInfo["wan"]["datablock"]) echo "selected";?>>Enable</option>
                                                                    <option value="0" <?php if ("0" == $ConfigInfo["wan"]["datablock"]) echo "selected";?>>Disable</option>
                                                                </select>
                                                            </td>
                                                        </tr>
                                                    </table>
                                                </div>
                                            </td>
                                        </tr>
                                        <tr>
                                            <td height="4" colspan="4" align="left" bgcolor="#4d92a8"></td>
                                        </tr>
                                        <tr>
                                            <td>
                                                <table width="100%" border="0" cellpadding="4" cellspacing="0">
                                                    <tr class="tablehead">
                                                        <td colspan="2" id="tdDDNSTitle"<?php if ($section_err["tdDDNSTitle"]) {echo ' style="color:red"';}?>>
                                                            <a onClick="ConfigAdvFolderUnFolder(true, 1)" style="cursor:pointer">+DDNS</a>
                                                        </td>
                                                    </tr>
                                                    <tr></tr>
                                                </table>
                                                <div id="divDDNSConfig" style="display:none">
                                                    <table width="100%" border="0" cellpadding="4" cellspacing="0">
                                                        <tr class="sorttable">
                                                            <td width="150">Active</td>
                                                            <td class="inputcol">
                                                                <select name="slt_DDNSActive" class="copy" id="slt_DDNSActive" onchange="ddnsServer()">
                                                                    <option value="0" <?php if ("0" == $ConfigInfo["ddns"]["mode"]) echo "selected";?>>Disable</option>
                                                                    <option value="1" <?php if ("1" == $ConfigInfo["ddns"]["mode"]) echo "selected";?>>DynDNS.org</option>
                                                                    <option value="2" <?php if ("2" == $ConfigInfo["ddns"]["mode"]) echo "selected";?>>No-IP.com</option>
                                                                </select>
                                                            </td>
                                                        </tr>
                                                        <tr class="sorttablenobg">
                                                            <td>DDNS Server</td>
                                                            <td class="inputcol">
                                                                <input name="txt_DDNSServer" type="text" class="copy" id="txt_DDNSServer" value="<?php echo htmlspecialchars($ConfigInfo['ddns']['system']); ?>" size="40" maxlength="64"/>
                                                            </td>
                                                        </tr>
                                                        <tr class="sorttable">
                                                            <td>ICX35 Domain Name</td>
                                                            <td class="inputcol">
                                                                <input name="txt_DDNSDomainName" id="txt_DDNSDomainName" type="text" class="copy" value="<?php echo htmlspecialchars($ConfigInfo['ddns']['hostname']); ?>" size="40" maxlength="64"/>
                                                            </td>
                                                        </tr>
                                                        <tr class="sorttablenobg">
                                                            <td>User</td>
                                                            <td class="inputcol">
                                                                <input name="txt_DDNSUserName" id="txt_DDNSUserName" type="text" class="copy" value="<?php echo htmlspecialchars($ConfigInfo['ddns']['username']); ?>" size="30" maxlength="50"/>
                                                            </td>
                                                        </tr>
                                                        <tr class="sorttable">
                                                            <td>Password</td>
                                                            <td class="inputcol">
                                                                <input name="txt_DDNSPass" id="txt_DDNSPass" type="password" class="copy" value="<?php echo htmlspecialchars($ConfigInfo['ddns']['password']);?>" size="30" maxlength="50"/>
                                                            </td>
                                                        </tr>
                                                    </table>
                                                </div>
                                            </td>
                                        </tr>
                                        <tr>
                                            <td height="4" colspan="4" align="left" bgcolor="#4d92a8"></td>
                                        </tr>
                                        <tr>
                                            <td>
                                                <table width="100%" border="0" cellpadding="4" cellspacing="0">
                                                    <tr class="tablehead">
                                                        <td colspan="2" id="tdVPNTitle"<?php if ($section_err["tdVPNTitle"]) {echo ' style="color:red"';}?>>
                                                            <a onClick="ConfigAdvFolderUnFolder(true,2)" style="cursor:pointer">+VPN</a>
                                                        </td>
                                                    </tr>
                                                </table>
                                                <div id="divVPNConfig" style="display:none">
                                                    <table width="100%" border="0" cellpadding="4" cellspacing="0">
                                                        <tr class="sorttable">
                                                            <td width="150">&nbsp;Client</td>
                                                            <td class="inputcol">
                                                                <select name="sltVPNType" class="copy" id="sltVPNType" onChange="ShowHideConfigAdvanced('sltVPNType')">
                                                                    <option value="0" <?php if ("0" == $ConfigInfo["ovpn"]["mode"] && "0" == $ConfigInfo["ipsec"]["mode"]) echo "selected";?>>Disable</option>
                                                                    <option value="1" <?php if ("5" == $ConfigInfo["ovpn"]["mode"]) echo "selected";?>>OpenVPN</option>
                                                                    <option value="2" <?php if ("0" != $ConfigInfo["ipsec"]["mode"]) echo "selected";?>>IPSec</option>
                                                                </select>
                                                            </td>
                                                        </tr>
                                                        <tr>
                                                            <td colspan="2" >
                                                                <div id="divVPNConfig_OpenVPN" style="display:<?php if ("5" == $ConfigInfo["ovpn"]["mode"]) echo "block"; else echo "none";?>">
                                                                    <table border="0" cellspacing="0" cellpadding="0" width="100%">
                                                                        <tr>
                                                                            <td height="25" colspan="5" bgcolor="#3399CC" class="STYLE1">
                                                                                <b>OpenVPN</b>
                                                                            </td>
                                                                        </tr>
                                                                        <tr class="sorttable">
                                                                            <td width="150">TLS Renegotiation Time</td>
                                                                            <td class="inputcol" colspan="4">
                                                                                <input name="txt_OpenVPN_TLS" id="txt_OpenVPN_TLS" type="text" class="copy" value="<?php echo htmlspecialchars($ConfigInfo["ovpn"]["tls"]);?>" size="15" maxlength="512"/> Seconds
                                                                            </td>
                                                                        </tr>
                                                                        <tr class="sorttablenobg">
                                                                            <!-- <td>Server IP / Domain</td> -->
                                                                            <td>Server IP</td>
                                                                            <td class="inputcol" colspan="4">
                                                                                <input name="txt_OpenVPN_ServerIP" id="txt_OpenVPN_ServerIP" type="text" value="<?php echo htmlspecialchars($ConfigInfo['ovpn']['server']); ?>" class="copy"  size="25" maxlength="512"/>
                                                                            </td>
                                                                        </tr>
                                                                        <tr class="sorttable">
                                                                            <td>Server Port</td>
                                                                            <td class="inputcol" colspan="4">
                                                                                <input name="txt_OpenVPN_ServerPort" id="txt_OpenVPN_ServerPort" type="text" value="<?php echo htmlspecialchars($ConfigInfo['ovpn']['port']); ?>" class="copy" size="15" maxlength="512"/>
                                                                            </td>
                                                                        </tr>
                                                                        <tr class="sorttablenobg">
                                                                            <td colspan="5">
                                                                                <table border="0" cellspacing="0" cellpadding="0" width="100%">
                                                                                    <tr>
                                                                                        <td colspan="4">
                                                                                            <b>Credential Files</b>
                                                                                        </td>
                                                                                    </tr>
                                                                                    <tr>
                                                                                        <td width="25"></td>
                                                                                        <td width="125">Certificate Authority </td>
                                                                                        <td width="200" id="caCurrentFile">Current File:  <?php echo $ConfigInfo["ovpn"]["certauth"];?> </td>
                                                                                        <td class="inputcol">
                                                                                            <input name="file_OpenVPN_CA" id="file_OpenVPN_CA" class="copy" size="25" type="file">
                                                                                        </td>
                                                                                    </tr>
                                                                                    <tr>
                                                                                        <td/>
                                                                                        <td>Client Certificate</td>
                                                                                        <td id="ccCurrentFile">Current File:  <?php echo $ConfigInfo["ovpn"]["clientcert"];?> </td>
                                                                                        <td class="inputcol">
                                                                                            <input name="file_OpenVPN_CC" id="file_OpenVPN_CC" class="copy" size="25" type="file">
                                                                                        </td>
                                                                                    </tr>
                                                                                    <tr>
                                                                                        <td/>
                                                                                        <td>Client Key</td>
                                                                                        <td id="ckCurrentFile">Current File:  <?php echo $ConfigInfo["ovpn"]["clientkey"];?> </td>
                                                                                        <td class="inputcol">
                                                                                            <input name="file_OpenVPN_CK" id="file_OpenVPN_CK" class="copy" size="25" type="file">
                                                                                        </td>
                                                                                    </tr>
                                                                                    <!-- <tr>
                                                                                            <td/>
                                                                                            <td>
                                                                                                <input name="btn_OVPNUploadCred" id="btn_OVPNUploadCred" onclick="ConfigAdvVpnFiles('Credential');" value="Upload Files" class="copy" type="button">
                                                                                            </td>
                                                                                        </tr> -->
                                                                                </table>
                                                                            </td>
                                                                        </tr>
                                                                        <tr class="sorttable">
                                                                            <td>Custom Config File</td>
                                                                            <td width="200">Current File:  <?php echo $ConfigInfo["ovpn"]["customconfig"];?> </td>
                                                                            <td class="inputcol" width="260">
                                                                                <input name="file_OpenVPN_conf" id="file_OpenVPN_conf" class="copy" size="25" type="file">
                                                                            </td>
                                                                            <!-- <td width="100"><input name="btn_OVPNUploadConf" id="btn_OVPNUploadConf" onclick="ConfigAdvVpnFiles('Config');" value="Upload" class="copy" type="button"> </td> -->
                                                                            <td>
                                                                                <input name="btn_OVPNDelConf" id="btn_OVPNDelConf" onclick="ConfigAdvVpnFiles('DelConfig');" value="Delete" class="copy" type="button">
                                                                            </td>
                                                                        </tr>
                                                                    </table>
                                                                </div>
                                                                <div id="divVPNConfig_ipsec" style="display:<?php if ("0" != $ConfigInfo["ipsec"]["mode"]) echo "block"; else echo "none";?>">
                                                                    <table border="0" cellspacing="0" cellpadding="0" width="100%">
                                                                        <tr>
                                                                            <td height="25" colspan="4" bgcolor="#3399CC" class="STYLE1">
                                                                                <b>IPSec</b>
                                                                            </td>
                                                                        </tr>
                                                                        <tr>
                                                                            <td width="150">Local Identifier</td>
                                                                            <td width="260" class="inputcol">
                                                                                <input name="txt_ipsec_localid" id="txt_ipsec_localid" type="text" class="copy" value="<?php if ($ConfigInfo['ipsec']['localid'] == "") {echo htmlspecialchars($ConfigInfo['module']['name']);} else {echo htmlspecialchars($ConfigInfo['ipsec']['localid']);}?>" size="15" maxlength="50"/>
                                                                            </td>
                                                                        </tr>
                                                                        <tr>
                                                                            <td>Remote IP</td>
                                                                            <td class="inputcol">
                                                                                <input name="txt_ipsec_remoteip" id="txt_ipsec_remoteip" type="text" class="copy" value="<?php echo htmlspecialchars($ConfigInfo['ipsec']['remoteip']); ?>" size="30" maxlength="512"/>
                                                                            </td>
                                                                            <td width="150">Remote Subnet</td>
                                                                            <td class="inputcol">
                                                                                <input name="txt_ipsec_remotesubnet" id="txt_ipsec_remotesubnet" type="text" class="copy" value="<?php echo htmlspecialchars($ConfigInfo['ipsec']['remotesubnet']); ?>" size="30" maxlength="512"/> / <input name="txt_ipsec_remoteCIDRmask" id="txt_ipsec_remoteCIDRmask" type="text" class="copy" value="<?php echo htmlspecialchars($ConfigInfo['ipsec']['remoteCIDRmask']); ?>" size="3" maxlength="10"/>
                                                                            </td>
                                                                        </tr>
                                                                        <tr>
                                                                            <td>Remote Identifier</td>
                                                                            <td class="inputcol">
                                                                                <input name="txt_ipsec_remoteid" id="txt_ipsec_remoteid" type="text" class="copy" value="<?php echo htmlspecialchars($ConfigInfo['ipsec']['remoteid']); ?>" size="30" maxlength="512"/>
                                                                            </td>
                                                                            <td>Pre-shared Key</td>
                                                                            <td class="inputcol">
                                                                                <input name="txt_ipsec_presharedkey" id="txt_ipsec_presharedkey" type="password" class="copy" value="<?php echo htmlspecialchars($ConfigInfo['ipsec']['presharedkey']); ?>" size="30" maxlength="50"/>
                                                                            </td>
                                                                        </tr>
                                                                    </table>
                                                                </div>
                                                            </td>
                                                        </tr>
                                                    </table>
                                                </div>
                                            </td>
                                        </tr>
                                        <tr>
                                            <td height="4" colspan="4" align="left" bgcolor="#4d92a8"></td>
                                        </tr>
                                        <!-- Workaround: TEG17308 - Serial Encapsulation DF1 Ful-Duplex, UDP port type data transfer failure -->
                                        <tr <?php if("0" != $ConfigInfo["ovpn"]["mode"]) echo "style='display: none'"; ?>>
                                            <td>
                                                <table width="100%" border="0" cellpadding="4" cellspacing="0">
                                                    <tr class="tablehead">
                                                        <td colspan="2" id="tdSerialTitle"<?php if ($section_err["tdSerialTitle"]) {echo ' style="color:red"';}?>>
                                                            <a onClick="ConfigAdvFolderUnFolder(true,3)" style="cursor:pointer">+Serial / Encapsulation</a>
                                                        </td>
                                                    </tr>
                                                </table>
                                                <div id="divSerialConfig" style="display:none">
                                                    <table width="100%" border="0" cellpadding="4" cellspacing="0">
                                                        <tr class="sorttable">
                                                            <td width="150">Port Mode</td>
                                                            <td class="inputcol">
                                                                <select name="slt_SL_PT" class="copy" id="slt_SL_PT" onchange="portMode();">
                                                                    <option value="0" <?php if ("0" == $ConfigInfo["serial"]["mode"]) echo "selected";?>>RS232</option>
                                                                    <option value="2" <?php if ("2" == $ConfigInfo["serial"]["mode"]) echo "selected";?>>RS485</option>
                                                                    <option value="1" <?php if ("1" == $ConfigInfo["serial"]["mode"]) echo "selected";?>>RS422</option>
                                                                </select>
                                                            </td>
                                                            <tr class="sorttablenobg">
                                                                <td>Baud Rate</td>
                                                                <td class="inputcol">
                                                                    <select name="slt_SL_BR" class="copy" id="slt_SL_BR" onchange="baudrateChangeSetTimeoutBits()">
                                                                        <option value="0" <?php if ("0" == $ConfigInfo["serial"]["bdrate"]) echo "selected";?>>110</option>
                                                                        <option value="1" <?php if ("1" == $ConfigInfo["serial"]["bdrate"]) echo "selected";?>>150</option>
                                                                        <option value="2" <?php if ("2" == $ConfigInfo["serial"]["bdrate"]) echo "selected";?>>300</option>
                                                                        <option value="3" <?php if ("3" == $ConfigInfo["serial"]["bdrate"]) echo "selected";?>>600</option>
                                                                        <option value="4" <?php if ("4" == $ConfigInfo["serial"]["bdrate"]) echo "selected";?>>1200</option>
                                                                        <option value="5" <?php if ("5" == $ConfigInfo["serial"]["bdrate"]) echo "selected";?>>2400</option>
                                                                        <option value="6" <?php if ("6" == $ConfigInfo["serial"]["bdrate"]) echo "selected";?>>4800</option>
                                                                        <option value="7" <?php if ("7" == $ConfigInfo["serial"]["bdrate"]) echo "selected";?>>9600</option>
                                                                        <option value="8" <?php if ("8" == $ConfigInfo["serial"]["bdrate"]) echo "selected";?>>19200</option>
                                                                        <option value="9" <?php if ("9" == $ConfigInfo["serial"]["bdrate"]) echo "selected";?>>28800</option>
                                                                        <option value="10" <?php if ("10" == $ConfigInfo["serial"]["bdrate"]) echo "selected";?>>38400</option>
                                                                        <option value="11" <?php if ("11" == $ConfigInfo["serial"]["bdrate"]) echo "selected";?>>57600</option>
                                                                        <option value="12" <?php if ("12" == $ConfigInfo["serial"]["bdrate"]) echo "selected";?>>115200</option>
                                                                        <option value="13" <?php if ("13" == $ConfigInfo["serial"]["bdrate"]) echo "selected";?>>230400</option>
                                                                    </select>
                                                                </td>
                                                            </tr>
                                                            <tr class="sorttable">
                                                                <td>Data Bits</td>
                                                                <td class="inputcol">
                                                                    <select name="slt_SL_DB" class="copy" id="slt_SL_DB">
                                                                        <!-- <option value="0" <?php if ("0" == $ConfigInfo["serial"]["data"]) echo "selected";?>>5 Bits</option> -->
                                                                        <!-- <option value="1" <?php if ("1" == $ConfigInfo["serial"]["data"]) echo "selected";?>>6 Bits</option> -->
                                                                        <option value="2" <?php if ("2" == $ConfigInfo["serial"]["data"]) echo "selected";?>>7 Bits</option>
                                                                        <option value="3" <?php if ("3" == $ConfigInfo["serial"]["data"]) echo "selected";?>>8 Bits</option>
                                                                    </select>
                                                                </td>
                                                            </tr>
                                                            <tr class="sorttablenobg">
                                                                <td>Parity</td>
                                                                <td class="inputcol">
                                                                    <select name="slt_SL_Parity" class="copy" id="slt_SL_Parity">
                                                                        <option value="0" <?php if ("0" == $ConfigInfo["serial"]["parity"]) echo "selected";?>>None</option>
                                                                        <option value="1" <?php if ("1" == $ConfigInfo["serial"]["parity"]) echo "selected";?>>Odd</option>
                                                                        <option value="2" <?php if ("2" == $ConfigInfo["serial"]["parity"]) echo "selected";?>>Even</option>
                                                                        <option value="3" <?php if ("3" == $ConfigInfo["serial"]["parity"]) echo "selected";?>>Mark </option>
                                                                        <option value="4" <?php if ("4" == $ConfigInfo["serial"]["parity"]) echo "selected";?>>Space </option>
                                                                    </select>
                                                                </td>
                                                            </tr>
                                                            <tr class="sorttable">
                                                                <td>Stop Bits</td>
                                                                <td class="inputcol">
                                                                    <select name="slt_SL_STB" class="copy" id="slt_SL_STB">
                                                                        <option value="0" <?php if ("0" == $ConfigInfo["serial"]["stop"]) echo "selected";?>>1 Bits</option>
                                                                        <option value="1" <?php if ("1" == $ConfigInfo["serial"]["stop"]) echo "selected";?>>2 Bits</option>
                                                                    </select>
                                                                </td>
                                                            </tr>
                                                            <!-- <tr <?php if ("0" == $ConfigInfo["serial"]["mode"]) {
                                                                        echo 'class="sorttablenobg"';
                                                                    } else {
                                                                        echo 'style="display: none"';
                                                                    } ?>>
                                                                <td>Flow Control</td>
                                                                <td class="inputcol">
                                                                    <select name="slt_SL_FC" class="copy" id="slt_SL_FC">
                                                                        <option value="0" <?php if ("0" == $ConfigInfo["serial"]["flow"]) echo "selected";?>>None</option>
                                                                        <option value="1" <?php if ("1" == $ConfigInfo["serial"]["flow"]) echo "selected";?>>RTS/CTS</option>
                                                                        <option value="2" <?php if ("2" == $ConfigInfo["serial"]["flow"]) echo "selected";?>>XON/XOFF</option>
                                                                    </select>
                                                                </td>
                                                            </tr>
                                                            <tr <?php if ("0" == $ConfigInfo["serial"]["mode"]) {
                                                                        echo 'class="sorttable"';
                                                                    } else {
                                                                        echo 'class="sorttablenobg"';
                                                                    } ?>> -->
                                                            <tr class="sorttablenobg">
                                                                <td>Protocol</td>
                                                                <td class="inputcol">
                                                                    <select name="slt_SL_Protocol" class="copy" id="slt_SL_Protocol" onChange="ShowHideConfigAdvanced('slt_SL_Protocol')">
                                                                        <option value="0" <?php if ("0" == $ConfigInfo["encap"]["mode"]) echo "selected";?>>Disabled</option>
                                                                        <option value="1" <?php if ("1" == $ConfigInfo["encap"]["mode"]) echo "selected";?>>Generic</option>
                                                                        <option value="2" <?php if ("2" == $ConfigInfo["encap"]["mode"]) echo "selected";?>>Modbus RTU</option>
                                                                        <option value="3" <?php if ("3" == $ConfigInfo["encap"]["mode"]) echo "selected";?>>Modbus ASCII</option>
                                                                        <option value="4" <?php if ("4" == $ConfigInfo["encap"]["mode"]) echo "selected";?>>Modbus RTU to TCP</option>
                                                                        <option value="5" <?php if ("5" == $ConfigInfo["encap"]["mode"]) echo "selected";?>>Modbus ASCII to TCP</option>
                                                                        <option value="6" <?php if ("6" == $ConfigInfo["encap"]["mode"]) echo "selected"; if ("2" == $ConfigInfo["serial"]["mode"]) echo "style='display: none;'";?>>DF1 Half-Duplex</option>
                                                                        <option value="7" <?php if ("7" == $ConfigInfo["encap"]["mode"]) echo "selected"; if ("2" == $ConfigInfo["serial"]["mode"]) echo "style='display: none;'";?>>DF1 Full-Duplex</option>
                                                                        <option value="8" <?php if ("8" == $ConfigInfo["encap"]["mode"]) echo "selected"; if ("2" == $ConfigInfo["serial"]["mode"]) echo "style='display: none;'";?>>DF1 Radio Modem</option>
                                                                        <!-- <option value="7" <?php if ("7" == $ConfigInfo["encap"]["mode"]) echo "selected"; if ("RS485" == trim($_POST['slt_SL_PT'])) echo "style='display: none;'";?>>DF1 Full-Duplex</option> -->
                                                                        <!-- <option value="8" <?php if ("8" == $ConfigInfo["encap"]["mode"]) echo "selected"; if ("RS485" == trim($_POST['slt_SL_PT'])) echo "style='display: none;'";?>>DF1 Radio Modem</option> -->
                                                                    </select>
                                                                </td>
                                                            </tr>
                                                    </table>
                                                    <table width="100%" border="0" cellpadding="4" cellspacing="0">
                                                        <tr>
                                                            <td>
                                                                <div id="divSerialConfig_All" style="display:none">
                                                                    <table border="0" cellspacing="0" cellpadding="0" width="100%">
                                                                        <tr>
                                                                            <td height="25" colspan="4" bgcolor="#3399CC" class="STYLE1" id="tdSerialSubTitle">&nbsp;</td>
                                                                        </tr>
                                                                        <tr>
                                                                            <td width="150">Role</td>
                                                                            <td class="inputcol">
                                                                                <select name="slt_gen_role" class="copy" id="slt_gen_role" onChange="ShowHideConfigAdvanced('slt_gen_role')">
                                                                                    <option value="0" <?php if ("0" == $ConfigInfo["encap"]["role"]) echo "selected";?>>Master</option>
                                                                                    <option value="1" <?php if ("1" == $ConfigInfo["encap"]["role"]) echo "selected";?>>Slave</option>
                                                                                    <?php /* Remove the Master/Slave role, as it was determined to be not needed. May be useful for future release. */ ?>
                                                                                    <!-- <option value="2" <?php if ("2" == $ConfigInfo["encap"]["role"]) echo "selected";?>>Master/Slave</option> -->
                                                                                </select>
                                                                            </td>
                                                                        </tr>
                                                                        <tr>
                                                                            <td>Timeout Bits</td>
                                                                            <td class="inputcol">
                                                                                <input name="text_gen_timeout" id="text_gen_timeout" type="text" class="copy" value="<?php echo htmlspecialchars($ConfigInfo['encap']['timeoutbits']); ?>" size="15" maxlength="512"/>
                                                                            </td>
                                                                            <td width="150" style="display: none;">Max Packet Size</td>
                                                                            <td class="inputcol" style="display: none;">
                                                                                <input name="txt_gen_maxpacket" id="txt_gen_maxpacket" type="text" class="copy" value="<?php echo htmlspecialchars($ConfigInfo['encap']['maxpkt']); ?>" size="15" maxlength="512"/>
                                                                            </td>
                                                                        </tr>
                                                                        <tr>
                                                                            <td width="150">Port Type</td>
                                                                            <td class="inputcol">
                                                                                <select name="txt_sl_server_pt" class="copy" id="txt_sl_server_pt">
                                                                                    <option value="0" <?php if ("0" == $ConfigInfo["encap"]["porttype"]) echo "selected";?>>TCP</option>
                                                                                    <option value="1" id="txt_sl_server_ptUDP" <?php if ("1" == $ConfigInfo["encap"]["porttype"]) echo "selected";?>>UDP</option>
                                                                                </select>
                                                                            </td>
                                                                            <!-- <td id="lbl_sl_server_lp" width="150">Listen Port</td>
                                                                            <td width="260" class="inputcol"><input name="txt_sl_server_lp" id="txt_sl_server_lp" type="text" class="copy" value="<?php echo htmlspecialchars($ConfigInfo['encap']['listenport']); ?>" size="15" maxlength="512"/></td> -->
                                                                        </tr>
                                                                    </table>
                                                                    <div id="divSerialConfig_Modbus" style="display:none">
                                                                        <table border="0" cellspacing="0" cellpadding="0" width="100%">
                                                                            <tr>
                                                                                <td width="150">Modbus Filter </td>
                                                                                <td class="inputcol">
                                                                                    <select name="slt_mbs_filter" class="copy" id="slt_mbs_filter">
                                                                                        <option value="0" <?php if ("0" == $ConfigInfo["encap"]["modfilter"]) echo "selected";?>>Disable</option>
                                                                                        <option value="1" <?php if ("1" == $ConfigInfo["encap"]["modfilter"]) echo "selected";?>>Enable</option>
                                                                                    </select>
                                                                                </td>
                                                                            </tr>
                                                                        </table>
                                                                    </div>
                                                                    <div id="divSerialConfig_DF1" style="display:none">
                                                                        <table border="0" cellspacing="0" cellpadding="0" width="100%">
                                                                            <tr>
                                                                                <td width="150">DF1 Error Checking </td>
                                                                                <td class="inputcol" align="left">
                                                                                    <select name="slt_df1_ex" class="copy" id="slt_df1_ex">
                                                                                        <option value="1" <?php if ("1" == $ConfigInfo["encap"]["df1errmode"]) echo "selected";?>>CRC</option>
                                                                                        <option value="0" <?php if ("0" == $ConfigInfo["encap"]["df1errmode"]) echo "selected";?>>BCC</option>
                                                                                    </select>
                                                                                </td>
                                                                            </tr>
                                                                        </table>
                                                                    </div>
                                                                    <div id="divSerialConfig_GenClient" style="display:block">
                                                                        <table border="0" cellspacing="0" cellpadding="0" width="100%">
                                                                            <tr>
                                                                                <!-- <td width="150">Remote IP / Domain </td> -->
                                                                                <td width="150">Remote IP</td>
                                                                                <td class="inputcol">
                                                                                    <input name="txt_genclient_addr" id="txt_genclient_addr" type="text" class="copy" value="<?php echo htmlspecialchars($ConfigInfo['encap']['table'][0]['ip']);?>" size="15" maxlength="512"></td>
                                                                            </tr>
                                                                        </table>
                                                                    </div>
                                                                    <div id="divSerialConfig_ClientSettings" style="display:none">
                                                                        <table id="tbl_encapclient" border="0" cellspacing="0" cellpadding="0" width="100%">
                                                                            <tr>
                                                                                <td height="25" colspan="7" bgcolor="#3399CC" class="STYLE1" id="tdEncapTable">
                                                                                    <b>Encapsulation Table</b>
                                                                                </td>
                                                                            </tr>
                                                                            <?php
                                                                                for ($i = 0; $i < $ConfigInfo["encap"]["tablelen"]; $i++) {
                                                                                    echo '<tr class="';
                                                                                    if ($i%2 == 1) {
                                                                                        echo 'sorttablenobg';
                                                                                    } else {
                                                                                        echo 'sorttable';
                                                                                    }
                                                                                    echo '"><td width="25" class="inputcol"><input name = "ck_encap_del[' . $i . ']" id="ck_encap_del[' . $i . ']" type="checkbox" class="copy"/></td> ';
                                                                                    echo '<td width="90">Serial Address</td>';
                                                                                    echo '<td width="260" class="inputcol"><input name="txt_encap_seraddr[' . $i . ']" id="txt_encap_seraddr[' . $i . ']" type="text" class="copy" value="' . htmlspecialchars($ConfigInfo["encap"]["table"][$i]["seraddr"]) . '" size="5" maxlength="10"/></td>';
                                                                                    // echo '<td width="150">IP / Domain </td>';
                                                                                    echo '<td width="100">IP Address</td>';
                                                                                    echo '<td class="inputcol"><input name="txt_encap_ipaddr[' . $i . ']" id="txt_encap_ipaddr[' . $i . ']" type="text" class="copy" value="' . htmlspecialchars($ConfigInfo["encap"]["table"][$i]["ip"]) . '" size="15" maxlength="512"/></td>';
                                                                                    // echo '<td width="150">Port </td>';
                                                                                    // echo '<td class="inputcol"><input name="txt_encap_port[' . $i . ']" id="txt_encap_port[' . $i . ']" type="text" class="copy" value="' . htmlspecialchars($ConfigInfo["encap"]["table"][$i]["port"]) . '" size="15" maxlength="512"/></td>';
                                                                                }
                                                                            ?>
                                                                        </table>
                                                                    <div>
                                                                        <input name="serial_addtolist" type="button" id="serial_addtolist" value="Add Entry to Client Table List" onClick="AddEncapTableEntry();" class="tablehead"/>
                                                                        <input name="serial_removefromlist" type="button" id="serial_removefromlist" value="Remove Selected Entres" onClick="RemoveEncapTableEntry();" class="tablehead"/>
                                                                    </div>
                                                                </div>
                                                                <div id="hidTableLen" style="display:none">
                                                                    <input name="encapTableLen" type="text" id="encapTableLen" value="<?php echo htmlspecialchars($ConfigInfo["encap"]["tablelen"]); ?>">
                                                                </div>
                                                            </td>
                                                        </tr>
                                                    </table>
                                                </div>
                                            </td>
                                        </tr>
                                        <!-- Workaround: TEG17308 - Serial Encapsulation DF1 Ful-Duplex, UDP port type data transfer failure -->
                                        <tr <?php if("0" != $ConfigInfo["ovpn"]["mode"]) echo "style='display: none'"; ?>>
                                            <td id= "tdSerialSeparator" height="4" colspan="4" align="left" bgcolor="#4d92a8"></td>
                                        </tr>

                                        <?php /* <!-- NTP - Network Time Protocol --> */ ?>
                                        <tr>
                                            <td>
                                                <table width="100%" border="0" cellpadding="4" cellspacing="0">
                                                    <tr class="tablehead">
                                                        <td colspan="2" id="tdNTPConfig"<?php if ($section_err["tdNTPConfig"]) {echo ' style="color:red"';}?>>
                                                            <a onClick="ConfigAdvFolderUnFolder(true, 6)" style="cursor:pointer">+NTP - Network Time Protocol</a>
                                                        </td>
                                                    </tr>
                                                    <tr></tr>
                                                </table>
                                                <div id="divNTPConfig" style="display:none">
                                                    <table width="100%" border="0" cellpadding="4" cellspacing="0">
                                                        <tr class="sorttable">
                                                            <td width="241">Enable NTP</td>
                                                            <td colspan="2" class="inputcol">
                                                                <select name="dropdown_NTPEnable" id="dropdown_NTPEnable" onchange="checkDropdownNtpEnable()">
                                                                    <option <?php if ($ConfigInfo["ntp"]["enable"] == 0) echo 'selected'; ?> value="0">Disable</option>
                                                                    <option <?php if ($ConfigInfo["ntp"]["enable"] == 1) echo 'selected'; ?> value="1">Enable</option>
                                                                </select>
                                                            </td>
                                                        </tr>
                                                        <tr class="sorttablenobg" <?php if ($ConfigInfo["ntp"]["enable"] == 0) echo 'style="display: none"'; ?>>
                                                            <td width="241">NTP Server</td>
                                                            <td colspan="2" class="inputcol">
                                                                <input name="txt_NTPServer" type="text" class="copy" id="txt_NTPServer" value="<?php echo htmlspecialchars($ConfigInfo["ntp"]["server"]); ?>" size="19" maxlength="64" />
                                                            </td>
                                                        </tr>
                                                        <tr class="sorttable" <?php if ($ConfigInfo["ntp"]["enable"] == 0) echo 'style="display: none"'; ?>>
                                                            <td width="241">NTP Mode</td>
                                                            <td colspan="2" class="inputcol">
                                                                <select name="dropdown_NTPMode" id="dropdown_NTPMode">
                                                                    <option <?php if ($ConfigInfo["ntp"]["mode"] == "0") echo 'selected'; ?> value="0">Client</option>
                                                                    <option <?php if ($ConfigInfo["ntp"]["mode"] == "1") echo 'selected'; ?> value="1">Client/Server</option>
                                                                </select>
                                                            </td>
                                                        </tr>
                                                    </table>
                                                </div>
                                            </td>
                                        </tr>
                                        <tr>
                                            <td height="4" colspan="4" align="left" bgcolor="#4d92a8"></td>
                                        </tr>

                                        <?php /* <!-- EIP - Ethernet/IP --> */ ?>
                                        <tr>
                                            <td>
                                                <table width="100%" border="0" cellpadding="4" cellspacing="0">
                                                    <tr class="tablehead">
                                                        <td colspan="2" id="tdEIPConfig">
                                                            <a onClick="ConfigAdvFolderUnFolder(true, 5)" style="cursor:pointer">+EIP - Ethernet/IP</a>
                                                        </td>
                                                    </tr>
                                                    <tr></tr>
                                                </table>
                                                <div id="divEIPConfig" style="display:none">
                                                    <table width="100%" border="0" cellpadding="4" cellspacing="0">
                                                        <tr class="sorttable">
                                                            <td width="241">Enable EIP</td>
                                                            <td colspan="2" class="inputcol">
                                                                <select name="dropdown_EIPEnable" id="dropdown_EIPEnable">
                                                                    <option <?php if ($ConfigInfo["eip"]["enable"] == 0) echo 'selected'; ?> value="0">Disable</option>
                                                                    <option <?php if ($ConfigInfo["eip"]["enable"] == 1) echo 'selected'; ?> value="1">Enable</option>
                                                                </select>
                                                            </td>
                                                        </tr>
                                                    </table>
                                                </div>
                                            </td>
                                        </tr>
                                        <tr>
                                            <td height="4" colspan="4" align="left" bgcolor="#4d92a8"></td>
                                        </tr>

                                        <tr class="tablehead">
                                           <td colspan="2" height="40" valign="bottom" align="center" >
                                                <input name="apply_all" type="submit" id="apply_all" value="Apply" class="copy">
                                            </td>
                                        </tr>
                                        <tr>
                                            <td>
                                                <table width="100%" border="0" cellspacing="0" cellpadding="0">
                                                    <tr>
                                                        <td>&nbsp;</td>
                                                    </tr>
                                                </table>
                                            </td>
                                        </tr>
                                    </table>
                                </form>
                            </td>
                        </tr>
                    </table>
                </td>
                <?php /* End content here */ ?>
                <td background="/images/bgright.jpg">&nbsp;</td>
            </tr>
            <tr height=16>
                <td >
                    <img src="/images/leftbtmangle.jpg" />
                </td>
                <td background="/images/bgbtm.jpg"></td>
                <td >
                    <img src="/images/rightbtmangle.jpg" />
                </td>
            </tr>
        </table>
        <?php /*End body */ ?>
        <script>
            <?php /* Set the top menu style */ ?>
            window.parent.document.getElementById("leftlink_configuration_basic").className = "leftNavLink";
            window.parent.document.getElementById("leftlink_configuration_advance").className = "leftNavLinkActive";
            window.parent.document.getElementById("leftlink_configuration_firewall").className = "leftNavLink";
            ShowHideConfigAdvanced("slt_gen_role");
            var errcnt = <?php echo $validate_err;?>;
            var errarray<?php
                if ($validate_err > 0) {
                    echo '=[';
                }
                for ($i = 0; $i < $validate_err; $i++) {
                    if ($i != 0 ) {
                        echo ',';
                    }
                    echo "\"$errArray[$i]\"";
                }
                if ($validate_err > 0) {
                    echo ']';
                }

            ?>;
            HighlightValidationErrors(errcnt, errarray);
            <?php
                if ($section_err["tdWANTitle"]) {
                    ;
                } else if ($section_err["tdDDNSTitle"]) {
                    echo "ConfigAdvFolderUnFolder(true, 1)";
                } else if ($section_err["tdCellUsage"]) {
                    echo "ConfigAdvFolderUnFolder(true, 4)";
                } else if ($section_err["tdVPNTitle"]) {
                  echo "ConfigAdvFolderUnFolder(true, 2)";
                } else if ($section_err["tdSerialTitle"]) {
                    echo "ConfigAdvFolderUnFolder(true, 3)";
                } else if ($section_err["tdCellUsage"]) {
                    echo "ConfigAdvFolderUnFolder(true, 4)";
                } else if ($section_err["tdNTPConfig"]) {
                    echo "ConfigAdvFolderUnFolder(true, 7)";
                }
            ?>

            <?php /* change ddns server to match the selected "Active" host */ ?>
            function ddnsServer() {
                var ddnsOption = document.getElementById("slt_DDNSActive").value;
                switch (ddnsOption) {
                    case '0':
                        document.getElementById("txt_DDNSServer").value = "";
                        document.getElementById("txt_DDNSDomainName").value = "";
                        document.getElementById("txt_DDNSUserName").value = "";
                        document.getElementById("txt_DDNSPass").value = "";
                        break;
                    case '1':
                        document.getElementById("txt_DDNSServer").value = "dyndns@dyndns.org";
                        break;
                    case '2':
                        document.getElementById("txt_DDNSServer").value = "default@no-ip.com";
                        break;
                }
            }
        </script>
    </body>
</html>
