<?php
    session_start();
    require_once('../inc/userfunction.php');
    require_once('../inc/systemfunction.php');

    // Check the login session
    if (CheckUserSession() == false) {
        header("Location: /components/session_expired.php");
        exit;
    }

    $method = $_SERVER['REQUEST_METHOD'];
    $csrf = $_POST['csrf'];

    // Check the CSRF token
    if (CheckCSRFToken($method, $csrf) == false) {
        error_log("Warning! Possible CSRF detected.", 0);
        exit;
    }

    $validate_err = 0;
    $rebootNeeded = 0;
    $newPin = 0;

    $validPin = 0;
    $validNewPin = 0;
    $validPuk = 0;
    $showNewPin = 0;

    exec("/psft/bin/gpioModify getSIMstatus", $simPresent);
    exec("/psft/bin/pinHandler status", $simStatus);
    exec("/psft/bin/pinHandler verification", $pinStatus);

    //Get current Settings
    $ConfigInfo=array();
    // Module Settings
    $temp = ConfigRead("module/name");
    $ConfigInfo["module/name"] = $temp[0];
    $temp = ConfigRead("ppp/apn");
    $ConfigInfo["ppp/apn"] = $temp[0];
    // LAN Settings
    $temp = ConfigRead("lan/ipadr");
    $ConfigInfo["lan/ipadr"] = $temp[0];
    $temp = ConfigRead("lan/ipCIDRmask");
    $ConfigInfo["lan/ipCIDRmask"] = $temp[0];
    $temp = ConfigRead("lan/dhcpserver");
    $ConfigInfo["lan/dhcpserver"] = $temp[0];
    $temp = ConfigRead("lan/dhcppoollo");
    $ConfigInfo["lan/dhcppoollo"] = $temp[0];
    $temp = ConfigRead("lan/dhcppoolhi");
    $ConfigInfo["lan/dhcppoolhi"] = $temp[0];
    $temp = ConfigRead("lan/dhcpleasetime");
    $ConfigInfo["lan/dhcpleasetime"] = $temp[0];
    $temp = ConfigRead("lan/dhcpleaseunits");
    $ConfigInfo["lan/dhcpleaseunits"] = $temp[0];
    $temp = ConfigRead("lan/dhcpleasenum");
    $ConfigInfo["lan/dhcpleasenum"] = $temp[0];
    for ($countLeaseNum = 0; $countLeaseNum < $ConfigInfo["lan/dhcpleasenum"]; $countLeaseNum++) {
        $temp = ConfigRead("lan/dhcplease" . $countLeaseNum);
        $ConfigInfo["lan/dhcplease" . $countLeaseNum] = explode(";", $temp[0]);
    }

    // SIM Settings
    $temp = ConfigRead("sim/pin");
    if (strlen($temp[0]) > 0) {
        $pinDec = pinCrypto("dec", $temp[0]);
        $ConfigInfo["sim/pin"] = $pinDec[0];
    } else {
        $ConfigInfo["sim/pin"] = "";
    }
    
    // Firewall
    $temp = ConfigRead("firewall/enableportfw");
    $ConfigInfo["firewall/enableportfw"] = $temp[0];
    $temp = ConfigRead("lan/passthrough");
    $ConfigInfo["lan/passthrough"] = $temp[0];

    $action = $_REQUEST['action'];
    if ($action == 1) {
        $ValidIpv4AddressRegex = "/^(([1-9][0-9]?|1[0-9][0-9]|2([0-4][0-9]|5[0-5]))\.)(([01]?[0-9]?[0-9]|2([0-4][0-9]|5[0-5]))\.){2}([01]?[0-9]?[0-9]|2([0-4][0-9]|5[0-5]))$/";
        $ValidChars = "/^[a-zA-Z0-9.\~\`\!\@\#\$\%\^\*\(\)\-\+\=\{\}\:\?\_]+$/i";
        $validMacAddressRegex = "/^([0-9A-Fa-f]{2}[:-]){5}([0-9A-Fa-f]{2})$/";
        $ValidPin = "/^[0-9]{4,8}$/";
        $ValidPuk = "/^[0-9]{8,12}$/";

        // Module Settings
        $txt_ModuleName = htmlspecialchars_decode(trim($_POST['txt_ModuleName']));
        $txt_APNName = htmlspecialchars(trim($_POST['txt_APNName']));

        // LAN Settings
        $txt_LANIPadr = htmlspecialchars_decode(trim($_POST['txt_LANIPadr']));
        $LANIP_Split = preg_split("/[\s.]+/", $txt_LANIPadr);

        $txt_LANIPmask = htmlspecialchars_decode(trim($_POST['txt_LANIPmask']));
        $dropdown_DHCP = $_POST['dropdown_DHCP'];
        $txt_DHCPRange = array(1 => htmlspecialchars_decode(trim($_POST['txt_DHCPRange-1'])),
                         2 => htmlspecialchars_decode(trim($_POST['txt_DHCPRange-2'])));

        $txt_DHCPLeaseTime = htmlspecialchars_decode(trim($_POST['txt_DHCPLeaseTime']));
        $dropdown_DHCPunit = $_POST['dropdown_DHCPunit'];

        $dhcpTableLen = htmlspecialchars_decode(trim($_POST['dhcpTableLen']));
        for ($countLeaseNum = 0; $countLeaseNum < $dhcpTableLen; $countLeaseNum++) {
            $txtDhcpHostname[$countLeaseNum] = htmlspecialchars_decode(trim($_POST['txt_dhcp_hostname' . $countLeaseNum]));
            $txtDhcpMacaddr[$countLeaseNum] = htmlspecialchars_decode(trim($_POST['txt_dhcp_macaddr' . $countLeaseNum]));
            $txtDhcpIpaddr[$countLeaseNum] = htmlspecialchars_decode(trim($_POST['txt_dhcp_ipaddr' . $countLeaseNum]));
        }

        // SIM Settings
        $dropdown_PinEnable = $_POST['dropdown_PinEnable'];
        $txt_simPin = htmlspecialchars_decode(trim($_POST['txt_simPin']));
        $txt_simPuk = htmlspecialchars_decode(trim($_POST['txt_simPuk']));
        $txt_simPin_new = htmlspecialchars_decode(trim($_POST['txt_simPin_new']));

        // Validate parameters
        if ((strlen($txt_ModuleName) == 0) || (strlen($txt_ModuleName) > 40)) {
            $errMsgs[$validate_err] = "Module Name may not be blank, or exceed 40 character limit.";
            $errArray[$validate_err++] = "txt_ModuleName";
        }

        if (!preg_match($ValidChars, $txt_ModuleName)) {
            $errMsgs[$validate_err] = "Module Name uses invalid characters.";
            $errArray[$validate_err++] = "txt_ModuleName";
        }

        if (strlen($txt_APNName) > 40) {
            $errMsgs[$validate_err] = "APN Name must not exceed 40 character limit.";
            $errArray[$validate_err++] = "txt_APNName";
        }

        if (!preg_match($ValidChars, $txt_APNName)) {
            $errMsgs[$validate_err] = "APN Name uses invalid characters.";
            $errArray[$validate_err++] = "txt_APNName";
        }

        /*
         * Validate LAN IP address and Subnet Mask to be able to correctly
         * calculate the subnet limits - required for end device/DHCP limits.
         * If one of these inputs is not correct we cannot validate the rest of
         * the parameters that depend on them.
         */
        $isValid = true;
        if (0 == preg_match($ValidIpv4AddressRegex, $txt_LANIPadr) ||
            preg_split("/[\s.]+/", $txt_LANIPadr)[0] == 127) {
            $errMsgs[$validate_err] = "LAN IP address is not a valid IP address.";
            $errArray[$validate_err++] = "txt_LANIPadr";
            $isValid = false;
        }

        $temp = trim($_POST['txt_LANIPmask']);
        /* Verify the Subnet Mask has a valid value */
        if (!ctype_digit($temp) || ($temp < 1) || ($temp > 32)) {
            $errMsgs[$validate_err] = "LAN subnet mask must have a numeric value between 1 and 32.";
            $errArray[$validate_err++] = "txt_LANIPmask";
            $isValid = false;
        }

        if (true == $isValid) {
            /* Calculate subnet limits based on LAN IP address and subnet mask*/
            $subnetLoLimit = ip2long(ipCalc("n", $txt_LANIPadr, $temp)[0]);
            $subnetHiLimit = ip2long(ipCalc("b", $txt_LANIPadr, $temp)[0]);

            /* Verify LAN IP is within the subnet limits */
            if (ip2long($txt_LANIPadr) <= $subnetLoLimit || $subnetHiLimit <= ip2long($txt_LANIPadr)) {
                $errMsgs[$validate_err] = "LAN IP address is out of bounds.";
                $errArray[$validate_err++] = "txt_LANIPadr";
            }

            if ($dropdown_DHCP == "1") {
                /* Validate DHCP IP range for valid format */
                if (0 == preg_match($ValidIpv4AddressRegex, $txt_DHCPRange[1])) {
                    $errMsgs[$validate_err] = "DHCP Lo Range IP address is not a valid IP address.";
                    $errArray[$validate_err++] = "txt_DHCPRange-1";
                    $isValid = false;
                }

                if (0 == preg_match($ValidIpv4AddressRegex, $txt_DHCPRange[2])) {
                    $errMsgs[$validate_err] = "DHCP Hi Range IP address is not a valid IP address.";
                    $errArray[$validate_err++] = "txt_DHCPRange-2";
                    $isValid = false;
                }

                /* Do not check further if the initial validations failed */
                if (true == $isValid) {
                    /* Verify the DHCP IP range is within the LAN subnet */
                    if ($subnetHiLimit <= ip2long($txt_DHCPRange[1]) ||
                    ip2long($txt_DHCPRange[1]) <= $subnetLoLimit) {
                        $errMsgs[$validate_err] = "DHCP Lo Range IP address is out of bounds.";
                        $errArray[$validate_err++] = "txt_LANIPadr";
                        $errArray[$validate_err++] = "txt_DHCPRange-1";
                    }
                    if ($subnetHiLimit <= ip2long($txt_DHCPRange[2]) ||
                        ip2long($txt_DHCPRange[2]) <= $subnetLoLimit) {
                            $errMsgs[$validate_err] = "DHCP Hi Range IP address is out of bounds.";
                            $errArray[$validate_err++] = "txt_LANIPadr";
                            $errArray[$validate_err++] = "txt_DHCPRange-2";
                    }

                    /* DHCP low limit <= DHCP high limit */
                    if (ip2long($txt_DHCPRange[2]) < ip2long($txt_DHCPRange[1])) {
                        $errMsgs[$validate_err] = "Invalid DHCP address range.";
                        $errArray[$validate_err++] = "txt_LANIPadr";
                        $errArray[$validate_err++] = "txt_DHCPRange-1";
                        $errArray[$validate_err++] = "txt_DHCPRange-2";
                    }
                }

                for ($countLeaseNum = 0; $countLeaseNum < $dhcpTableLen; $countLeaseNum++) {
                    /* Check "Host Name" for valid size */
                    if ((strlen($txtDhcpHostname[$countLeaseNum]) == 0) ||
                        (strlen($txtDhcpHostname[$countLeaseNum]) > 12)) {
                        $errMsgs[$validate_err] = "Lease Host Name may not be blank, or exceed 12 character limit.";
                        $errArray[$validate_err++] = "txt_dhcp_hostname" . $countLeaseNum;
                    }

                    /* Check "Hostname" for valid characters */
                    if (!preg_match($ValidChars, $txtDhcpHostname[$countLeaseNum])) {
                            $errMsgs[$validate_err] = "Lease Host Name uses invalid characters.";
                            $errArray[$validate_err++] = "txt_dhcp_hostname" . $countLeaseNum;
                    }

                    /* Check "MAC Address" for valid format */
                    if ((strlen($txtDhcpMacaddr[$countLeaseNum]) == 0) ||
                        (!preg_match($validMacAddressRegex, $txtDhcpMacaddr[$countLeaseNum]))) {
                        $errMsgs[$validate_err] = "Lease MAC Address is not a valid MAC address.";
                        $errArray[$validate_err++] = "txt_dhcp_macaddr" . $countLeaseNum;
                    }

                    /* Check "IP Address" for valid format */
                    if (!preg_match($ValidIpv4AddressRegex, $txtDhcpIpaddr[$countLeaseNum])) {
                        $errMsgs[$validate_err] = "Lease IP Address is not a valid IP address.";
                        $errArray[$validate_err++] = "txt_dhcp_ipaddr" . $countLeaseNum;
                    }

                    /* Validate "IP Address" in the LAN subnet  */
                    $temp = ip2long($txtDhcpIpaddr[$countLeaseNum]);
                    if (($subnetHiLimit <= $temp) || ($temp <= $subnetLoLimit)) {
                        $errMsgs[$validate_err] = "Lease IP Address is out of bounds.";
                        $errArray[$validate_err++] = "txt_dhcp_ipaddr" . $countLeaseNum;
                    }

                }

                /* Validate table for duplicate entries */
                for ($countLeaseNum = 0; $countLeaseNum < $dhcpTableLen; $countLeaseNum++) {
                    for ($it = $countLeaseNum + 1; $it < $dhcpTableLen; $it++) {
                        /* Check "Hostname" for duplicate entries */
                        if (strcmp(strtolower($txtDhcpHostname[$countLeaseNum]), strtolower($txtDhcpHostname[$it])) == 0) {
                            $errMsgs[$validate_err] = "Duplicate Hostname entries.";
                            $errArray[$validate_err++] = "txt_dhcp_hostname" . $countLeaseNum;
                            $errArray[$validate_err++] = "txt_dhcp_hostname" . $it;
                        }
                        /* Check "MAC Address" for duplicate entries */
                        if (strcmp(strtolower($txtDhcpMacaddr[$countLeaseNum]), strtolower($txtDhcpMacaddr[$it])) == 0) {
                            $errMsgs[$validate_err] = "Duplicate Lease MAC Address entries.";
                            $errArray[$validate_err++] = "txt_dhcp_macaddr" . $countLeaseNum;
                            $errArray[$validate_err++] = "txt_dhcp_macaddr" . $it;
                        }

                        /* Check "IP Address" for duplicate entries */
                        $temp = ip2long($txtDhcpIpaddr[$countLeaseNum]);
                        if ($temp == ip2long($txtDhcpIpaddr[$it])) {
                            $errMsgs[$validate_err] = "Duplicate Lease IP Address entries.";
                            $errArray[$validate_err++] = "txt_dhcp_ipaddr" . $countLeaseNum;
                            $errArray[$validate_err++] = "txt_dhcp_ipaddr" . $it;
                        }
                    }
                }

                /* Validate DHCP lease time - integer greater than 0 */
                $temp = trim($_POST['txt_DHCPLeaseTime']);
                if (!ctype_digit($temp) || $temp < 1) {
                    $errMsgs[$validate_err] = "DHCP Lease Time must have a numeric value greater than 0.";
                    $errArray[$validate_err++] = "txt_DHCPLeaseTime";
                }
            }
        }

        // Validate SIM settings
        if (strlen($txt_simPin) > 0) {
            if (!preg_match($ValidPin, $txt_simPin)) {
                $errMsgs[$validate_err] = "Invalid PIN code.";
                $errArray[$validate_err++] = "txt_simPin";
            } else {
                $validPin = 1;
            }
        }
        
        if (strlen($txt_simPin_new) > 0) {
            if (!preg_match($ValidPin, $txt_simPin_new)) {
                $errMsgs[$validate_err] = "Invalid PIN code.";
                $errArray[$validate_err++] = "txt_simPin_new";
                $showNewPin = 1;
            } else {
                $validNewPin = 1;
            }
        }
        
        if (strlen($txt_simPuk) > 0) {
            if (!preg_match($ValidPuk, $txt_simPuk)) {
                $errMsgs[$validate_err] = "Invalid PUK code.";
                $errArray[$validate_err++] = "txt_simPuk";
            } else if ($validNewPin == 0) {
                $errMsgs[$validate_err] = "New PIN code required.";
                $errArray[$validate_err++] = "txt_simPin_new";
            } else {
                $validPuk = 1;
            }
        }

        if (($validPuk == 0) && ($validPin == 1) && ($validNewPin == 1)) {
            if (strcmp($txt_simPin, $txt_simPin_new) == 0) {
                $errMsgs[$validate_err] = "New PIN code cannot be the same";
                $errArray[$validate_err++] = "txt_simPin";
                $errArray[$validate_err++] = "txt_simPin_new";
                $showNewPin = 1;
                $validPin = $validNewPin = 0;
            }
        }

        // Unblock SIM
        if (($simStatus[0] == "SIM PUK") && ($validPuk == 1) && ($validNewPin == 1)) {
            $retval = pinHandler("unblock", $txt_simPuk, $txt_simPin_new);
            if ($retval != 0) {
                $errMsgs[$validate_err] = "Incorrect PUK code !";
                $errArray[$validate_err++] = "txt_simPuk";
            } else {
                $newPin = 1;
                echo "<SCRIPT>alert('SIM unblocked. New PIN code set.');</SCRIPT>";
                $simStatus[0] = "READY";
            }
		}
		
        // Unlock SIM
        if (($simStatus[0] == "SIM PIN") && ($validPin == 1)) {
            $retval = pinHandler("unlock", $txt_simPin);
            if ($retval != 0) {
                $errMsgs[$validate_err] = "Incorrect PIN code !";
                $errArray[$validate_err++] = "txt_simPin";
            } else {
                echo "<SCRIPT>alert('SIM unlocked.');</SCRIPT>";
                $simStatus[0] = "READY";
            }
		}
		
        // Change PIN
        if (($simStatus[0] == "READY") && ($validPin == 1) && ($validNewPin == 1) && ($pinStatus[0] == 1)) {
            $retval = pinHandler("change", $txt_simPin, $txt_simPin_new);
            if ($retval != 0) {
                $errMsgs[$validate_err] = "Incorrect PIN code !";
                $errArray[$validate_err++] = "txt_simPin";
            } else {
                $newPin = 1;
                echo "<SCRIPT>alert('PIN code changed.');</SCRIPT>";
            }
        // Check PIN
        } else if (($simStatus[0] == "READY") && ($validPin == 1) && ($pinStatus[0] == 1)) {
            $retval = pinHandler("change", $txt_simPin, $txt_simPin);
            if ($retval != 0) {
                $errMsgs[$validate_err] = "Incorrect PIN code!";
                $errArray[$validate_err++] = "txt_simPin";
            }
        }
        
        // PIN verification status changed
        if (($dropdown_PinEnable != $pinStatus[0]) && ($validate_err == 0)) {
            if ($validPin == 1) {
                // Enable PIN verification
                if ($dropdown_PinEnable == "1") {
                    $retval = pinHandler("enable", $txt_simPin);
                    if ($retval != 0) {
                        $errMsgs[$validate_err] = "Incorrect PIN code!";
                        $errArray[$validate_err++] = "txt_simPin";
                    } else {
                        echo "<SCRIPT>alert('PIN Verification enabled.');</SCRIPT>";
                        $pinStatus[0] = 1;
                    }
                }
                // Disable PIN verification
                if ($dropdown_PinEnable == "0") {
                    $retval = pinHandler("disable", $txt_simPin);
                    if ($retval != 0) {
                        $errMsgs[$validate_err] = "Incorrect PIN code!";
                        $errArray[$validate_err++] = "txt_simPin";
                    } else {
                        echo "<SCRIPT>alert('PIN Verification disabled.');</SCRIPT>";
                        $pinStatus[0] = 0;
                    }
                }
            } else {
				$errMsgs[$validate_err] = "PIN code required.";
                $errArray[$validate_err++] = "txt_simPin";
			}
        }

        /* Save parameters */
        if (0 == $validate_err) {
            $error = false;
            /* if validation OK, write settings to flash */
            // Module Settings
            if ($ConfigInfo["module/name"] != $txt_ModuleName) {
                $retval = ConfigWrite("module/name", $txt_ModuleName);
                $rebootNeeded = 1;
                if (0 > $retval[0]) {
                    echo "<SCRIPT>
                        alert('Error setting End Device IP address!');
                        window.location = 'configuration_basic.php';</SCRIPT>";
                    $error = true;
                }
            }
            if ($ConfigInfo["ppp/apn"] != $txt_APNName) {
                $retval = ConfigWrite("ppp/apn", $txt_APNName);
                $rebootNeeded = 1;
                if (0 > $retval[0]) {
                    echo "<SCRIPT>
                        alert('Error setting APN Name!');
                        window.location = 'configuration_basic.php';</SCRIPT>";
                    $error = true;
                }
            }
            // LAN Settings
            if ($ConfigInfo["lan/ipadr"] != $txt_LANIPadr) {
                $retval = ConfigWrite("lan/ipadr",$txt_LANIPadr);
                $rebootNeeded = 1;
                if (0 > $retval[0]) {
                    echo "<SCRIPT>
                        alert('Error setting LAN IP address!');
                        window.location = 'configuration_basic.php';</SCRIPT>";
                    $error = true;
                }
            }
            if ($ConfigInfo["lan/ipCIDRmask"] != $txt_LANIPmask) {
                $retval = ConfigWrite("lan/ipCIDRmask",$txt_LANIPmask);
                $rebootNeeded = 1;
                if (0 > $retval[0]) {
                    echo "<SCRIPT>
                        alert('Error setting LAN subnet mask!');
                        window.location = 'configuration_basic.php';</SCRIPT>";
                    $error = true;
                }
            }

            // LAN subnet change invalidates port forwarding rules outside the subnet
            $lanBroadcast = getLanBroadcast();
            $lanNetwork = getLanNetwork();
            for ($portFwNum = 0; $portFwNum < 10; $portFwNum++) {
                $pfEntry = ConfigRead("firewall/portfw" . $portFwNum);
                $pfData = explode(";", $pfEntry[0]);
                if (((ip2long($pfData[4]) <= ip2long($lanNetwork[0])) ||
                     (ip2long($pfData[4]) >= ip2long($lanBroadcast[0]))) &&
                    ($pfEntry[0] != "")) {
                    $retval = ConfigWrite("firewall/portfw" . $portFwNum, "");
                    $rebootNeeded = 1;
                    if (0 > $retval[0]) {
                        echo "<SCRIPT
                            alert('Error setting Port Forwarding entry!');
                            window.location = 'configuration_basic.php';</SCRIPT>";
                        $error = true;
                    }
                }
            }
            if ($ConfigInfo["lan/dhcpserver"] != $dropdown_DHCP) {
                $retval = ConfigWrite("lan/dhcpserver",$dropdown_DHCP);
                $rebootNeeded = 1;
                if (0 > $retval[0]) {
                    echo "<SCRIPT>
                        alert('Error setting DHCP server!');
                        window.location = 'configuration_basic.php';</SCRIPT>";
                    $error = true;
                }
            }
            if ($ConfigInfo["lan/dhcppoollo"] != $txt_DHCPRange[1]) {
                $retval = ConfigWrite("lan/dhcppoollo",$txt_DHCPRange[1]);
                $rebootNeeded = 1;
                if (0 > $retval[0]) {
                    echo "<SCRIPT>
                        alert('Error setting lo DHCP Range!');
                        window.location = 'configuration_basic.php';</SCRIPT>";
                    $error = true;
                }
            }
            if ($ConfigInfo["lan/dhcppoolhi"] != $txt_DHCPRange[2]) {
                $retval = ConfigWrite("lan/dhcppoolhi",$txt_DHCPRange[2]);
                $rebootNeeded = 1;
                if (0 > $retval[0]) {
                    echo "<SCRIPT>
                        alert('Error setting hi DHCP Range!');
                        window.location = 'configuration_basic.php';</SCRIPT>";
                    $error = true;
                }
            }
            if ($ConfigInfo["lan/dhcpleasetime"] != $txt_DHCPLeaseTime) {
                $retval = ConfigWrite("lan/dhcpleasetime",$txt_DHCPLeaseTime);
                $rebootNeeded = 1;
                if (0 > $retval[0]) {
                    echo "<SCRIPT>
                        alert('Error setting DHCP Lease Time!');
                        window.location = 'configuration_basic.php';</SCRIPT>";
                    $error = true;
                }
            }
            if ($ConfigInfo["lan/dhcpleaseunits"] != $dropdown_DHCPunit) {
                $retval = ConfigWrite("lan/dhcpleaseunits",$dropdown_DHCPunit);
                $rebootNeeded = 1;
                if (0 > $retval[0]) {
                    echo "<SCRIPT>
                        alert('Error setting DHCP lease time unit!');
                        window.location = 'configuration_basic.php';</SCRIPT>";
                    $error = true;
                }
            }
            if ($ConfigInfo["lan/dhcpleasenum"] != $dhcpTableLen) {
                $retval = ConfigWrite("lan/dhcpleasenum", $dhcpTableLen);
                $rebootNeeded = 1;

                if (0 > $retval[0]) {
                    echo "<SCRIPT>
                        alert('Error settings DHCP Static Leases!');
                        window.location = 'configuration_basic.php';</SCRIPT>";
                    $error = true;
                }
            }
            for ($countLeaseNum = 0; $countLeaseNum < $dhcpTableLen; $countLeaseNum++) {
                    if (($ConfigInfo["lan/dhcplease" . $countLeaseNum][0] != $txtDhcpHostname[$countLeaseNum]) ||
                        ($ConfigInfo["lan/dhcplease" . $countLeaseNum][1] != $txtDhcpMacaddr[$countLeaseNum]) ||
                        ($ConfigInfo["lan/dhcplease" . $countLeaseNum][2] != $txtDhcpIpaddr[$countLeaseNum])) {
                        $temp = $txtDhcpHostname[$countLeaseNum] . ';' . $txtDhcpMacaddr[$countLeaseNum] . ';' . $txtDhcpIpaddr[$countLeaseNum];
                        $retval = ConfigWrite("lan/dhcplease" . $countLeaseNum, $temp);
                        $rebootNeeded = 1;

                    if (0 > $retval[0]) {
                        echo "<SCRIPT>
                            alert('Error settings DHCP Static Lease" . $countLeaseNum + 1 . "!');
                            window.location = 'configuration_basic.php';</SCRIPT>";
                    $error = true;
                    }
                }
            }

            // SIM Settings
            if (isset($_POST['savePinCB'])) {
                if ($newPin == 1) {
                    $pinEnc = pinCrypto("enc", $txt_simPin_new);
                    $ConfigInfo["sim/pin"] = $txt_simPin_new;
                } else if ($validPin == 1) {
                    $pinEnc = pinCrypto("enc", $txt_simPin);
                    $ConfigInfo["sim/pin"] = $txt_simPin;
                } else {
                    $ConfigInfo["sim/pin"] = $pinEnc[0] = "";
                }
            } else {
                $ConfigInfo["sim/pin"] = $pinEnc[0] = "";
            }
            $retval = ConfigWrite("sim/pin", $pinEnc[0]);
            if (0 > $retval[0]) {
                echo "<SCRIPT>
                    alert('Error saving PIN to config file!');
                    window.location = 'configuration_basic.php';</SCRIPT>";
                $error = true;
                $ConfigInfo["sim/pin"] = "";
            }

            $retval = ConfigToFlash();
            if (1 != $retval[0]) {
                echo "<SCRIPT>
                    alert('Error saving new configuration!');
                    window.location = 'configuration_basic.php';</SCRIPT>";
                $error = true;
            }

            if (!$error && ($rebootNeeded == 1)) {
                RebootModule();
                echo "<script>window.location = 'rebooting.php';</script>";
            }
        }
    }

    if ($validate_err > 0) {
        // Module Settings
        $ConfigInfo["module/name"] = trim($_POST['txt_ModuleName']);
        $ConfigInfo["ppp/apn"] =trim($_POST['txt_APNName']);
        // LAN Settings
        $ConfigInfo["lan/ipadr"] = trim($_POST['txt_LANIPadr']);
        $ConfigInfo["lan/ipCIDRmask"] = trim($_POST['txt_LANIPmask']);
        $ConfigInfo["lan/dhcpserver"] = trim($_POST['dropdown_DHCP']);
        $ConfigInfo["lan/dhcppoollo"] = trim($_POST['txt_DHCPRange-1']);
        $ConfigInfo["lan/dhcppoolhi"] = trim($_POST['txt_DHCPRange-2']);
        $ConfigInfo["lan/dhcpleasetime"] = trim($_POST['txt_DHCPLeaseTime']);
        $ConfigInfo["lan/dhcpleaseunits"] = trim($_POST['dropdown_DHCPunit']);
        $ConfigInfo["lan/dhcpleasenum"] = trim($_POST['dhcpTableLen']);
        for ($countLeaseNum = 0; $countLeaseNum < $ConfigInfo["lan/dhcpleasenum"]; $countLeaseNum++) {
            $ConfigInfo["lan/dhcplease" . $countLeaseNum][0] = trim($_POST['txt_dhcp_hostname' . $countLeaseNum]);
            $ConfigInfo["lan/dhcplease" . $countLeaseNum][1] = trim($_POST['txt_dhcp_macaddr' . $countLeaseNum]);
            $ConfigInfo["lan/dhcplease" . $countLeaseNum][2] = trim($_POST['txt_dhcp_ipaddr' . $countLeaseNum]);
        }
    }

    /* Update SIM status and retries count */
    $pinRetry = pinHandler("pinretry");
    $pukRetry = pinHandler("pukretry");
    
    if ($pinRetry[0] == "0") {
        $simStatus[0] = "SIM PUK";
    }

    if ($simPresent[0]) {
        $cellStatus = '<span id="simStatus" class="style3">Not found</span>';
    } elseif ($simStatus[0] == "READY") {
        $cellStatus = '<table width="450" border=0 align="left" cellspacing=0 style="margin:0px;">' .
                      '<tr><td width="265"><span class="style2">Unlocked</span></td>' .
                      '<td><span>Remaining: ' . $pinRetry[0] .'</span></td></tr></table>';
    } elseif ($simStatus[0] == "SIM PIN") {
        $cellStatus = '<table width="450" border=0 align="left" cellspacing=0 style="margin:0px;">' .
                      '<tr><td width="265"><span class="style3">Locked</span></td>' .
                      '<td><span>Remaining: ' . $pinRetry[0] .'</span></td></tr></table>';
    } elseif ($simStatus[0] == "SIM PUK") {
        $cellStatus = '<table width="450" border=0 align="left" cellspacing=0 style="margin:0px;">' .
                      '<tr><td width="265"><span class="style4">Blocked</span></td>' .
                      '<td><span>Remaining: ' . $pukRetry[0] .'</span></td></tr></table>';
    }
?>

<html>
    <head>
        <META HTTP-EQUIV="Pragma" CONTENT="no-cache"/>
        <META HTTP-EQUIV="Expires" CONTENT="-1"/>
        <link rel="stylesheet" href="/css/styles.css" type="text/css" />
        <script src="/scripts/angular.min.js" type="text/JavaScript"></script>
        <script src="/scripts/jquery.min.js" type="text/JavaScript"></script>
        <script src="/js/application.js" type="text/JavaScript"></script>
        <script src="/js/main.js" type="text/JavaScript"></script>
        <style type="text/css">
            .style2 {
                color: #000000;
                background-color: #00FF33;
            }
            .style3 {
                color: #000000;
                background-color: #FFFF33;
            }
            .style4 {
                color: #000000;
                background-color: #FF3333;
            }
        </style>
    </head>

    <body leftmargin="0" topmargin="0" rightmargin="0" bottommargin="0" marginwidth="0" onload='parent.resizeIframe(document.body.scrollHeight)'>
        <?php /* <!-- Start Body --> */ ?>
        <table width=100% height="550" cellpadding="0" cellspacing="0" >
            <tr height=13>
                <td width=14 >
                    <img src="/images/leftopangle.jpg" />
                </td>
                <td background="/images/bgtop.jpg" ></td>
                <td width=14 >
                    <img src="/images/rightopangle.jpg" />
                </td>
            </tr>
            <tr>
                <td background="/images/bgleft.jpg" >&nbsp;</td>
                <td align="center" valign="top">
                    <?php /* <!--Start content here --> */ ?>
                    <table width="98%" border="0" cellspacing="0" cellpadding="0" <?php /*bgcolor="#002569"*/ ?>>
                        <tr>
                            <td width="100%" <?php /*bgcolor="#FFFFFF"*/ ?>>
                                <table width="100%" border="0" cellspacing="0" cellpadding="0">
                                    <tr>
                                        <td width="75%"></td>
                                    </tr>
                                </table>
                                <?php /* <!-- Body starts here --> */ ?>
                                <table width="100%" border="0" cellspacing="0" cellpadding="0">
                                    <tr>
                                        <td valign="top">
                                            <?php
                                                if ($validate_err > 0) {
                                                    echo "<h3 style='color:#f04040'>The parameters highlighted in red are incorrect.  Please update with a valid value.</h3>";
                                                    for ($i = 0; $i < $validate_err; $i++) {
                                                        echo "<h4 style='color:#f04040'>" . $errMsgs[$i] . "</h4>";
                                                    }
                                                }
                                            ?>
                                            <form action="configuration_basic.php?action=1" method="POST"  name="basicconfig" id="basicconfig" >
                                              <input type="hidden" name="csrf" value="<?php echo $_SESSION['csrf']; ?>" />
                                                <table width="100%" border="0" cellspacing="0" cellpadding="0">
                                                    <tr>
                                                        <td>
                                                            <table width="100%" border="0" cellpadding="4" cellspacing="0">
                                                                <tr class="tablehead">
                                                                    <td colspan="3">Module Settings</td>
                                                                </tr>
                                                                <tr>
                                                                    <td height="4" colspan="5" align="left" bgcolor="#4d92a8"></td>
                                                                </tr>
                                                                <tr class="sorttable">
                                                                    <td width="241">Module Name</td>
                                                                    <td colspan="2" class="inputcol">
                                                                        <input name="txt_ModuleName" type="text" class="copy" id="txt_ModuleName" value="<?php echo htmlspecialchars($ConfigInfo['module/name']); ?>" size="40" maxlength="512"/>
                                                                    </td>
                                                                </tr>
                                                                <tr class="sorttablenobg">
                                                                    <td width="241">APN Name</td>
                                                                    <td colspan="2" class="inputcol">
                                                                        <input name="txt_APNName" type="text" class="copy" id="txt_APNName" value="<?php echo htmlspecialchars($ConfigInfo['ppp/apn']); ?>" size="40" maxlength="512"/>
                                                                    </td>
                                                                </tr>
                                                                <tr>
                                                                    <td height="4" colspan="5" align="left"></td>
                                                                </tr>
                                                                <tr>
                                                                    <td height="4" colspan="5" align="left"></td>
                                                                </tr>
                                                            </table>
                                                        </td>
                                                    </tr>
                                                    <tr>
                                                        <td>
                                                            <table width="100%" border="0" cellpadding="4" cellspacing="0">
                                                                <tr class="tablehead">
                                                                    <td colspan="3">LAN Settings</td>
                                                                </tr>
                                                                <tr>
                                                                    <td height="4" colspan="5" align="left" bgcolor="#4d92a8"></td>
                                                                </tr>
                                                                <tr class="sorttable">
                                                                    <td width="241">LAN IP Address / Subnet Mask </td>
                                                                    <td colspan="2" class="inputcol">
                                                                        <div <?php if($ConfigInfo['firewall/enableportfw'] || $ConfigInfo["lan/passthrough"]) echo 'data-tip="Cannot change LAN IP Address / Subnet Mask while Port Forwarding or Pass Through is enabled."'; else echo 'data-tip="Changing LAN IP Address/Subnet Mask will remove invalid Port Forwarding entries."'; ?>>
                                                                            <input name="txt_LANIPadr" type="text" class="copy" id="txt_LANIPadr" value="<?php echo htmlspecialchars($ConfigInfo['lan/ipadr']); ?>" size="40" maxlength="64" <?php if($ConfigInfo['firewall/enableportfw'] || $ConfigInfo["lan/passthrough"]) echo 'readonly'; ?>/> /
                                                                            <input name="txt_LANIPmask" id="txt_LANIPmask" type="text" class="copy" value="<?php echo htmlspecialchars($ConfigInfo['lan/ipCIDRmask']); ?>" size="3" maxlength="3" <?php if($ConfigInfo['firewall/enableportfw'] || $ConfigInfo["lan/passthrough"]) echo 'readonly'; ?>/>
                                                                        </div>
                                                                    </td>
                                                                </tr>
                                                                <tr class="sorttablenobg">
                                                                    <td width="241">DHCP Server</td>
                                                                    <td colspan="2" class="inputcol">
                                                                        <select name="dropdown_DHCP" id="dropdown_DHCP" onchange="checkDropdownDhcp()">
                                                                            <option <?php if($ConfigInfo['lan/dhcpserver'] == 0) echo 'selected'; ?> value="0">Disable</option>
                                                                            <option <?php if($ConfigInfo['lan/dhcpserver'] == 1) echo 'selected'; ?> value="1">Enable</option>
                                                                        </select>
                                                                    </td>
                                                                </tr>
                                                                <tr class="sorttable" <?php if($ConfigInfo['lan/dhcpserver'] == 0) echo 'style="display: none"'; ?>>
                                                                    <td width="241">DHCP Range</td>
                                                                    <td colspan="2" class="inputcol">
                                                                        <input name="txt_DHCPRange-1" type="text" class="copy" id="txt_DHCPRange-1" value="<?php echo htmlspecialchars($ConfigInfo['lan/dhcppoollo']); ?>" size="19" maxlength="64"/> -
                                                                        <input name="txt_DHCPRange-2" type="text" class="copy" id="txt_DHCPRange-2" value="<?php echo htmlspecialchars($ConfigInfo['lan/dhcppoolhi']); ?>" size="19" maxlength="64"/>
                                                                    </td>
                                                                </tr>
                                                                <tr class="sorttablenobg" <?php if($ConfigInfo['lan/dhcpserver'] == 0) echo 'style="display: none"'; ?>>
                                                                    <td width="241">Lease Time</td>
                                                                    <td colspan="2" class="inputcol">
                                                                        <input name="txt_DHCPLeaseTime" type="text" class="copy" id="txt_DHCPLeaseTime" value="<?php echo htmlspecialchars($ConfigInfo['lan/dhcpleasetime']); ?>" size="19" maxlength="64"/>
                                                                            <select name="dropdown_DHCPunit" id="dropdown_DHCPunit">
                                                                                <option <?php if($ConfigInfo['lan/dhcpleaseunits'] == "h") echo 'selected'; ?> value="h">h</option>
                                                                                <option <?php if($ConfigInfo['lan/dhcpleaseunits'] == "m") echo 'selected'; ?> value="m">m</option>
                                                                                <option <?php if($ConfigInfo['lan/dhcpleaseunits'] == "s") echo 'selected'; ?> value="s">s</option>
                                                                            </select>
                                                                    </td>
                                                                </tr>
                                                            </table>
                                                            <table id="tblStaticLease" border="0" cellspacing="0" cellpadding="0" width="100%">
                                                                <tr <?php if($ConfigInfo['lan/dhcpserver'] == 0) echo 'style="display: none"'; ?>>
                                                                    <td height="25" colspan="7" bgcolor="#3399CC" class="STYLE1" id="tdStaticLease">
                                                                        <b>Static Lease Table</b>
                                                                    </td>
                                                                </tr>
                                                                <?php
                                                                    for ($i = 0; $i < $ConfigInfo["lan/dhcpleasenum"]; $i++) {
                                                                        echo '<tr class=';
                                                                        if ($i % 2 == 1) {
                                                                            echo '"sorttablenobg"';
                                                                        } else {
                                                                            echo '"sorttable"';
                                                                        }
                                                                        if ($ConfigInfo['lan/dhcpserver'] == 0) {
                                                                            echo ' style="display: none';
                                                                        }
                                                                        echo '"><td width="25" class="inputcol"><input name = "ck_dhcp_del' . $i . '" id="ck_dhcp_del' . $i . '" type="checkbox" class="copy"/></td> ';
                                                                        echo '<td width="90">Host Name</td>';
                                                                        echo '<td width="100" class="inputcol"><input name="txt_dhcp_hostname' . $i . '" id="txt_dhcp_hostname' . $i . '" type="text" class="copy" value="' . htmlspecialchars($ConfigInfo["lan/dhcplease" . $i][0]) . '" size="12" maxlength="12"/></td>';
                                                                        echo '<td width="90">MAC Address</td>';
                                                                        echo '<td width="100" class="inputcol"><input name="txt_dhcp_macaddr' . $i . '" id="txt_dhcp_macaddr' . $i . '" type="text" class="copy" value="' . htmlspecialchars($ConfigInfo["lan/dhcplease" . $i][1]) . '" size="17" maxlength="17"/></td>';
                                                                        echo '<td width="100">IP Address</td>';
                                                                        echo '<td class="inputcol"><input name="txt_dhcp_ipaddr' . $i . '" id="txt_dhcp_ipaddr' . $i . '" type="text" class="copy" value="' . htmlspecialchars($ConfigInfo["lan/dhcplease" . $i][2]) . '" size="15" maxlength="15"/></td>';
                                                                    }
                                                                ?>
                                                            </table>
                                                            <tr class="sorttable" <?php if($ConfigInfo['lan/dhcpserver'] == 0) echo 'style="display: none"'; ?>>
                                                                <td>
                                                                    <input name="dhcp_addtolist" type="button" id="dhcp_addtolist" value="Add Entry to Lease Table List" onClick="addLeaseTableEntry();" class="tablehead"/>
                                                                    <input name="dhcp_removefromlist" type="button" id="dhcp_removefromlist" value="Remove Selected Entries" onClick="removeLeaseTableEntry();" class="tablehead"/>
                                                                </td>
                                                            </tr>
                                                            <tr id="hidTableLen" style="display:none">
                                                                <td>
                                                                    <input name="dhcpTableLen" type="text" id="dhcpTableLen" value="<?php echo htmlspecialchars($ConfigInfo["lan/dhcpleasenum"]); ?>">
                                                                </td>
                                                            </tr>
                                                            <tr>
                                                                <td height="4" colspan="5" align="left"></td>
                                                            </tr>
                                                            <tr>
                                                                <td height="4" colspan="5" align="left"></td>
                                                            </tr>
                                                        </td>
                                                    </tr>
                                                    <tr>
                                                      <td><table width="100%" border="0" cellpadding="4" cellspacing="0">
                                                        <tr class="tablehead"> <td colspan="3">SIM Settings</td> </tr>
                                                        <tr> <td height="4" colspan="5" align="left" bgcolor="#4d92a8"></td> </tr>
                                                        <tr class="sorttable">
                                                          <td width="241">SIM Status</td>
                                                          <td colspan="2" class="inputcol">
                                                            <?php echo $cellStatus; ?>
                                                          </td>
                                                        </tr>
                                                        <tr class="sorttablenobg" <?php if($simPresent[0]) echo 'style="display: none"'; ?>>
                                                          <td width="241">PIN Verification</td>
                                                          <td colspan="2" class="inputcol">
                                                            <select <?php if($simStatus[0] != "READY") echo 'disabled'; ?> name="dropdown_PinEnable" id="dropdown_PinEnable" onchange="check_dropdownPin()">
                                                              <option <?php if($pinStatus[0] != 1) echo 'selected'; ?> value="0">Disable</option>
                                                              <option <?php if($pinStatus[0] == 1) echo 'selected'; ?> value="1">Enable</option>
                                                            </select>
                                                          </td>
                                                        </tr>
                                                        <tr class="sorttable" <?php if(($simStatus[0] == "SIM PUK") || ($pinStatus[0] != 1)) echo 'style="display: none"'; ?>>
                                                          <td width="241">PIN Code</td>
                                                          <td colspan="2" class="inputcol">
                                                            <input name="txt_simPin" type="password" class="copy" id="txt_simPin" value="<?php echo htmlspecialchars($ConfigInfo['sim/pin']); ?>" size="40" maxlength="64"/>
                                                            <input name="changePIN" <?php if (($simStatus[0] != "READY") || ($showNewPin == 1)) echo 'style="display: none"'; ?> type="button" class="copy" id="changePIN" value="Change" onClick="changePINjs();"/>
                                                          </td>
                                                        </tr>
                                                        <tr class="sorttable" <?php if($simStatus[0] != "SIM PUK") echo 'style="display: none"'; ?>>
                                                          <td width="241">PUK Code</td>
                                                          <td colspan="2" class="inputcol">
                                                            <input name="txt_simPuk" type="password" class="copy" id="txt_simPuk" size="40" maxlength="64"/>
                                                          </td>
                                                        </tr>
                                                        <tr class="sorttablenobg" <?php if(($simStatus[0] != "SIM PUK") && ($showNewPin == 0)) echo 'style="display: none"'; ?>>
                                                          <td width="241">New PIN Code</td>
                                                          <td colspan="2" class="inputcol">
                                                            <input name="txt_simPin_new" type="password" class="copy" id="txt_simPin_new" size="40" maxlength="64"/>
                                                            <input name="cancelChangePIN" <?php if ($simStatus[0] == "SIM PUK") echo 'style="display: none"'; ?> type="button" class="copy" id="cancelChangePIN" value="Cancel" onClick="cancelChangePINjs();"/>
                                                          </td>
                                                        </tr>
                                                        <tr class="sorttablenobg" <?php if($pinStatus[0] != 1) echo 'style="display: none"'; ?>>
                                                          <td width="241">Show characters</td>
                                                          <td colspan="2" class="inputcol">
                                                            <input name="showPinCB" id="showPinCB" type="checkbox" class="copy" onClick="showPINjs()"/>
                                                          </td>
                                                        </tr>
                                                        <tr class="sorttable" <?php if($pinStatus[0] != 1) echo 'style="display: none"'; ?>>
                                                          <td width="241">Save PIN</td>
                                                          <td colspan="2" class="inputcol">
                                                            <input name="savePinCB" id="savePinCB" type="checkbox" class="copy" <?php if(strlen($ConfigInfo['sim/pin']) > 0) echo 'checked'; ?>/>
                                                          </td>
                                                        </tr>
                                                      </table></td>
                                                    </tr>
                                                    <tr>
                                                        <td>&nbsp;</td>
                                                    </tr>
                                                    <tr>
                                                        <td>
                                                            <table width="100%" border="0" cellspacing="0" cellpadding="0">
                                                                <tr>
                                                                    <td>
                                                                        <input name="Apply" type="submit" id="Apply" value="Apply" class="copy">
                                                                    </td>
                                                                </tr>
                                                            </table>
                                                        </td>
                                                    </tr>
                                                    <tr>
                                                        <td>&nbsp;</td>
                                                    </tr>
                                                </table>
                                            </form>
                                        </td>
                                    </tr>
                                </table>
                            </td>
                        </tr>
                    </table>
                    <?php /* <!--End content here --> */ ?>
                </td>
                <td background="/images/bgright.jpg">&nbsp;</td>
            </tr>
            <tr height=16>
                <td >
                    <img src="/images/leftbtmangle.jpg" />
                </td>
                <td background="/images/bgbtm.jpg"></td>
                <td >
                    <img src="/images/rightbtmangle.jpg" />
                </td>
            </tr>
        </table>
        <?php /* <!--End body --> */ ?>
        <SCRIPT>
            <?php //Set the top menu style
            //alert(window.parent.document.getElementById("leftlink_configuration_basic")); ?>
            window.parent.document.getElementById("leftlink_configuration_basic").className = "leftNavLinkActive";
            window.parent.document.getElementById("leftlink_configuration_advance").className = "leftNavLink";
            window.parent.document.getElementById("leftlink_configuration_firewall").className = "leftNavLink";
            var errcnt = <?php echo $validate_err;?>;
            var errarray<?php
                if ($validate_err > 0) {
                    echo '=[';
                }
                for ($i = 0; $i < $validate_err; $i++) {
                    if ($i != 0 ) {
                        echo ',';
                    }
                    echo "\"$errArray[$i]\"";
                }
                if ($validate_err > 0) {
                    echo ']';
                }
            ?>;

            HighlightValidationErrors(errcnt, errarray);
        </SCRIPT>
    </body>
</html>
