<?php
    require_once('../inc/systemfunction.php');
    require_once('../inc/json_services.php');

    class PortForwardingEntry {
        public $Application = '';
        public $Protocol = '';
        public $IpAddress = '';
        public $FromBeginPort = '';
        public $FromEndPort = '';
        public $ToBeginPort = '';
        public $ToEndPort = '';
        public $ExceptionPorts = '';

        function setApplication($value) {
            $this->Application = $value;
        }

        function setProtocol($value) {
            $this->Protocol = $value;
        }

        function setIpAddress($value) {
            $this->IpAddress = $value;
        }

        function setFromBeginPort($value) {
            $this->FromBeginPort = $value;
        }

        function setFromEndPort($value) {
            $this->FromEndPort = $value;
        }

        function setToBeginPort($value) {
            $this->ToBeginPort = $value;
        }

        function setToEndPort($value) {
            $this->ToEndPort = $value;
        }
    }

    class DeviceFirewallHandler {
        public $encoded = '';
        
        function getConfiguration() {
            /* Variable initialisation */
            $error = '';
            $enablePassThrough = 'false';
            $endDeviceAddress = '';
            $enablePortFw = 'false';
            $portsFw = array();
            $lanIpAddr = '';
            $lanBroadcast = '';
            $lanNetwork = '';
            $portsEx = array();

            try {
                $enablePassThrough = ConfigRead('lan/passthrough');
                $endDeviceAddress = ConfigRead('lan/devipadr0');
                $enablePortFw = ConfigRead('firewall/enableportfw');
                $lanIpAddr = ConfigRead('lan/ipadr');
                $lanBroadcast = GetLanBroadcast();
                $lanNetwork = GetLanNetwork();

                /**
                 * Retrieve the list of restricted ports in the current 
                 * configuration
                 */
                if (strpos(GetBoardFWVer(), "ssh")) {
                    array_push($portsEx, ConfigRead('ftp/controlport'));
                    array_push($portsEx, ConfigRead('ssh/port'));
                }
                
                if ("0" != ConfigRead('encap/mode')[0]) {
                    array_push($portsEx, ConfigRead('encap/listenport'));
                }

                if ("0" != ConfigRead('ovpn/mode')[0]) {
                    array_push($portsEx, ConfigRead('ovpn/port'));
                }

                if ("0" != ConfigRead('ipsec/mode')[0]) {
                    array_push($portsEx, ConfigRead('ipsec/L2TPport'));
                    array_push($portsEx, ConfigRead('ipsec/IKEport'));
                }

                if ("0" != ConfigRead('system/ntpenable')[0]) {
                    array_push($portsEx, ConfigRead('system/ntpport'));
                }
                array_push($portsEx, ConfigRead('dns/port'));
                array_push($portsEx, ConfigRead('web/port'));
                array_push($portsEx, ConfigRead('discserv/port'));

                for ($i = 0; $i <= 9; $i++) {
                    /* Read from configuration file */
                    $value = ConfigRead('firewall/portfw'.$i);

                    if (0 < sizeof($value))  {      
                        /* Object to store the values */
                        $fwPort = new PortForwardingEntry();

                        /* Deserialise the config settings */
                        $config_vector = explode(';',$value[0]);
                        $fwPort->setApplication($config_vector[0]);
                        $fwPort->setProtocol($config_vector[1]);
                        $fwPort->setFromBeginPort($config_vector[2]);
                        $fwPort->setFromEndPort($config_vector[3]);
                        $fwPort->setIpAddress($config_vector[4]);
                        $fwPort->setToBeginPort($config_vector[5]);
                        $fwPort->setToEndPort($config_vector[6]);

                        array_push($portsFw, $fwPort);
                    }
                }
            }
            catch(Exception $e) {
                $error = $e->getMessage();
                $enablePortFw = 0;
            }

            $json = new Services_JSON(); 
            $response = array();
            $response['EnablePassThrough'] = $enablePassThrough;
            $response['EndDeviceAddr'] = $endDeviceAddress;
            $response['EnablePortFw'] = $enablePortFw;
            $response['LanIpAddr'] = $lanIpAddr;
            $response['PortFwValues'] = $portsFw;
            $response['PortExValues'] = $portsEx;
            $response['LanBroadcast'] = $lanBroadcast;
            $response['LanNetwork'] = $lanNetwork;
            $response['Error'] = $error;
            $encoded = $json->encode($response);
            return $encoded;
        }
    }

    /* Send response back to java and end script execution */
    die((new DeviceFirewallHandler()) -> getConfiguration());
?>