<?php
    require_once('common_functions.php');
    require_once('class.parseProgs.inc.php');

    abstract class serviceAction {
        const None      = 0;
        const Stop      = 1;
        const Start     = 2;
        const Restart   = 3;
        const Reboot    = 4;
    }

	/*
	 * Wrapper for array_walk callback function
	 * which accepts two arguments to avoid php warning
	 */
	function awEscapeShellCmd($value, $key)
	{
			/* key is not used */
			escapeshellcmd($value);
	}

    /*
     * Function:    safeExec
     *
     * Summary:     Construct and execute a sanitized command
     *
     * Parameters:  cmdFormat - string containing the command template
     *              retType - content to be returned by function
     *              args - variable number of arguments to complete template
     *
     * Note:        Does not verify for the correct number of arguments
     */
    function safeExec($cmdFormat, $retType, ...$args) {
        /* Sanitize elements in "args" array */
        array_walk($args, 'awEscapeShellCmd');

        /* Construct final "exec" command */
        $cmdString = vsprintf($cmdFormat, $args);

        /* Execute sanitized command */
        exec($cmdString, $output, $retval);

        switch ($retType) {
            case 0:
                /* Return $output of the exec command */
                return $output;
                break;
            case 1:
                /* Return exit status of the exec command */
                return $retval;
                break;
            case 2:
                /* Prepend exit status to the output array and return output */
                array_push($output, $retval);
                return $output;
                break;
        }
    }

    // get our apache SERVER_NAME or vhost
    function vhostname () {
        if (! ($result = getenv('SERVER_NAME'))) {
            $result = 'N.A.';
        }
        return $result;
    }

    // get the IP address of our vhost name
    function vip_addr () {
        return gethostbyname(vhostname());
    }

    // get the ifconfiginfo of module
    function ifconfiginfo () {
        $results = array();
        $return_array = array();
        $mac_addr;
        @exec("ifconfig ", $return_array);
        //$return_array = @system("ifconfig -a");
        //echo $return_array;
        $temp_array = array();
        foreach ($return_array as $value) {
            if (' ' != ($value[0])) {
                $ifId = sscanf($value, "%s");
            }
             //echo $value;
            if (preg_match("/[0-9a-f][0-9a-f][:-]"."[0-9a-f][0-9a-f][:-]"."[0-9a-f][0-9a-f][:-]"."[0-9a-f][0-9a-f][:-]"."[0-9a-f][0-9a-f][:-]"."[0-9a-f][0-9a-f]/i", $value, $temp_array)) {
                $results[$ifId[0]]['macaddress'] = $temp_array[0];
                //echo $temp_array[0];
           }
            if (preg_match_all('/(\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})/', $value , $temp_arrayip)) {
                $results[$ifId[0]]['ipaddress'] = $temp_arrayip[1][0];
                $results[$ifId[0]]['getway'] = $temp_arrayip[1][1];
                $results[$ifId[0]]['subnetmask'] = $temp_arrayip[1][2];
           }
        }
        unset($temp_array);
        unset($temp_arrayip);
        unset($return_array);
        return $results;
    }

    // get our canonical hostname
    function chostname () {
        $result = rfts( '/proc/sys/kernel/hostname', 1 );
        if ($result == "ERROR") {
            $result = "N.A.";
        } else {
            //$result = gethostbyaddr( gethostbyname( trim( $result ) ) );
        }
        return $result;
    }

    // get the IP address of our canonical hostname
    function ip_addr () {
        if (!($result = getenv('SERVER_ADDR'))) {
            $result = gethostbyname(chostname());
        }
        return $result;
    }

    function kernel () {
        $buf = rfts( '/proc/version', 1);
        if ($buf == "ERROR") {
            $result = "N.A.";
        } else {
            if (preg_match('/version (.*?) /', $buf, $ar_buf)) {
                $result = $ar_buf[1];
                if (preg_match('/SMP/', $buf)) {
                    $result .= ' (SMP)';
                }
            }
        }
        return $result;
    }

    function uptime () {
        $buf = rfts( '/proc/uptime', 1 );
        $ar_buf = preg_split( '/ +/', $buf);
        $result = trim( $ar_buf[0]);

        return $result;
    }

    function users () {
       $strResult = 0;
       $strBuf = execute_program('who', '-q');
       if ($strBuf != "ERROR") {
            $arrWho = preg_split( '=', $strBuf);
            $strResult = $arrWho[1];
        }
        return $strResult;
    }

    function loadavg ($bar = false) {
        $buf = rfts( '/proc/loadavg' );
        if ($buf == "ERROR") {
            $results['avg'] = array('N.A.', 'N.A.', 'N.A.');
        } else {
            $results['avg'] = preg_split("/\s/", $buf, 4);
            unset($results['avg'][3]);  // don't need the extra values, only first three
        }
        if ($bar) {
            $buf = rfts( '/proc/stat', 1 );
            if($buf != "ERROR") {
                sscanf($buf, "%*s %Ld %Ld %Ld %Ld", $ab, $ac, $ad, $ae);
                // Find out the CPU load
                // user + sys = load
                // total = total
                $load = $ab + $ac + $ad;    // cpu.user + cpu.sys
                $total = $ab + $ac + $ad + $ae; // cpu.total

                // we need a second value, wait 1 second befor getting (< 1 second no good value will occour)
                sleep(1);
                $buf = rfts( '/proc/stat', 1);
                sscanf($buf, "%*s %Ld %Ld %Ld %Ld", $ab, $ac, $ad, $ae);
                $load2 = $ab + $ac + $ad;
                $total2 = $ab + $ac + $ad + $ae;
                $results['load'] = $load2 - $load;
                $results['total'] = $total2 - $total;
                $results['cpupercent'] = (100*($load2 - $load)) / ($total2 - $total);
            }
        }
        return $results;
    }

    function cpu_info () {
        $bufr = rfts( '/proc/cpuinfo');
        $results = array("cpus" => 0);

        if ( $bufr != "ERROR" ) {
            $bufe = explode("\n", $bufr);

            $results = array('cpus' => 0, 'bogomips' => 0);
            $ar_buf = array();

            foreach ( $bufe as $buf ) {
                $arrBuff = preg_split('/\s+:\s+/', trim($buf));
                if ( count( $arrBuff ) == 2) {
                    $key = $arrBuff[0];
                    $value = $arrBuff[1];
                    // All of the tags here are highly architecture dependant.
                    // the only way I could reconstruct them for machines I don't
                    // have is to browse the kernel source.  So if your arch isn't
                    // supported, tell me you want it written in.
                    switch ($key) {
                        case 'model name':
                            $results['model'] = $value;
                            break;
                        case 'cpu MHz':
                            $results['cpuspeed'] = sprintf('%.2f', $value);
                            break;
                        case 'cycle frequency [Hz]': // For Alpha arch - 2.2.x
                            $results['cpuspeed'] = sprintf('%.2f', $value / 1000000);
                            break;
                        case 'clock': // For PPC arch (damn borked POS)
                            $results['cpuspeed'] = sprintf('%.2f', $value);
                            break;
                        case 'cpu': // For PPC arch (damn borked POS)
                            $results['model'] = $value;
                            break;
                        case 'L2 cache': // More for PPC
                            $results['cache'] = $value;
                            break;
                        case 'revision': // For PPC arch (damn borked POS)
                            $results['model'] .= ' ( rev: ' . $value . ')';
                            break;
                        case 'cpu model': // For Alpha arch - 2.2.x
                            $results['model'] .= ' (' . $value . ')';
                            break;
                        case 'cache size':
                            $results['cache'] = $value;
                            break;
                        case 'bogomips':
                            $results['bogomips'] += $value;
                            break;
                        case 'BogoMIPS': // For alpha arch - 2.2.x
                            $results['bogomips'] += $value;
                            break;
                        case 'BogoMips': // For sparc arch
                            $results['bogomips'] += $value;
                            break;
                        case 'cpus detected': // For Alpha arch - 2.2.x
                            $results['cpus'] += $value;
                            break;
                        case 'system type': // Alpha arch - 2.2.x
                            $results['model'] .= ', ' . $value . ' ';
                            break;
                        case 'platform string': // Alpha arch - 2.2.x
                            $results['model'] .= ' (' . $value . ')';
                            break;
                        case 'processor':
                            $results['cpus'] += 1;
                            break;
                        case 'Cpu0ClkTck': // Linux sparc64
                            $results['cpuspeed'] = sprintf('%.2f', hexdec($value) / 1000000);
                            break;
                        case 'Cpu0Bogo': // Linux sparc64 & sparc32
                            $results['bogomips'] = $value;
                            break;
                        case 'ncpus probed': // Linux sparc64 & sparc32
                            $results['cpus'] = $value;
                            break;
                    }
                }
            }

            // sparc64 specific code follows
            // This adds the ability to display the cache that a CPU has
            // Originally made by Sven Blumenstein <bazik@gentoo.org> in 2004
            // Modified by Tom Weustink <freshy98@gmx.net> in 2004
            $sparclist = array('SUNW,UltraSPARC@0,0', 'SUNW,UltraSPARC-II@0,0', 'SUNW,UltraSPARC@1c,0', 'SUNW,UltraSPARC-IIi@1c,0', 'SUNW,UltraSPARC-II@1c,0', 'SUNW,UltraSPARC-IIe@0,0');
            foreach ($sparclist as $name) {
                $buf = rfts( '/proc/openprom/' . $name . '/ecache-size',1 , 32, false);
                if ( $buf != "ERROR" ) {
                    $results['cache'] = base_convert($buf, 16, 10)/1024 . ' KB';
                }
            }
            // sparc64 specific code ends

            // XScale detection code
            if ($results['cpus'] == 0) {
                foreach ( $bufe as $buf ) {
                    $fields = preg_split('/\s*:\s*/', trim($buf), 2);
                    if (sizeof($fields) == 2) {
                        list($key, $value) = $fields;
                        switch ($key) {
                            case 'Processor':
                                $results['cpus'] += 1;
                                $results['model'] = $value;
                                break;
                            case 'BogoMIPS': //BogoMIPS are not BogoMIPS on this CPU, it's the speed, no BogoMIPS available
                                $results['cpuspeed'] = $value;
                                break;
                            case 'I size':
                                $results['cache'] = $value;
                                break;
                            case 'D size':
                                $results['cache'] += $value;
                                break;
                        }
                    }
                }
                $results['cache'] = $results['cache'] / 1024 . " KB";
            }
        }
        $keys = array_keys($results);
        $keys2be = array('model', 'cpuspeed', 'cache', 'bogomips', 'cpus');

        while ($ar_buf = each($keys2be)) {
            if (! in_array($ar_buf[1], $keys)) {
                $results[$ar_buf[1]] = 'N.A.';
            }
        }

        $buf = rfts('/proc/acpi/thermal_zone/THRM/temperature', 1, 4096, false);
        if ( $buf != "ERROR" ) {
            $results['temp'] = substr($buf, 25, 2);
        }

        return $results;
    }

    function pci () {
        $arrResults = array();
        $booDevice = false;

        if (! $arrResults = $this->parser->parse_lspci()) {
            $strBuf = rfts( '/proc/pci', 0, 4096, false );
            if ($strBuf != "ERROR") {
                $arrBuf = explode("\n", $strBuf);
                foreach ($arrBuf as $strLine) {
                    if (preg_match('/Bus/', $strLine)) {
                        $booDevice = true;
                        continue;
                    }
                    if($booDevice) {
                        list($strKey, $strValue) = preg_split(': ', $strLine, 2);
                        if(! preg_match( '/bridge/i', $strKey) && ! preg_match('/USB/i ', $strKey)) {
                            $arrResults[] = preg_replace( '/\([^\)]+\)\.$/', '', trim($strValue));
                        }
                        $booDevice = false;
                    }
                }
                asort($arrResults);
            }
        }
        return $arrResults;
    }

    function ide () {
        $results = array();
        $bufd = gdc( '/proc/ide', false );

        foreach ($bufd as $file) {
            if (preg_match('/^hd/', $file)) {
                $results[$file] = array();
                $buf = rfts("/proc/ide/" . $file . "/media", 1);
                if ($buf != "ERROR") {
                    $results[$file]['media'] = trim($buf);
                    if ($results[$file]['media'] == 'disk') {
                        $results[$file]['media'] = 'Hard Disk';
                        $buf = rfts( "/proc/ide/" . $file . "/capacity", 1, 4096, false);
                        if($buf == "ERROR") {
                            $buf = rfts( "/sys/block/" . $file . "/size", 1, 4096, false);
                        }
                        if ($buf != "ERROR") {
                           $results[$file]['capacity'] = trim($buf);
                        }
                    } elseif ($results[$file]['media'] == 'cdrom') {
                        $results[$file]['media'] = 'CD-ROM';
                        unset($results[$file]['capacity']);
                    }
                } else {
                    unset($results[$file]);
                }

                $buf = rfts("/proc/ide/" . $file . "/model", 1);
                if ($buf != "ERROR") {
                    $results[$file]['model'] = trim($buf);
                    if (preg_match('/WDC/', $results[$file]['model'])) {
                        $results[$file]['manufacture'] = 'Western Digital';
                    } elseif (preg_match('/IBM/', $results[$file]['model'])) {
                        $results[$file]['manufacture'] = 'IBM';
                    } elseif (preg_match('/FUJITSU/', $results[$file]['model'])) {
                        $results[$file]['manufacture'] = 'Fujitsu';
                    } else {
                        $results[$file]['manufacture'] = 'Unknown';
                    }
                }
            }
        }

        asort($results);
        return $results;
    }

    function scsi () {
        $results = array();
        $dev_vendor = '';
        $dev_model = '';
        $dev_rev = '';
        $dev_type = '';
        $s = 1;
        $get_type = 0;

        $bufr = execute_program('lsscsi', '-c', false);
        if ($bufr == "ERROR") {
           $bufr = rfts( '/proc/scsi/scsi', 0, 4096, false);
        }
        if ($bufr != "ERROR") {
            $bufe = explode("\n", $bufr);
            foreach($bufe as $buf) {
                if (preg_match('/Vendor/', $buf)) {
                    preg_match('/Vendor: (.*) Model: (.*) Rev: (.*)/i', $buf, $dev);
                    list($key, $value) = preg_split(': ', $buf, 2);
                    $dev_str = $value;
                    $get_type = true;
                    continue;
                }

                if ($get_type) {
                    preg_match('/Type:\s+(\S+)/i', $buf, $dev_type);
                    $results[$s]['model'] = "$dev[1] $dev[2] ($dev_type[1])";
                    $results[$s]['media'] = "Hard Disk";
                    $s++;
                    $get_type = false;
                }
            }
        }
        asort($results);
        return $results;
    }

    function usb () {
        $results = array();
        $devnum = -1;

        $bufr = execute_program('lsusb', '', false);
        if ($bufr == "ERROR") {
            $bufr = rfts('/proc/bus/usb/devices', 0, 4096, false);
            if ($bufr != "ERROR") {
                $bufe = explode("\n", $bufr);
                foreach ($bufe as $buf) {
                   if (preg_match('/^T/', $buf)) {
                        $devnum += 1;
                        $results[$devnum] = "";
                    } elseif (preg_match('/^S:/', $buf)) {
                        list($key, $value) = preg_split(': ', $buf, 2);
                        list($key, $value2) = preg_split('=', $value, 2);
                        if (trim($key) != "SerialNumber") {
                            $results[$devnum] .= " " . trim($value2);
                            $devstring = 0;
                        }
                    }
                }
            }
        } else {
            $bufe = explode( "\n", $bufr );
            foreach ($bufe as $buf) {
                $device = preg_split("/ /", $buf, 7);
                if (isset($device[6]) && trim($device[6] ) != "") {
                    $results[$devnum++] = trim($device[6]);
                }
            }
        }
        return $results;
    }

    function sbus () {
        $results = array();
        $_results[0] = "";
        // TODO. Nothing here yet. Move along.
        $results = $_results;
        return $results;
    }

    function network () {
        $results = array();

        $bufr = rfts('/proc/net/dev');
        if ($bufr != "ERROR") {
            $bufe = explode("\n", $bufr);
            foreach ($bufe as $buf) {
                if (preg_match('/:/', $buf)) {
                    list($dev_name, $stats_list) = preg_split('/:/', $buf, 2);
                    //echo $dev_name;

                    $stats = preg_split('/\s+/', trim($stats_list));
                    $results[trim($dev_name)] = array("rx_bytes" => $stats[0],
                                                      "rx_packets" => $stats[1],
                                                      "rx_errs" => $stats[2],
                                                      "rx_drop" => $stats[3],
                                                      "tx_bytes" => $stats[8],
                                                      "tx_packets" => $stats[9],
                                                      "tx_errs" => $stats[10],
                                                      "tx_drop" => $stats[11],
                                                      "errs" => $stats[2] + $stats[10],
                                                      "drop" => $stats[3] + $stats[11]);
                }
            }
        }
        return $results;
    }

    function memory () {
        $results['ram'] = array('total' => 0, 'free' => 0, 'used' => 0, 'percent' => 0);
        $results['swap'] = array('total' => 0, 'free' => 0, 'used' => 0, 'percent' => 0);
        $results['devswap'] = array();

        $bufr = rfts('/proc/meminfo');
        if ($bufr != "ERROR") {
            $bufe = explode("\n", $bufr);
            foreach ( $bufe as $buf ) {
                if (preg_match('/^MemTotal:\s+(.*)\s*kB/i', $buf, $ar_buf)) {
                    $results['ram']['total'] = $ar_buf[1];
                } else if (preg_match('/^MemFree:\s+(.*)\s*kB/i', $buf, $ar_buf)) {
                    $results['ram']['free'] = $ar_buf[1];
                } else if (preg_match('/^Cached:\s+(.*)\s*kB/i', $buf, $ar_buf)) {
                    $results['ram']['cached'] = $ar_buf[1];
                } else if (preg_match('/^Buffers:\s+(.*)\s*kB/i', $buf, $ar_buf)) {
                    $results['ram']['buffers'] = $ar_buf[1];
                }
            }

            $results['ram']['used'] = $results['ram']['total'] - $results['ram']['free'];
            $results['ram']['percent'] = round(($results['ram']['used'] * 100) / $results['ram']['total']);

            // values for splitting memory usage
            if (isset($results['ram']['cached']) && isset($results['ram']['buffers'])) {
                $results['ram']['app'] = $results['ram']['used'] - $results['ram']['cached'] - $results['ram']['buffers'];
                $results['ram']['app_percent'] = round(($results['ram']['app'] * 100) / $results['ram']['total']);
                $results['ram']['buffers_percent'] = round(($results['ram']['buffers'] * 100) / $results['ram']['total']);
                $results['ram']['cached_percent'] = round(($results['ram']['cached'] * 100) / $results['ram']['total']);
            }

            $bufr = rfts( '/proc/swaps' );
            if ( $bufr != "ERROR" ) {
                $swaps = explode("\n", $bufr);
                for ($i = 1; $i < (sizeof($swaps)); $i++) {
                    if ( trim( $swaps[$i] ) != "" ) {
                        $ar_buf = preg_split('/\s+/', $swaps[$i], 6);
                        $results['devswap'][$i - 1] = array();
                        $results['devswap'][$i - 1]['dev'] = $ar_buf[0];
                        $results['devswap'][$i - 1]['total'] = $ar_buf[2];
                        $results['devswap'][$i - 1]['used'] = $ar_buf[3];
                        $results['devswap'][$i - 1]['free'] = ($results['devswap'][$i - 1]['total'] - $results['devswap'][$i - 1]['used']);
                        $results['devswap'][$i - 1]['percent'] = round(($ar_buf[3] * 100) / $ar_buf[2]);
                        $results['swap']['total'] += $ar_buf[2];
                        $results['swap']['used'] += $ar_buf[3];
                        $results['swap']['free'] = $results['swap']['total'] - $results['swap']['used'];
                        $results['swap']['percent'] = round(($results['swap']['used'] * 100) / $results['swap']['total']);
                    }
                }
            }
        }
        return $results;
    }

    function text_cpuusage () {
        $intScalefactor = 1;
        $cpuLoad = loadavg(true);
        $strcpu = create_bargraph($cpuLoad['load'], $cpuLoad['total'], $intScalefactor);
        $strcpu .= "&nbsp;&nbsp;" . $cpuLoad['cpupercent'] . "% ";
        $_text = "<table width=\"650\">"
                ."<tr>"
                ."<td width=\"230\" align=\"left\" valign=\"top\">CPU Load</td>"
                ."<td align=\"left\" valign=\"top\">".$strcpu."</td>"
                ."</tr>";
        $_text .= "</table>";

        return $_text;
    }

    function text_memoryusage() {
        $arrMem = memory();
        $intScalefactor = 1;

        $strRam = create_bargraph( $arrMem['ram']['used'], $arrMem['ram']['total'], $intScalefactor );
        $strRam .= "&nbsp;&nbsp;" . $arrMem['ram']['percent'] . "% ";

        $strSwap = create_bargraph( $arrMem['swap']['used'], $arrMem['swap']['total'], $intScalefactor );
        $strSwap .= "&nbsp;&nbsp;" . $arrMem['swap']['percent'] . "% ";

        $_text = "<table width=\"650\">"
                ."<tr>"
                ."<td width=\"230\" align=\"left\" valign=\"top\"><b>Type</b></td>"
                ."<td width=\"185\" align=\"left\" valign=\"top\"><b>Percent Capacity</b></td>"
                ."<td align=\"right\" valign=\"top\"><b>Free</b></td>"
                ."<td align=\"right\" valign=\"top\"><b>Used</b></td>"
                ."<td align=\"right\" valign=\"top\"><b>Size</b></td>"
                ."</tr>";

        $_text .= "<tr>";
        $_text .= "<td align=\"left\" valign=\"top\">Ram</td>"
                ."<td align=\"left\" valign=\"top\">".$strRam."</td>"
                ."<td align=\"right\">".format_bytesize($arrMem['ram']['free'])."</td>"
                ."<td align=\"right\">".format_bytesize($arrMem['ram']['used'])."</td>"
                ."<td align=\"right\">".format_bytesize($arrMem['ram']['total'])."</td>";
        $_text .= "<tr>";

        if (isset($arrMem['swap']['total']) && $arrMem['swap']['total'] > 10) {
            $_text .= "<tr>";
            $_text .="<td align=\"left\" valign=\"top\">Swap</td>"
                    ."<td align=\"left\" valign=\"top\">".$strSwap."</td>"
                    ."<td align=\"right\">". format_bytesize( $arrMem['swap']['free'])."</td>"
                    ."<td align=\"right\">". format_bytesize( $arrMem['swap']['used'])."</td>"
                    ."<td align=\"right\">". format_bytesize( $arrMem['swap']['total'])."</td>";
            $_text .= "<tr>";
        }
       
        // foreach ( $arrMem['devswap'] as $arrDevice) {
        //     $_text .= "<tr>";
        //     $_text .= "<td lign=\"left\" valign=\"top\">".$arrDevice['dev']."</td>"
        //             ."<td align=\"left\" valign=\"top\">". format_bytesize( $arrDevice['free'])."</td>"
        //             ."<td align=\"right\">". format_bytesize( $arrDevice['free'])."</td>"
        //             ."<td align=\"right\">". format_bytesize( $arrDevice['used'])."</td>"
        //             ."<td align=\"right\">". format_bytesize( $arrDevice['total'])."</td>";
        //     $_text .= "<tr>";
        // }
       
        $_text .= "</table>";

        return $_text;
    }

    function filesystems () {
        $parser = new Parser();
        return $parser->parse_filesystems();
    }

    function text_filesystems () {
       //global $show_mount_point;
       $arrFs = filesystems ();
       $intScalefactor = 1;

       $_text = "<table width=\"650\">"
                ."<tr>"
                ."<td width=\"110\" align=\"left\" valign=\"top\"><b>Mount</b></td>"
                ."<td width=\"40\" align=\"left\" valign=\"top\"><b>Type</b></td>"
                ."<td width=\"80\" align=\"left\" valign=\"top\"><b>Partition</b></td>"
                ."<td width=\"185\" align=\"left\" valign=\"top\"><b>Percent Capacity</b></td>"
                ."<td align=\"right\" valign=\"top\"><b>Free</b></td>"
                ."<td align=\"right\" valign=\"top\"><b>Used</b></td>"
                ."<td align=\"right\" valign=\"top\"><b>Size</b></td>"
               ."</tr>";

        for ($i = 0, $max = sizeof($arrFs); $i < $max; $i++) {
            $strPercent  = create_bargraph($arrFs[$i]['used'], $arrFs[$i]['size'], $intScalefactor);
            $strPercent .= "&nbsp;&nbsp;" .create_pencent($arrFs[$i]['used'], $arrFs[$i]['size'], $intScalefactor). "% ";

            $_text .= "<tr>";
            $_text .="<td width=\"25\" align=\"left\" valign=\"top\">" .htmlspecialchars($arrFs[$i]['mount'], ENT_QUOTES) . "\n</td>"
                    ."<td width=\"10\" align=\"left\" valign=\"top\">". htmlspecialchars( $arrFs[$i]['fstype'], ENT_QUOTES ) . "\n</td>"
                    ."<td width=\"25\" align=\"left\" valign=\"top\">". htmlspecialchars( $arrFs[$i]['disk'], ENT_QUOTES ) . "\n</td>"
                    ."<td align=\"left\" valign=\"top\">".$strPercent."\n</td>"
                    ."<td align=\"right\">". format_bytesize( $arrFs[$i]['free'])."</td>"
                    ."<td align=\"right\">". format_bytesize( $arrFs[$i]['used'])."</td>"
                    ."<td align=\"right\">". format_bytesize( $arrFs[$i]['size'])."</td>";
            $_text .= "<tr>";
        }
        $_text .= "</table>";

        return $_text;
    }

    // Owen add
    function text_signalleve ($signalval) {
        $intScalefactor = 1;
        $strsignallevel = create_bargraph($signalval, 31, $intScalefactor);
        $strsignallevel .= "&nbsp;&nbsp;" . round($signalval*100/31) . "%&nbsp;&nbsp;" . (2*$signalval-113) . "dBm";

        return $strsignallevel;
    }

    function graph_cellularusage($dataUsage, $dataLimit) {
        $intScalefactor = 1;
        $strcellularusage = create_bargraph( $dataUsage, $dataLimit, $intScalefactor );

        return $strcellularusage;
    }

    // Owen
    function ShowPingMessage($pingaddress, $pingtimes) {
        /* "ping" command template */
        $cmdTemplate = "ping '%s' -c '%s' 2>&1";
        /* Number of pings to be executed */
        $pingtimes = 4;

        $retval = safeExec($cmdTemplate, 0, $pingaddress, $pingtimes);

        return $retval;
    }

    function GetBoardFWVer() {
        exec('/bin/grep "VERSION=" /etc/os-release', $OS_release);
        $rel_len = strlen($OS_release[0]);
        return substr($OS_release[0], 9, $rel_len-10);
    }

    function ConfigRead($param) {
        /* Parameter "read" command template */
        $cmdTemplate = "/psft/bin/cfgModify read '%s'";

        $retval = safeExec($cmdTemplate, 0, $param);

        return $retval;
    }

    function ConfigReadList($param) {
        /* Parameter "read" command template */
        $cmdTemplate = "/psft/bin/cfgModify readlist '%s'";

        $retval = safeExec($cmdTemplate, 0, $param);

        return $retval;
    }

    function ConfigWrite($param, $value) {
        /* Parameter "write" command template */
        $cmdTemplate = "/psft/bin/cfgModify set '%s' '%s'";

        $retval = safeExec($cmdTemplate, 0, $param, $value);

        return $retval;
    }

    function ConfigToFlash() {
        exec("/psft/bin/cfgModify writetofile", $retval);

        return $retval;
    }

    function LoadFirewall() {
        exec("/www/lighttpdocs/scripts/systemfunction.sh loadfirewall", $retval);

        return $retval;
    }

    function ConfigRemove($param) {
        /* Parameter "remove" command template */
        $cmdTemplate = "/psft/bin/cfgModify remove '%s'";

        $retval = safeExec($cmdTemplate, 0, $retval);

        return $retval;
    }

    function ConfigDisplayGroup($param) {
        /* Parameter "group read" command template */
        $cmdTemplate = "/psft/bin/cfgModify displaygroup '%s'";

        $retval = safeExec($cmdTemplate, 0, $param);

        return $retval;
    }

    function NetfilterAddIP($param) {
        /* Netfilter "add to whitelist" command template */
        $cmdTemplate = "/psft/bin/nfcli inWANipv4WL '%s' add";

        $retval = safeExec($cmdTemplate, 0, $param);

        return $retval;
    }

    function NetfilterRemoveIP($param) {
        /* Netfilter "remove from whitelist" command template */
        $cmdTemplate = "/psft/bin/nfcli inWANipv4WL '%s' remove";

        $retval = safeExec($cmdTemplate, 0, $param);

        return $retval;
    }

    function RebootModule() {
        exec("reboot");
    }

    function getArrayIndex(&$ar, $index) {
        return $ar[$index];
    }

    function getLanBroadcast() {
        exec("/www/lighttpdocs/scripts/systemfunction.sh getlanbroadcast", $retval);

        return $retval;
    }

    function getLanNetwork() {
        exec("/www/lighttpdocs/scripts/systemfunction.sh getlannetwork", $retval);

        return $retval;
    }

    function serviceModify($serviceName, $serviceState) {
        switch ($serviceName) {
            case 'serviceKeepAlive':
                exec("/etc/init.d/S73-keepalive " . $serviceState, $retval);
                break;
            case 'serviceNTP':
                exec("/etc/init.d/S77-ntpd " . $serviceState, $retval);
                break;
            case 'serviceEIP':
                exec("/psft/scripts/icx35_eip.sh " . $serviceState .
                    "&> /dev/null", $retval);
                break;
            case 'serviceIPSec':
                exec("/psft/bin/oswanrc ". $serviceState, $retval);
                break;
            case 'serviceVPN':
                exec("/psft/bin/openvpnd " . $serviceState, $retval);
                break;
            case 'serviceSerencap':
                exec("/etc/init.d/S74-serencap " . $serviceState, $retval);
                break;
            case 'serviceDDNS':
                exec("/etc/init.d/S79-inadyn-mt " . $serviceState, $retval);
                break;
            case 'serviceCellUsage':
                exec("/etc/init.d/S80-usaged " . $serviceState, $retval);
                break;
        }

        return $retval;
    }

    function ipCalc($option, $ipAddress, $netmask) {
        /* "ipcalc" command template */
        $cmdTemplate = "/bin/busybox ipcalc -'%s' '%s'/'%s' | cut -d'=' -f2";

        $retval = safeExec($cmdTemplate, 0, $option, $ipAddress, $netmask);

        return $retval;
    }

    /*
     * Function:    pinHandler
     *
     * Summary:     Constructs and executes a safe call to pinHandler
     *
     * Parameters:  handlerAction - action to be performed by pinHandler
     *              args - variable list of arguments determined by handlerAction
     */
    function pinHandler($handlerAction, ...$args) {
        /* "pinHandler" command template */
        $cmdTemplate = "/psft/bin/pinHandler '%s'";

        /* Add specifier for each element in the "args" array */

        if (count($args) == 0) {
            $retval = safeExec($cmdTemplate, 0, $handlerAction, ...$args);
        } else {
            foreach ($args as $arg) {
                $cmdTemplate .= " '%s'";
            }

            $retval = safeExec($cmdTemplate, 1, $handlerAction, ...$args);
        }

        return $retval;
    }

    /*
     * Function:    pinCrypto
     *
     * Summary:     Constructs and executes a safe call to icx35-pass-enc.sh
     *
     * Parameters:  cryptoAction - action to be performed by icx35-pass-enc.sh
     *              simPin - string to be manipulated by icx35-pass-enc.sh
     */
    function pinCrypto($cryptoAction, $simPin) {
        /* PIN encryption command template */
        $cmdTemplate = "/psft/scripts/icx35-pass-enc.sh '%s' '%s'";

        $retval = safeExec($cmdTemplate, 0, $cryptoAction, $simPin);

        return $retval;
    }
?>
