var myApp = angular.module("LightDocs", []);

myApp.controller('CloudCtrl', function ($scope, $http, $interval)
{
    $scope.loggingBool = true; //always display errors, if any
    $scope.errorMessage = '';
    $scope.mutex = true;
    $scope.waitForDataLoad = true;

    $scope.connectionStatusString = 'Activate';

    $interval(function ()
    {
    	$scope.errorMessage = '';
        if (false == $scope.mutex) return;
        //$scope.connect controlls the logic for the buttton if reconecting or not
        try
        {

            $http({ method: 'GET', url: '/inc/cloud_connection_status.php' })
            .success(function (data)
            {
                if (false == $scope.mutex) return;
                //get he connection cloud status
                $scope.response = data;

                if ($scope.response.IsConnected.indexOf('true') != -1) {
                    $http({ method: 'GET', url: '/inc/cloud_connection_details.php' })
                     .success(function (data) {
                         //return data from Json file
                         $scope.settings = data;

                         if (data.ErrorMessage != '') {
                             //return the php error message in case exists
                             // $scope.errorMessage = data.ErrorMessage;
                             $scope.errorMessage = 'Error: ' + data.ErrorMessage + '. Check if gateway has internet access.';
                             return;
                         }

                         if ($scope.settings != null) {
                             $scope.isActivatingBool = false;
                             $scope.showActivationPanel = false;

                             if ($scope.settings.Activation.status == 'NOT_ACTIVATED') {
                                 if ($scope.settings.Activation.key == '')
                                 {
                                     $scope.settings.Activation.key = 'Waiting for activation key...';
                                 }
                                 $scope.isActivatingBool = true;
                                 $scope.showActivationPanel = true;
                             }
                         }
                     })
                      .error(function (response) {
                          // $scope.errorMessage = response;
                          $scope.errorMessage = 'Error: Cloud connection details not accessible.  Check if the gateway is online and try again.';
                      });
                }
                else {
                    // show the other panel (beware, there are two panels, they don't show both at the same time)
                    $scope.showActivationPanel = false;
                    $scope.isActivatingBool = false;

                    // reset the content in the shown panel
                    $scope.connectionStatusString = 'Activate';
                    $scope.isCloudManaged = false;
                }
            })
            .error(function (response)
            {
                // $scope.errorMessage = response;
                $scope.errorMessage = 'Error: Cloud connection status not accessible. Check if the gateway is online and try again.';
            })
            .finally(function(){
	            $scope.waitForDataLoad = false;
            });
        }
        catch (err)
        {
            $scope.errorMessage = 'Error: ' + err.message + '. Please refresh the browser window and try again.';
            $scope.waitForDataLoad = false;
        }

    }, 2000);

    $scope.connectToCloud = function ()
    {

    	$scope.errorMessage = '';
        try
        {
            $http({ method: 'GET', url: '/inc/cloud_connection_manager.php', params: { connection_status: !$scope.isCloudManaged } })
            .success(function (data)
            {
                if (data.ErrorMessage != '')
                {
                    $scope.isCloudManaged = false;
                    //return the php error message in case exists
	                $scope.errorMessage = 'Error: ' + data.ErrorMessage + '. Check if gateway has internet access and try again.';
                    return;
                }
                else
                {
                    $scope.isCloudManaged = !$scope.isCloudManaged;

                    if ($scope.isCloudManaged)
                    {
                        $scope.connectionStatusString = 'Deactivate';
                        $scope.settings.Activation.key = 'Waiting for activation key...';
                        $scope.showActivationPanel = true;
                        $scope.isActivatingBool = true;
                    }
                    else
                    {
                        $scope.connectionStatusString = 'Activate';
                    }
                }
            })
            .error(function (response)
            {
                // $scope.errorMessage = response;
                $scope.errorMessage = 'Error: Cloud connection manager not accessible. Check if the gateway is online and try again.';
                $scope.isCloudManaged = false;
            });
        }
        catch (err)
        {
            $scope.errorMessage = 'Error: ' + err.message + '. Please refresh the browser window and try again.';
            $scope.isCloudManaged = false;
        }
    };

    $scope.cancelActivation = function ()
    {	
    	$scope.errorMessage = '';
        $scope.mutex = false;
        try {
            $scope.showActivationPanel = false;
            $scope.isActivatingBool = false;
            $scope.connectionStatusString = 'Activate';
            $http({ method: 'GET', url: '/inc/cloud_connection_manager.php', params: { connection_status: false } })
            .success(function (data)
            {
                if (data.ErrorMessage != '')
                {
                    //return the php error message in case exists
                    $scope.errorMessage = 'Error: ' + data.ErrorMessage + '. Check if the gateway is online and try again.';
                    return;
                }
                else
                {
                    $scope.isCloudManaged = false;
                    $scope.showActivationPanel = false;
                    $scope.connectionStatusString = 'Activate';
                    $scope.isActivatingBool = false;
                }
                $scope.mutex = true;
            })
            .error(function (response)
            {
                // $scope.errorMessage = response;
                $scope.errorMessage = 'Error: Cloud connection manager not accessible. Check if the gateway is online and try again.';
                $scope.isCloudManaged = false;
                $scope.mutex = true;
            });
        }
        catch (err) {
            $scope.errorMessage = err.message;
            $scope.isCloudManaged = false;
            $scope.errorMessage = 'Error: ' + err.message + '. Please refresh the browser window and try again.';
        }
    };

    $scope.loadPageData = function ()
    {	
    	
    	$scope.errorMessage = '';

        $http({ method: 'GET', url: '/inc/cloud_connection_status.php' })
        .success(function (data)
        {
            //get he connection cloud status
            $scope.response = data;
            $scope.isActivatingBool = false;
            $scope.showActivationPanel = false;

            if ($scope.response.IsConnected.indexOf('true') != -1) {
                $scope.isCloudManaged = true;
                $scope.connectionStatusString = 'Deactivate';
            }
            else if ($scope.response.IsConnected.indexOf('false') != -1) {
                $scope.isCloudManaged = false;
                $scope.connectionStatusString = 'Activate';
            }

            //update the json information in the cloud managed case


            $http({ method: 'GET', url: '/inc/cloud_connection_details.php' })
            .success(function (data)
            {
                //return data from Json file
                $scope.settings = data;

                if (data.ErrorMessage != '') {
                    //return the php error message in case exists
                    $scope.errorMessage = 'Error: ' + data.ErrorMessage + '.';
                    return;
                }

                if (($scope.settings != null) && ($scope.isCloudManaged))
                {
                    //asuming the cloud it is connected
                    if ($scope.settings.Activation.status == 'ACTIVATED')
                    {
                        $scope.isActivatingBool = false;
                        $scope.connectionStatusString = 'Deactivate';
                    }
                    else if ($scope.settings.Activation.status == 'NOT_ACTIVATED')
                    {
                        if ($scope.settings.Activation.key == '')
                        {
                            $scope.settings.Activation.key = 'Waiting for activation key...';
                        }
                        $scope.isActivatingBool = true;
                        $scope.showActivationPanel = true;
                    }
                }
            })
            .error(function (response)
            {
                // $scope.errorMessage = response;
	            $scope.errorMessage = 'Error: Cloud connection details not available. Check if the gateway is online and try again.';
            })
            .finally(function(){
                $scope.waitForDataLoad = false;
            });
        })
        .error(function (response)
        {	
            // $scope.errorMessage = response;
            $scope.errorMessage = 'Error: Cloud connection status not accessible. Check if the gateway is online and try again.';
            $scope.waitForDataLoad = false;
        });
    }
});
