﻿var app = angular.module('FirewallApplication', ['ngGrid']);
app.controller('FirewallController', function ($scope, $http) {
    $scope.errorMessage = '';
    $scope.mySelections = [];

    /* Restricted ports array */
    $scope.portExValues = [];

    /* Pass Through enablement status */
    $scope.passThroughStatus = 0;

    /* Form port forwarding status dummy value */
    $scope.selectedPTStatus = { text: '' };

    /* End Device IP Address */
    $scope.endDeviceAddr = '';

    /* Port forwarding enablement status */
    $scope.portFwStatus = 0;

    /* Grid - port forwardin entries */
    $scope.portFwList = [];

    /* LAN port IP address */
    $scope.ipAddress = '192.168.1.1';

    /* LAN IP range end */
    $scope.lanBroadcast = '192.168.1.250';

    /* LAN IP range start */
    $scope.lanNetwork = '192.168.0.0';

    /* Form selected protocol */
    $scope.protocol = 0;

    /* Form data validity status */
    $scope.ptFormValid = true;

    /* Form data validity status */
    $scope.fwFormValid = true;

    /* Pass Through Form warning message */
    $scope.ptWarningMessage = '';

    /* Forwarding Form warning message */
    $scope.fwWarningMessage = '';

    /* Form procotol dummy value */
    $scope.selectedProtocol = { text: '' };

    /* Form port forwarding status dummy value */
    $scope.selectedPortFwStatus = { text: '' };

    /* Form rule name dummy value */
    $scope.application =  { text: 'Name' };

    /* Form inbound port range dummy value */
    $scope.fromPort = { textFromIn: '1',  textFromOut: '1' };

    /* Form outbound port range dummy value */
    $scope.toPort = { textToIn: '1',  textToOut: '1' };

    /* Checkbox cell template */
    var ct_nocheck = "<div class=\"ngSelectionCell\"><input  tabindex=\"-1\" type=\"checkbox\" ng-checked=\"row.selected\" /></div>";

    /* Grid template - populates port forwarding entries */
    $scope.gridOptions = {
        data: 'portFwList',
        columnDefs: [{ field: 'Application', displayName: 'Application' },
                     { field: 'Protocol', displayName: 'Protocol' },
                     { field: 'IpAddress', displayName: 'IP Address' },
                     {
                        field: 'BeginPort',
                        displayName: 'From Port Range',
                        cellTemplate: '<div class="ngCell">{{row.entity.FromBeginPort}}&nbsp; - &nbsp;{{row.entity.FromEndPort}}</div>'
                     },
                     {
                        field: 'BeginPort',
                        displayName: 'To Port Range',
                        cellTemplate: '<div class="ngCell">{{row.entity.ToBeginPort}}&nbsp; - &nbsp;{{row.entity.ToEndPort}}</div>'
                     },
                     {
                        field: 'ExceptionPorts',
                        displayName: 'Exception Ports',
                        cellTemplate: '<div class="ngCell">{{row.entity.ExceptionPorts}}</div>'
                     }],
        checkboxHeaderTemplate: '<input class="ngSelectionHeader" type="checkbox" ng-model="allSelected" ng-change="toggleSelectAll(allSelected)"/>Select',
        showSelectionCheckbox: true,
        selectWithCheckboxOnly: false,
        checkboxCellTemplate: ct_nocheck,
        selectedItems: $scope.mySelections,
        multiSelect: true
    };

    /**
     *
     * scope.loadData
     * send http request to retrieve data from config file
     *
     */
    $scope.loadData = function() {
        try {
            $http({method: 'GET', url: '/inc/forwarding_load_configurations.php'})
                .success(function (response) {
                    $scope.passThroughStatusList =
                        [{passThroughStatusId: "0", name: "Disable"},
                         {passThroughStatusId: "1", name: "Enable"}];
                    $scope.passThroughStatusList.defaultId = $scope.passThroughStatus.toString();
                    $scope.passThroughStatus = response.EnablePassThrough.toString();
                    $scope.endDeviceAddr = response.EndDeviceAddr;

                    /* Deconstruct JSON message */
                    $scope.portFwList = response.PortFwValues;
                    $scope.portExValues = response.PortExValues;
                    $scope.ipAddress = (String(response.LanIpAddr));
                    $scope.lanBroadcast = response.LanBroadcast;
                    $scope.lanNetwork = response.LanNetwork;
                    $scope.protocols =
                        [{protocolId: "0", name: "TCP"},
                         {protocolId: "1", name: "UDP"},
                         {protocolId: "2", name: "BOTH"}];
                    $scope.portFwStatusList =
                        [{portFwStatusId: "0", name: "Disable"},
                         {portFwStatusId: "1", name: "Enable"}];

                    angular.forEach($scope.portFwList, function(selectedEntry) {
                        angular.forEach($scope.portExValues, function(selectedPort) {
                            if (parseInt(selectedEntry.FromBeginPort) <= parseInt(selectedPort) &&
                                parseInt(selectedEntry.FromEndPort) >= parseInt(selectedPort))
                            {
                                if (selectedEntry.ExceptionPorts != '') {
                                    selectedEntry.ExceptionPorts += ', ';
                                }
                                selectedEntry.ExceptionPorts += selectedPort;
                            }
                        });
                    });
                    $scope.portFwStatusList.defaultId = $scope.portFwStatus.toString();
                    $scope.portFwStatus = response.EnablePortFw.toString();
                    $scope.loggingBool = true;
                    $scope.errorMessage = response.ErrorMessage;
                })
                .error(function(error) {
                    $scope.loggingBool = true;
                    $scope.errorMessage = error;
                });
        }
        catch (err) {
            $scope.loggingBool = true;
            $scope.errorMessage = err.message;
        }
    }

    /**
     *
     * scope.clearFields
     * Clear all input data from the current form
     *
     */
    $scope.clearFields = function() {
        try {
            $scope.application.text = '';
            $scope.fromPort.textFromIn = '0';
            $scope.fromPort.textFromOut = '0';
            $scope.ipAddress = '';
            $scope.toPort.textToIn = '0';
            $scope.toPort.textToOut = '0';
        }
        catch (err) {  
            alert(err.message);
        }
    }

    /**
     *
     * scope.deleteRows
     * Delete selected port forwarding entries from the grid
     *
     */
    $scope.deleteRows = function() {
        try {
            angular.forEach($scope.gridOptions.selectedItems, function (selectedObject) {    
                var deleteIndex = $scope.portFwList.indexOf(selectedObject);
                if (-1 < deleteIndex) {
                    $scope.portFwList.splice(deleteIndex, 1);
                }
            });
        }
        catch (err) {
            $scope.loggingBool = true;
            $scope.errorMessage = err.message;
        }
    }

    /**
     *
     * scope.saveData
     * Construct response JSON string and send http request to save to
     * configuration file
     *
     */
    $scope.saveData = function() {

        $scope.submitPTForm();
        if ($scope.fwFormValid && $scope.ptFormValid) {
            try {
                /* Send JSON message with port forwarding entries from grid */
                $http({ method: 'GET', url: '/inc/forwarding_save_configurations.php', params: { ptEnabled: $scope.passThroughStatusList.defaultId, ptAddress: $scope.endDeviceAddr, fwEnabled: $scope.portFwStatusList.defaultId, fwData: angular.toJson($scope.portFwList) } })
                .success(function (response) {
                    alert('Configuration saved! ');
                    $scope.loggingBool = true;
                    $scope.errorMessage = response.ErrorMessage;
                })
                .error(function (error) {
                    $scope.loggingBool = true;
                    $scope.errorMessage = error;
                });
            }
            catch (err) {
                $scope.loggingBool = true;
                $scope.errorMessage = err.message;
            }
        }
    }

    /**
     *
     * scope.submitFWForm
     * Validate form data and add new entry to grid
     *
     */
    $scope.submitFWForm = function() {
        try {
            /* Check to make sure the form is completely valid */
            if ($scope.fwForm.$valid) {
                $scope.fwFormValid = true;
                var selectedPortVal = '0';
                var exceptionPorts = '';

                /* Check that restricted ports are not forwarded */
                angular.forEach($scope.portExValues, function (iterator) {
                    if ((((parseInt(iterator)) >= (parseInt($scope.fromPort.textFromIn)))) &&
                         (((parseInt(iterator)) <= (parseInt($scope.fromPort.textFromOut)))))
                    {
                        if (exceptionPorts != '') {
                            exceptionPorts += ', ';
                        }
                        exceptionPorts += iterator;
                    }
                });

                if (!$scope.fwFormValid) {
                    $scope.fwWarningMessage = 'cannot foward restricted port ' + selectedPortVal + '. ';
                    return;
                }

                /* Verify that the same port range or part of it is not already forwarded */
                $scope.fwFormValid = true;
                angular.forEach($scope.portFwList, function (iterator) {
                    if ((((parseInt(iterator.FromBeginPort)) <= (parseInt($scope.fromPort.textFromIn)))) &&
                         (((parseInt(iterator.FromEndPort)) >= (parseInt($scope.fromPort.textFromIn)))))
                    {
                        $scope.fwFormValid = false;
                        return;
                    }

                    if ((((parseInt(iterator.FromBeginPort)) <= (parseInt($scope.fromPort.textFromOut)))) &&
                         (((parseInt(iterator.FromEndPort)) >= (parseInt($scope.fromPort.textFromOut)))))
                    {
                        $scope.fwFormValid = false;
                        return;
                    }
                });

                if (!$scope.fwFormValid) {
                    $scope.fwWarningMessage = 'overlapping port ranges. ';
                    return;
                }

                /* Verify that port forwarding entry name is not duplicate */
                $scope.fwFormValid = true;
                angular.forEach($scope.portFwList, function (iterator) {
                    if (iterator.Application == $scope.application.text) {
                        $scope.fwFormValid = false;
                        return;
                    }
                });
              
                if (!$scope.fwFormValid) {
                    $scope.fwWarningMessage = 'duplicate port forwarding entry application name. ';
                    return;
                }

                /* Check if number of port forwarding entries, 10, is reached */
                if ($scope.portFwList.length == 10) {
                    $scope.fwWarningMessage = 'maximum number of entries reached. ';
                    $scope.fwFormValid = false;
                    return;
                }

                /* Check if port values are valid  0 < port < 656536 */
                if ((parseInt($scope.fromPort.textFromIn) < 1) || 
                    (parseInt($scope.fromPort.textFromIn) > 65535))
                {
                    $scope.fwWarningMessage = 'From Port lower value out of bounds!. ';
                    $scope.fwFormValid = false;
                    return;
                }

                if ((parseInt($scope.fromPort.textFromOut) < 1) ||
                    (parseInt($scope.fromPort.textFromOut) > 65535))
                {
                    $scope.fwWarningMessage = 'From Port upper value out of bounds. ';
                    $scope.fwFormValid = false;
                    return;
                }

                if ((parseInt($scope.toPort.textToIn) < 1) ||
                    (parseInt($scope.toPort.textToIn) > 65535))
                {
                    $scope.fwWarningMessage = 'To Port lower value out of bounds. ';
                    $scope.fwFormValid = false;
                    return;
                }

                if ((parseInt($scope.toPort.textToOut) < 1) ||
                    (parseInt($scope.toPort.textToOut) > 65535))
                {
                    $scope.fwWarningMessage = 'To Port upper value out of bounds. ';
                    $scope.fwFormValid = false;
                    return;
                }

                /* Verify that the port range is valid, innerboud < outerbound */
                if ((parseInt($scope.fromPort.textFromOut)) < (parseInt($scope.fromPort.textFromIn))) {
                    $scope.fwWarningMessage = 'From Port invalid range. ';
                    $scope.fwFormValid = false;
                    return;
                }

                if ((parseInt($scope.toPort.textToOut)) < (parseInt($scope.toPort.textToIn))) {
                    $scope.fwWarningMessage = 'To Port invalid range. ';
                    $scope.fwFormValid = false;
                    return;
                }

                /**
                 * Convert IP address to int format and verify that the IP address
                 * correspond to the LAN IP/netmask
                 */ 
                if ((dot2num($scope.ipAddress) <= dot2num($scope.lanNetwork)) ||
                    (dot2num($scope.ipAddress) >= dot2num($scope.lanBroadcast)))
                {
                    $scope.fwWarningMessage = 'IP address out of bounds. ';
                    $scope.fwFormValid = false;
                    return;
                }

                /* Verify if a valid protocol was selected */
                if (null == $scope.selectedProtocol.name) {
                    $scope.fwWarningMessage = 'invalid protocol selected. ';
                    $scope.fwFormValid = false;
                    return;
                }

                $scope.fwForm.$valid = $scope.fwFormValid;
                $scope.portFwList.push({
                    Application: $scope.application.text,
                    Protocol: $scope.selectedProtocol.name,
                    IpAddress: $scope.ipAddress,
                    FromBeginPort: $scope.fromPort.textFromIn,
                    FromEndPort: $scope.fromPort.textFromOut,
                    ToBeginPort: $scope.toPort.textToIn,
                    ToEndPort: $scope.toPort.textToOut,
                    ExceptionPorts: exceptionPorts,
                });
            }
        }
        catch (err) {
            $scope.loggingBool = true;
            $scope.errorMessage = err.message;
        }
    }

    /**
     *
     * scope.submitPTForm
     * Validate form data and add new entry to grid
     *
     */
    $scope.submitPTForm = function() {
        try {
            /* Check to make sure the form is completely valid */
            if ($scope.ptForm.$valid) {
                $scope.ptFormValid = true;

                /**
                 * Convert IP address to int format and verify that the IP address
                 * correspond to the LAN IP/netmask
                 */ 
                if ((dot2num($scope.endDeviceAddr) <= dot2num($scope.lanNetwork)) ||
                    (dot2num($scope.endDeviceAddr) >= dot2num($scope.lanBroadcast)))
                {
                    $scope.ptWarningMessage = 'IP address out of bounds. ';
                    $scope.ptFormValid = false;
                    return;
                }

                $scope.ptForm.$valid = $scope.ptFormValid;
            }
        }
        catch (err) {
            $scope.loggingBool = true;
            $scope.errorMessage = err.message;
        }
    }

    /**
     *
     * scope.selectProtocol
     * Save in scope current protocol selected in the form
     *
     */
    $scope.selectProtocol = function() {
        $scope.selectedProtocol = $scope.protocols[$scope.protocol];
    }

    /**
     *
     * scope.selectPortFwStatus
     * Save in scope current Port Forwarding selected status in the form
     *
     */
    $scope.selectPortFwStatus = function() {
        $scope.selectedPortFwStatus = $scope.portFwStatusList[$scope.portFwStatus];
    }

    /**
     *
     * scope.selectPTStatus
     * Save in scope current Port Forwarding selected status in the form
     *
     */
    $scope.selectPTStatus = function() {
        $scope.selectedPTStatus = $scope.passThroughStatusList[$scope.passThroughStatus];
    }
});

/**
 *
 * dot2num
 * Utility function:
 * convert IP address format to int format
 *
 */
function dot2num(ipAddtessObj) {
    var ipNum = 0;

    try {
        var splitedIpaddr = (String(ipAddtessObj)).split('.');

        if (splitedIpaddr.length > 0) {
            ipNum = ((((((+splitedIpaddr[0]) * 256) +
                         (+splitedIpaddr[1])) * 256) +
                         (+splitedIpaddr[2])) * 256) +
                         (+splitedIpaddr[3]);
        }
    }
    catch (err) {
        alert(err.message);
    }

    return ipNum;
}
